@@ -38,19 +38,9 @@ static tree handle_warn_unused_result_attribute (tree *, tree, tree, int,
 						 bool *);
 static tree handle_sentinel_attribute (tree *, tree, tree, int, bool *);
 
-//static void check_function_nonnull (tree, tree);
-static void check_nonnull_arg (void *, tree, unsigned HOST_WIDE_INT);
-static bool nonnull_check_p (tree, unsigned HOST_WIDE_INT);
 static bool get_nonnull_operand (tree, unsigned HOST_WIDE_INT *);
-//static int resort_field_decl_cmp (const void *, const void *);
 
 /* extra for gdc copy: */
-static void check_function_arguments_recurse (void (*)
-					      (void *, tree,
-					       unsigned HOST_WIDE_INT),
-					      void *, tree,
-					      unsigned HOST_WIDE_INT);
-
 static tree
 handle_format_arg_attribute (tree *node ATTRIBUTE_UNUSED, tree name ATTRIBUTE_UNUSED,
 			     tree args ATTRIBUTE_UNUSED, int flags ATTRIBUTE_UNUSED, bool *no_add_attrs ATTRIBUTE_UNUSED)
@@ -132,8 +122,9 @@ const struct attribute_spec d_common_attribute_table[] =
   { "nothrow",                0, 0, true,  false, false,
 			      handle_nothrow_attribute },
   { "may_alias",	      0, 0, false, true, false, NULL },
-  /* not in gdc// { "cleanup",		      1, 1, true, false, false,
-     handle_cleanup_attribute },*/
+/* not in gdc
+  { "cleanup",		      1, 1, true, false, false,
+			      handle_cleanup_attribute },*/
   { "warn_unused_result",     0, 0, false, true, true,
 			      handle_warn_unused_result_attribute },
   { "sentinel",               0, 1, false, true, true,
@@ -1239,101 +1230,6 @@ handle_nonnull_attribute (tree *node, tree ARG_UNUSED (name),
   return NULL_TREE;
 }
 
-
-/* Check that the Nth argument of a function call (counting backwards
-   from the end) is a (pointer)0.  */
-
-static void
-check_function_sentinel (tree attrs, tree params)
-{
-  tree attr = lookup_attribute ("sentinel", attrs);
-
-  if (attr)
-    {
-      if (!params)
-	warning ("missing sentinel in function call");
-      else
-        {
-	  tree sentinel, end;
-	  unsigned pos = 0;
-	  
-	  if (TREE_VALUE (attr))
-	    {
-	      tree p = TREE_VALUE (TREE_VALUE (attr));
-	      STRIP_NOPS (p);
-	      pos = TREE_INT_CST_LOW (p);
-	    }
-
-	  sentinel = end = params;
-
-	  /* Advance `end' ahead of `sentinel' by `pos' positions.  */
-	  while (pos > 0 && TREE_CHAIN (end))
-	    {
-	      pos--;
-	      end = TREE_CHAIN (end);
-	    }
-	  if (pos > 0)
-	    {
-	      warning ("not enough arguments to fit a sentinel");
-	      return;
-	    }
-
-	  /* Now advance both until we find the last parameter.  */
-	  while (TREE_CHAIN (end))
-	    {
-	      end = TREE_CHAIN (end);
-	      sentinel = TREE_CHAIN (sentinel);
-	    }
-
-	  /* Validate the sentinel.  */
-	  if (!POINTER_TYPE_P (TREE_TYPE (TREE_VALUE (sentinel)))
-	      || !integer_zerop (TREE_VALUE (sentinel)))
-	    warning ("missing sentinel in function call");
-	}
-    }
-}
-
-/* Helper for check_function_nonnull; given a list of operands which
-   must be non-null in ARGS, determine if operand PARAM_NUM should be
-   checked.  */
-
-static bool
-nonnull_check_p (tree args, unsigned HOST_WIDE_INT param_num)
-{
-  unsigned HOST_WIDE_INT arg_num = 0;
-
-  for (; args; args = TREE_CHAIN (args))
-    {
-      bool found = get_nonnull_operand (TREE_VALUE (args), &arg_num);
-
-      gcc_assert (found);
-
-      if (arg_num == param_num)
-	return true;
-    }
-  return false;
-}
-
-/* Check that the function argument PARAM (which is operand number
-   PARAM_NUM) is non-null.  This is called by check_function_nonnull
-   via check_function_arguments_recurse.  */
-
-static void
-check_nonnull_arg (void * ARG_UNUSED (ctx), tree param,
-		   unsigned HOST_WIDE_INT param_num)
-{
-  /* Just skip checking the argument if it's not a pointer.  This can
-     happen if the "nonnull" attribute was given without an operand
-     list (which means to check every pointer argument).  */
-
-  if (TREE_CODE (TREE_TYPE (param)) != POINTER_TYPE)
-    return;
-
-  if (integer_zerop (param))
-    warning ("null argument where non-null required (argument %lu)",
-	     (unsigned long) param_num);
-}
-
 /* Helper for nonnull attribute handling; fetch the operand number
    from the attribute argument list.  */
 
@@ -1442,85 +1338,3 @@ handle_sentinel_attribute (tree *node, tree name, tree args,
 }
 
 
-/* Generic argument checking recursion routine.  PARAM is the argument to
-   be checked.  PARAM_NUM is the number of the argument.  CALLBACK is invoked
-   once the argument is resolved.  CTX is context for the callback.  */
-void
-check_function_arguments_recurse (void (*callback)
-				  (void *, tree, unsigned HOST_WIDE_INT),
-				  void *ctx, tree param,
-				  unsigned HOST_WIDE_INT param_num)
-{
-  if (TREE_CODE (param) == NOP_EXPR)
-    {
-      /* Strip coercion.  */
-      check_function_arguments_recurse (callback, ctx,
-					TREE_OPERAND (param, 0), param_num);
-      return;
-    }
-
-  if (TREE_CODE (param) == CALL_EXPR)
-    {
-      tree type = TREE_TYPE (TREE_TYPE (TREE_OPERAND (param, 0)));
-      tree attrs;
-      bool found_format_arg = false;
-
-      /* See if this is a call to a known internationalization function
-	 that modifies a format arg.  Such a function may have multiple
-	 format_arg attributes (for example, ngettext).  */
-
-      for (attrs = TYPE_ATTRIBUTES (type);
-	   attrs;
-	   attrs = TREE_CHAIN (attrs))
-	if (is_attribute_p ("format_arg", TREE_PURPOSE (attrs)))
-	  {
-	    tree inner_args;
-	    tree format_num_expr;
-	    int format_num;
-	    int i;
-
-	    /* Extract the argument number, which was previously checked
-	       to be valid.  */
-	    format_num_expr = TREE_VALUE (TREE_VALUE (attrs));
-	    while (TREE_CODE (format_num_expr) == NOP_EXPR
-		   || TREE_CODE (format_num_expr) == CONVERT_EXPR
-		   || TREE_CODE (format_num_expr) == NON_LVALUE_EXPR)
-	      format_num_expr = TREE_OPERAND (format_num_expr, 0);
-
-	    gcc_assert (TREE_CODE (format_num_expr) == INTEGER_CST
-			&& !TREE_INT_CST_HIGH (format_num_expr));
-
-	    format_num = TREE_INT_CST_LOW (format_num_expr);
-
-	    for (inner_args = TREE_OPERAND (param, 1), i = 1;
-		 inner_args != 0;
-		 inner_args = TREE_CHAIN (inner_args), i++)
-	      if (i == format_num)
-		{
-		  check_function_arguments_recurse (callback, ctx,
-						    TREE_VALUE (inner_args),
-						    param_num);
-		  found_format_arg = true;
-		  break;
-		}
-	  }
-
-      /* If we found a format_arg attribute and did a recursive check,
-	 we are done with checking this argument.  Otherwise, we continue
-	 and this will be considered a non-literal.  */
-      if (found_format_arg)
-	return;
-    }
-
-  if (TREE_CODE (param) == COND_EXPR)
-    {
-      /* Check both halves of the conditional expression.  */
-      check_function_arguments_recurse (callback, ctx,
-					TREE_OPERAND (param, 1), param_num);
-      check_function_arguments_recurse (callback, ctx,
-					TREE_OPERAND (param, 2), param_num);
-      return;
-    }
-
-  (*callback) (ctx, param, param_num);
-}
