@@ -1,6 +1,9 @@
 /* GDC -- D front-end for GCC
    Copyright (C) 2004 David Friedman
    
+   Modified by
+    Michael Parrott, Iain Buclaw, (C) 2010
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -44,8 +47,8 @@ d_bi_init(int nt, int nb)
 	error("cannot represent built in va_list type in D");
 	abort();
     }
-    if (! d_gcc_builtin_va_list_d_type->ctype)
-	d_gcc_builtin_va_list_d_type->toCtype();
+    // generate ctype if it doesn't already exist
+    d_gcc_builtin_va_list_d_type->toCtype();
 }
 
 /*
@@ -92,7 +95,7 @@ gcc_type_to_d_type(tree t)
     case INTEGER_TYPE:
 	{
 	    unsigned sz = tree_low_cst( TYPE_SIZE_UNIT( t ), 1 );
-	    bool unsgn = TREE_UNSIGNED( t );
+	    bool unsgn = TYPE_UNSIGNED( t );
 
 	    // This search assumes that integer types come before char and bit...
 	    for (int i = 0; i < (int) TMAX; i++) {
@@ -208,7 +211,11 @@ gcc_type_to_d_type(tree t)
 	    
 	    tree t_arg_types = TYPE_ARG_TYPES(t);
 	    int varargs = t_arg_types != NULL_TREE;
+#if V2 //Until 2.037
 	    Arguments * args = new Arguments;
+#else
+	    Parameters * args = new Parameters;
+#endif
 	    args->reserve(list_length(t_arg_types));
 	    for (tree tl = t_arg_types; tl != NULL_TREE; tl = TREE_CHAIN(tl))
 	    {
@@ -226,7 +233,11 @@ gcc_type_to_d_type(tree t)
 		    Type * d_arg_type = gcc_type_to_d_type(ta);
 		    if (! d_arg_type)
 			return NULL;
+#if V2 //Until 2.037
 		    args->push(new Argument(io, d_arg_type, NULL, NULL));
+#else
+		    args->push(new Parameter(io, d_arg_type, NULL, NULL));
+#endif
 		}
 		else
 		    varargs = 0;
@@ -295,8 +306,13 @@ d_gcc_magic_stdarg_module(Module *m, bool is_c_std_arg)
 			tf = (TypeFunction *) fd->type;
 			assert( tf->ty == Tfunction &&
 			    tf->parameters && tf->parameters->dim >= 1 );
+#if V2 //Until 2.037
 			((Argument*) tf->parameters->data[0])->storageClass &= ~(STCin|STCout|STCref);
 			((Argument*) tf->parameters->data[0])->storageClass |= STCin;
+#else
+			((Parameter*) tf->parameters->data[0])->storageClass &= ~(STCin|STCout|STCref);
+			((Parameter*) tf->parameters->data[0])->storageClass |= STCin;
+#endif
 		    }
 		}
 	    }
