@@ -1,6 +1,9 @@
 /* GDC -- D front-end for GCC
    Copyright (C) 2004 David Friedman
    
+   Modified by
+    Iain Buclaw, Michael Parrott, (C) 2010
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -412,8 +415,8 @@ IRState::convertTo(tree exp, Type * exp_type, Type * target_type)
 	    exp = d_convert_basic(d_type_for_size(POINTER_SIZE, 1), exp);
 	break;
     default:
-	if (exp_type->isreal() && target_type->isimaginary() ||
-	    exp_type->isimaginary() && target_type->isreal()) {
+	if (( exp_type->isreal() && target_type->isimaginary() )
+	    || ( exp_type->isimaginary() && target_type->isreal() )) {
 	    // warn? handle in front end?
 	    
 	    result = build_real_from_int_cst( target_type->toCtype(), integer_zero_node );
@@ -476,7 +479,11 @@ IRState::convertTo(tree exp, Type * exp_type, Type * target_type)
 }
 
 tree
+#if V2 //Until 2.037
 IRState::convertForArgument(Expression * exp, Argument * arg)
+#else
+IRState::convertForArgument(Expression * exp, Parameter * arg)
+#endif
 {
     if ( isArgumentReferenceType(arg) ) {
 	tree exp_tree = this->toElemLvalue(exp);
@@ -606,7 +613,8 @@ IRState::convertForCondition(tree exp_tree, Type * exp_type) {
 	a = delegateObjectRef(tmp);
 	b = delegateMethodRef(tmp);
 	if (TYPE_MODE(TREE_TYPE(a)) == TYPE_MODE(TREE_TYPE(b)))
-	    result = build2(BIT_IOR_EXPR, TREE_TYPE(a), a, b);
+	    result = build2(BIT_IOR_EXPR, TREE_TYPE(a), a,
+			    convert(TREE_TYPE(a), b));
 	else {
 	    a = d_truthvalue_conversion(a);
 	    b = d_truthvalue_conversion(b);
@@ -681,9 +689,9 @@ IRState::pointerIntSum(tree ptr_node, tree idx_exp)
     {
 	
 	if (TYPE_PRECISION (TREE_TYPE (intop)) != TYPE_PRECISION (sizetype)
-	    || TREE_UNSIGNED (TREE_TYPE (intop)) != TREE_UNSIGNED (sizetype))
+	    || TYPE_UNSIGNED (TREE_TYPE (intop)) != TYPE_UNSIGNED (sizetype))
 	    intop = convert (d_type_for_size (TYPE_PRECISION (sizetype), 
-				 TREE_UNSIGNED (sizetype)), intop);
+				 TYPE_UNSIGNED (sizetype)), intop);
 	intop = convert (prod_result_type,
 	    build2/*_binary_op*/ (MULT_EXPR, TREE_TYPE(size_exp), intop,  // the type here may be wrong %%
 		convert (TREE_TYPE (intop), size_exp)));
@@ -719,10 +727,8 @@ IRState::pointerOffsetOp(int op, tree ptr, tree idx)
     }
     return build2(POINTER_PLUS_EXPR, TREE_TYPE(ptr), ptr, 
 	convert(sizetype, idx));
-#elif D_GCC_VER >= 40
-    return build2((enum tree_code) op, TREE_TYPE(ptr), ptr, idx);
 #else
-    return build2(op, TREE_TYPE(ptr), ptr, idx);
+    return build2((enum tree_code) op, TREE_TYPE(ptr), ptr, idx);
 #endif    
 }
 
@@ -765,7 +771,7 @@ IRState::boundsCond(tree index, tree upper_bound, bool inclusive)
 	convert( d_unsigned_type(TREE_TYPE(index)), index ),
 	upper_bound);
     
-    if (! TREE_UNSIGNED( TREE_TYPE( index ))) {
+    if (! TYPE_UNSIGNED( TREE_TYPE( index ))) {
 	bound_check = build2(TRUTH_ANDIF_EXPR, boolean_type_node, bound_check,
 	    // %% conversions
 	    build2(GE_EXPR, boolean_type_node, index, integer_zero_node));
@@ -819,7 +825,6 @@ IRState::hwi2toli(HOST_WIDE_INT low, HOST_WIDE_INT high)
 
 
 #if D_GCC_VER >= 40
-int xxx;
 tree
 IRState::binding(tree var_chain, tree body)
 {
@@ -830,46 +835,6 @@ IRState::binding(tree var_chain, tree body)
     // %%EXPER -- if a BIND_EXPR is an a SAVE_EXPR, gimplify dies
     // in mostly_copy_tree_r.  prevent the latter from seeing
     // our shameful BIND_EXPR by wrapping it in a TARGET_EXPR
-    
-    /*
-    
-    tree bt = build3(BIND_EXPR, TREE_TYPE(body), var_chain, body, NULL_TREE);
-    tree tt = build_decl(VAR_DECL, NULL_TREE, TREE_TYPE(bt));
-    DECL_CONTEXT( tt ) = DECL_CONTEXT(var_chain); // cheat -> exprVar
-    DECL_ARTIFICIAL( tt ) = 1;
-    DECL_IGNORED_P( tt ) = 1;    
-    
-    return build4(TARGET_EXPR, TREE_TYPE(tt), tt, bt, NULL_TREE, NULL_TREE);
-    
-    */
-    /*
-    tree ini = DECL_INITIAL(var_chain);
-    DECL_INITIAL(var_chain) = NULL_TREE;
-    
-    return compound( build4(TARGET_EXPR, void_type_node, var_chain, ini,
-	NULL_TREE, NULL_TREE), body );
-    */
-    /*
-    tree ini = DECL_INITIAL(var_chain);
-    if (ini)
-    {
-	DECL_INITIAL(var_chain) = NULL_TREE;
-	
-	return compound( build4(TARGET_EXPR, void_type_node, var_chain, ini,
-	    NULL_TREE, NULL_TREE), body );
-    }
-    else
-    {
-	 ++xxx;
-	 
-	tree bt = build3(BIND_EXPR, TREE_TYPE(body), var_chain, body, NULL_TREE);
-	tree tt = build_decl(VAR_DECL, NULL_TREE, TREE_TYPE(bt));
-	DECL_CONTEXT( tt ) = DECL_CONTEXT(var_chain); // cheat -> exprVar
-	DECL_ARTIFICIAL( tt ) = 1;
-	DECL_IGNORED_P( tt ) = 1;    
-
-	return build4(TARGET_EXPR, TREE_TYPE(tt), tt, bt, NULL_TREE, NULL_TREE);
-    }*/
     if ( DECL_INITIAL(var_chain) )
     {
 	tree ini = build2(MODIFY_EXPR, void_type_node, var_chain, DECL_INITIAL(var_chain));
@@ -1045,8 +1010,13 @@ IRState::call(TypeFunction *func_type, tree callable, tree object, Array * argum
     if (object != NULL_TREE)
 	actual_arg_list.cons( object );
 
+#if V2 //Until 2.037
     Arguments * formal_args = func_type->parameters; // can be NULL for genCfunc decls
     size_t n_formal_args = formal_args ? (int) Argument::dim(formal_args) : 0;
+#else
+    Parameters * formal_args = func_type->parameters; // can be NULL for genCfunc decls
+    size_t n_formal_args = formal_args ? (int) Parameter::dim(formal_args) : 0;
+#endif
     size_t n_actual_args = arguments ? arguments->dim : 0;
     size_t fi = 0;
     
@@ -1060,8 +1030,11 @@ IRState::call(TypeFunction *func_type, tree callable, tree object, Array * argum
 	    actual_arg_tree = actual_arg_exp->toElem(this);
 	} else if (fi < n_formal_args) {
 	    // Actual arguments for declared formal arguments
+#if V2 //Until 2.037
 	    Argument * formal_arg = Argument::getNth(formal_args, fi);
-
+#else
+	    Parameter * formal_arg = Parameter::getNth(formal_args, fi);
+#endif
 	    actual_arg_tree = convertForArgument(actual_arg_exp, formal_arg);
 
 	    // from c-typeck.c: convert_arguments, default_conversion, ...
@@ -1201,11 +1174,10 @@ IRState::getLibCallDecl(LibCall lib_call)
 	    break;
 	case LIBCALL_NEWARRAYMTP:
 	case LIBCALL_NEWARRAYMITP:
-	    arg_types.push( Type::tsize_t );
-	    arg_types.push( Type::tint32 ); // Currently 'int', even if 64-bit
 	    arg_types.push( Type::typeinfo->type );
-	    if (lib_call == LIBCALL_NEWARRAYMITP)
-		arg_types.push( Type::tsize_t );
+	    arg_types.push( Type::tint32 ); // Currently 'int', even if 64-bit
+	    arg_types.push( Type::tsize_t );
+	    return_type = Type::tvoid->arrayOf();
 	    break;
 	case LIBCALL_ALLOCMEMORY:
 	    arg_types.push( Type::tsize_t );
@@ -1346,6 +1318,7 @@ IRState::getLibCallDecl(LibCall lib_call)
 	case LIBCALL_ARRAYCTOR:
 	    arg_types.push(Type::typeinfo->type);
 	    arg_types.push(Type::tvoid->arrayOf());
+	    arg_types.push(Type::tvoid->arrayOf());
 	    return_type = Type::tvoid->arrayOf();
 	    break;
 	case LIBCALL_ARRAYSETASSIGN:
@@ -1403,10 +1376,18 @@ IRState::getLibCallDecl(LibCall lib_call)
 	{
 	    TypeFunction * tf = (TypeFunction *) decl->type;
 	    tf->varargs = varargs ? 1 : 0;
+#if V2 //Until 2.037
 	    Arguments * args = new Arguments;
+#else
+	    Parameters * args = new Parameters;
+#endif
 	    args->setDim( arg_types.dim );
 	    for (unsigned i = 0; i < arg_types.dim; i++)
+#if V2 //Until 2.037
 		args->data[i] = new Argument( STCin, (Type *) arg_types.data[i],
+#else
+		args->data[i] = new Parameter( STCin, (Type *) arg_types.data[i],
+#endif
 		    NULL, NULL);
 	    tf->parameters = args;
 	}
@@ -1754,7 +1735,7 @@ IRState::darrayVal(tree type, uinteger_t len, tree data)
     len_value = integerConstant(len, TREE_TYPE(len_field));
 
     ce.cons(len_field, len_value);
-    ce.cons(ptr_field, ptr_value ); // shouldn't need to convert the pointer...
+    ce.cons(ptr_field, convert(TREE_TYPE(ptr_field), ptr_value));
     
     CONSTRUCTOR_ELTS( ctor ) = ce.head;
 
@@ -1944,10 +1925,18 @@ IRState::delegateVal(tree method_exp, tree object_exp, Type * d_type)
 	obj_field = TYPE_FIELDS( type );
 	func_field = TREE_CHAIN( obj_field );
     }
-    ce.cons(obj_field,  object_exp);
-    ce.cons(func_field, method_exp);
-    CONSTRUCTOR_ELTS( ctor ) = ce.head;
 
+    if (obj_field)
+	ce.cons(obj_field, convert (TREE_TYPE(obj_field), object_exp));
+    else
+	ce.cons(obj_field, object_exp);
+
+    if (func_field)
+	ce.cons(func_field, convert (TREE_TYPE(func_field), method_exp));
+    else
+	ce.cons(func_field, method_exp);
+
+    CONSTRUCTOR_ELTS( ctor ) = ce.head;
     return ctor;
 }
 
@@ -2053,7 +2042,14 @@ IRState::assignValue(Expression * e, VarDeclaration * v)
     {
 	AssignExp * a_exp = (AssignExp *) e;
 	if (a_exp->e1->op == TOKvar && ((VarExp *) a_exp->e1)->var == v)
-	    return convertForAssignment(a_exp->e2, v->type);
+	{
+	    tree a_val = convertForAssignment(a_exp->e2, v->type);
+	    // Look for reference initializations
+	    if (e->op == TOKconstruct && v->storage_class & (STCout | STCref))
+		return addressOf(a_val);
+	    else
+		return a_val;
+	}
 	//else
 	    //return e->toElem(this);
     }
@@ -2081,11 +2077,10 @@ IRState::twoFieldType(tree rec_type, tree ft1, tree ft2, Type * d_type, const ch
 	TYPE_STUB_DECL(rec_type) = stub_decl;
 	TYPE_NAME(rec_type) = stub_decl;
 	DECL_ARTIFICIAL(stub_decl) = 1;
-#if USE_MAPPED_LOCATION
+#if D_USE_MAPPED_LOCATION
 	DECL_SOURCE_LOCATION(stub_decl) = BUILTINS_LOCATION;
 	DECL_SOURCE_LOCATION(f0) = BUILTINS_LOCATION;
 	DECL_SOURCE_LOCATION(f1) = BUILTINS_LOCATION;
-// %% not sure if >= 4.3 and ! USE_MAPPED_LOCATION
 #endif
 	g.ofile->rodc(stub_decl, 0);
     }
@@ -2304,7 +2299,11 @@ IRState::trueDeclarationType(Declaration * decl)
 
 // These should match the Declaration versions above
 bool
+#if V2 //Until 2.037
 IRState::isArgumentReferenceType(Argument * arg)
+#else
+IRState::isArgumentReferenceType(Parameter * arg)
+#endif
 {
     Type * base_type = arg->type->toBasetype();
 
@@ -2320,7 +2319,11 @@ IRState::isArgumentReferenceType(Argument * arg)
 }
 
 tree
+#if V2 //Until 2.037
 IRState::trueArgumentType(Argument * arg)
+#else
+IRState::trueArgumentType(Parameter * arg)
+#endif
 {
     tree arg_type = arg->type->toCtype();
     if ( isArgumentReferenceType( arg )) {
@@ -2435,7 +2438,12 @@ IRState::attributes(Expressions * in_attrs)
 }
 
 tree
-IRState::integerConstant(xdmd_integer_t value, tree type) {
+IRState::integerConstant(xdmd_integer_t value, tree type)
+{
+    // The type is error_mark_node, we can't do anything.
+    if (type == error_mark_node) {
+	return type;
+    }
 #if D_GCC_VER < 40
     // Assuming xdmd_integer_t is 64 bits
 # if HOST_BITS_PER_WIDE_INT == 32
@@ -2454,21 +2462,12 @@ IRState::integerConstant(xdmd_integer_t value, tree type) {
     }
 #else
 # if HOST_BITS_PER_WIDE_INT == 32
-    tree tree_value = build_int_cst_wide(type,
-	value & 0xffffffff, (value >> 32) & 0xffffffff);
+    tree tree_value = build_int_cst_type(type, value & 0xffffffff);
 # elif HOST_BITS_PER_WIDE_INT == 64
     tree tree_value = build_int_cst_type(type, value);
 # else
 #  error Fix This
 # endif
-
-# if D_GCC_VER < 43
-    /* VALUE may be an incorrect representation for TYPE.  Example:
-         uint x = cast(uint) -3; // becomes "-3u" -- value=0xfffffffffffffd type=Tuns32
-       Constant folding will not work correctly unless this is done. */
-    tree_value = force_fit_type(tree_value, 0, 0, 0);
-# endif
-
 #endif
     return tree_value;
 }
@@ -3244,6 +3243,7 @@ IRState::doJump(Statement * stmt, tree t_label)
     if (stmt)
 	g.ofile->doLineNote( stmt->loc );
     addExp(build1(GOTO_EXPR, void_type_node, t_label));
+    TREE_USED(t_label) = 1;
 }
 
 tree
