@@ -1,5 +1,9 @@
 /* GDC -- D front-end for GCC
    Copyright (C) 2004 David Friedman
+
+   Modified by
+    Michael Parrot, (C) 2009, 2010
+    Iain Buclaw, (C) 2010
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -42,13 +46,6 @@
 #include "d-lang.h"
 #include "d-codegen.h"
 
-void slist_add(Symbol */*s*/)
-{
-}
-void slist_reset()
-{
-}
-
 /********************************* SymbolDeclaration ****************************/
 
 SymbolDeclaration::SymbolDeclaration(Loc loc, Symbol *s, StructDeclaration *dsym)
@@ -162,7 +159,19 @@ static void uniqueName(Declaration * d, tree t, const char * asm_name) {
 	sv->intvalue++;
     }
 
-    SET_DECL_ASSEMBLER_NAME(t, get_identifier(out_name));
+    tree id;
+#if D_GCC_VER >= 43
+    /* In 4.3.x, it is now the job of the front-end to ensure decls get mangled for their target.
+       We'll only allow FUNCTION_DECLs and VAR_DECLs for variables with static storage duration
+       to get a mangled DECL_ASSEMBLER_NAME. And the backend should handle the rest. */
+    if (f || (v && (v->protection == PROTpublic || v->storage_class & (STCstatic | STCextern)))) {
+	id = targetm.mangle_decl_assembler_name(t, get_identifier(out_name));
+    }
+    else
+#endif
+    id = get_identifier(out_name);
+
+    SET_DECL_ASSEMBLER_NAME(t, id);
 
     if (alloc_name)
 	free(alloc_name);
@@ -325,8 +334,8 @@ Symbol *VarDeclaration::toSymbol()
 	    // VarExp -- makeAddressOf could switch back to the VAR_DECL
 
 	    // if ( typs->isscalar() ) CONST_DECL...
-	    
 	    TREE_READONLY( var_decl ) = 1;
+
 	    // can at least do this...
 	    //  const doesn't seem to matter for aggregates, so prevent problems..
 	    if ( type->isscalar() ) {
@@ -354,7 +363,10 @@ Symbol *VarDeclaration::toSymbol()
 #ifdef TARGET_DLLIMPORT_DECL_ATTRIBUTES
 	// Have to test for import first
 	if (isImportedSymbol())
+	{
 	    gen.addDeclAttribute( var_decl, "dllimport" );
+	    DECL_DLLIMPORT_P( var_decl ) = 1;        
+	}
 	else if (isExport())
 	    gen.addDeclAttribute( var_decl, "dllexport" );
 #endif
@@ -567,7 +579,10 @@ Symbol *FuncDeclaration::toSymbol()
 #ifdef TARGET_DLLIMPORT_DECL_ATTRIBUTES
 	    // Have to test for import first
 	    if (isImportedSymbol())
+	    {
 		gen.addDeclAttribute( fn_decl, "dllimport" );
+		DECL_DLLIMPORT_P( fn_decl ) = 1;
+	    }
 	    else if (isExport())
 		gen.addDeclAttribute( fn_decl, "dllexport" );
 #endif
@@ -595,7 +610,7 @@ Symbol *FuncDeclaration::toSymbol()
 		    // %% hack: on darwin (at least) using a DECL_EXTERNAL (IRState::getLibCallDecl)
 		    // and TREE_STATIC FUNCTION_DECLs causes the stub label to be output twice.  This
 		    // is a work around.  This doesn't handle the case in which the normal
-		    // getLibCallDecl has already bee created and used.  Note that the problem only
+		    // getLibCallDecl has already been created and used.  Note that the problem only
 		    // occurs with function inlining is used.
 		    gen.replaceLibCallDecl(this);
 		    break;
@@ -758,8 +773,10 @@ Symbol *ClassDeclaration::toSymbol()
 	tree decl;
 	csym = toSymbolX("__Class", SCextern, 0, "Z");
 	slist_add(csym);
-	decl = build_decl( VAR_DECL, get_identifier( csym->Sident ),
-	    TREE_TYPE( ClassDeclaration::classinfo->type->toCtype() )); // want the RECORD_TYPE, not the REFERENCE_TYPE
+	decl = build_decl(VAR_DECL, get_identifier(csym->Sident),
+	    TREE_TYPE(ClassDeclaration::classinfo != NULL
+		? ClassDeclaration::classinfo->type->toCtype() // want the RECORD_TYPE, not the REFERENCE_TYPE
+		: error_mark_node));
 	csym->Stree = decl;
 	dkeep(decl);
  
@@ -909,8 +926,8 @@ Symbol *AggregateDeclaration::toInitializer()
 	// CONSTRUCTOR itself?
 
 	TREE_ADDRESSABLE( t ) = 1;
+	TREE_READONLY( t ) = 1;
 	TREE_CONSTANT( t ) = 1;
-	TREE_INVARIANT( t ) = 1;
 	DECL_CONTEXT( t ) = 0; // These are always global
     }
     return sinit;
@@ -994,11 +1011,7 @@ Symbol *Module::toModuleArray()
  *		1	add value signature
  */
 
-#if V1
 Symbol *TypeAArray::aaGetSymbol(const char *func, int flags)
-#else
-Symbol *TypeAArray::aaGetSymbol(char *func, int flags)
-#endif
 {
     // This is not used in GCC (yet?)
     return 0;
