@@ -1,5 +1,8 @@
 /* GDC -- D front-end for GCC
    Copyright (C) 2004 David Friedman
+
+   Modified by
+    Iain Buclaw, (C) 2010
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -82,6 +85,18 @@ extern "C" {
 #endif
 }
 
+// Define our own macro for handling mapped locations as
+// future versions of GCC (> 4.3) will poison it's use.
+#ifndef D_USE_MAPPED_LOCATION
+#if D_GCC_VER <= 43
+#  ifdef USE_MAPPED_LOCATION
+#    define D_USE_MAPPED_LOCATION 1
+#  endif
+#else
+#define D_USE_MAPPED_LOCATION 1
+#endif
+#endif
+
 // Undefine things that give us problems
 #undef RET
 
