@@ -1,5 +1,8 @@
 /* GDC -- D front-end for GCC
    Copyright (C) 2004 David Friedman
+
+   Modified by
+    Iain Buclaw, (C) 2010
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -335,7 +338,12 @@ IRBase::endFlow()
 void
 IRBase::doLabel(tree t_label)
 {
-    addExp(build1(LABEL_EXPR, void_type_node, t_label));
+    /* Don't write out label unless it is marked as used by the frontend.
+       This makes auto-vectorization possible in conditional loops.
+       The only excemption to this is in LabelStatement::toIR, in which
+       all computed labels are marked regardless.  */
+    if(TREE_USED(t_label))
+	addExp(build1(LABEL_EXPR, void_type_node, t_label));
 }
 
 #endif
