@@ -1,5 +1,9 @@
 /* GDC -- D front-end for GCC
    Copyright (C) 2004 David Friedman
+
+   Modified by
+    Michael Parrott, (C) 2009, 2010
+    Iain Buclaw, (C) 2010
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,6 +31,11 @@
 #include "cond.h"
 #include "mars.h"
 
+#if ! V2
+#include "async.h"
+#include "json.h"
+#endif
+
 #include <assert.h>
 #include "d-gcc-includes.h"
 #include "options.h"
@@ -142,10 +151,10 @@ static const char * fonly_arg;
 static const char * multilib_dir;
 
 static unsigned int
-d_init_options (unsigned int, const char **)
+d_init_options (unsigned int, const char ** argv)
 {
     // Set default values
-    global.params.argv0 = (char *) progname;
+    global.params.argv0 = (char *) argv[0]; //progname
     global.params.link = 1;
     global.params.useAssert = 1;
     global.params.useInvariants = 1;
@@ -278,7 +287,7 @@ d_cpp_forall_callback(cpp_reader *, cpp_hashnode * hn, void *)
 
 /* Supports CPP builtins. */
 cpp_reader * parse_in;
-void
+extern "C" void
 builtin_define_std (const char *macro)
 {
     // Do nothing.  No need for these (yet).
@@ -324,7 +333,7 @@ d_init ()
 #  endif
 #endif
 
-#ifdef USE_MAPPED_LOCATION
+#ifdef D_USE_MAPPED_LOCATION
     /* input_location is initially set to BUILTINS_LOCATION (2).  This 
        will cause a segfault if errors are reported before any line maps 
        are created. Setting input_location to zero stops those segfaults,
@@ -357,21 +366,21 @@ d_init ()
 #  endif
 #endif
     if (cpu_versym)
-	VersionCondition::addPredefinedGlobalIdent((char*) cpu_versym);
+	VersionCondition::addPredefinedGlobalIdent(cpu_versym);
 #ifdef D_OS_VERSYM
-    VersionCondition::addPredefinedGlobalIdent((char*) D_OS_VERSYM);
+    VersionCondition::addPredefinedGlobalIdent(D_OS_VERSYM);
     if (strcmp(D_OS_VERSYM, "darwin") == 0)
-    VersionCondition::addPredefinedGlobalIdent((char*) "OSX");
+    VersionCondition::addPredefinedGlobalIdent("OSX");
     if (strcmp(D_OS_VERSYM, "Win32") == 0)
 	 {
 		 VersionCondition::addPredefinedGlobalIdent("Windows");
 		 is_target_win32 = true;
 	 }
     if (strcmp(D_OS_VERSYM, "freebsd") == 0)
-    VersionCondition::addPredefinedGlobalIdent((char*) "FreeBSD");
+    VersionCondition::addPredefinedGlobalIdent("FreeBSD");
 #endif
 #ifdef D_OS_VERSYM2
-    VersionCondition::addPredefinedGlobalIdent((char*) D_OS_VERSYM2);
+    VersionCondition::addPredefinedGlobalIdent(D_OS_VERSYM2);
     if (strcmp(D_OS_VERSYM2, "Win32") == 0)
 	is_target_win32 = true;
 #endif
@@ -435,8 +444,6 @@ d_init ()
 # define TARGET_OBJFMT_CPP_BUILTINS()
 #endif
 
-# define preprocessing_asm_p() (cpp_get_options (pfile)->lang == CLK_ASM)
-# define preprocessing_trad_p() (cpp_get_options (pfile)->traditional)
 # define preprocessing_asm_p() (0)
 # define preprocessing_trad_p() (0)
 # define c_dialect_cxx() (0)
@@ -610,6 +617,14 @@ d_handle_option (size_t scode, const char *arg, int value)
       case OPT_fdebug_c:
 	  strcpy(lang_name, value ? "GNU C" : "GNU D");
 	  break;
+#if ! V2
+      case OPT_fdeps_:
+	  global.params.moduleDepsFile = (char*)arg;
+	  if (!global.params.moduleDepsFile[0])
+	      error("bad argument for -fdeps");
+	  global.params.moduleDeps = new OutBuffer;
+	  break;
+#endif
       case OPT_fignore_unknown_pragmas:
 	  global.params.ignoreUnsupportedPragmas = value;
 	  break;
@@ -689,12 +704,20 @@ d_handle_option (size_t scode, const char *arg, int value)
 	  // ignored
 	  break;
       case OPT_Wall:
-	  global.params.warnings = value;
+	  global.params.warnings = 2;
+	  gen.warnSignCompare = value;
+	  break;
+      case OPT_Werror:
+	  global.params.warnings = 1;
 	  gen.warnSignCompare = value;
 	  break;
       case OPT_Wsign_compare:
 	  gen.warnSignCompare = value;
 	  break;
+      case OPT_fXf_:
+	  global.params.doXGeneration = 1;
+	  global.params.xfilename = (char*)arg;
+	  break;
       default:
 	  break;
       }
@@ -748,7 +771,7 @@ d_write_global_declarations()
 #endif
     
 
-    for (int i = 0; i < globalFunctions.dim; i++)
+    for (unsigned i = 0; i < globalFunctions.dim; i++)
 	debug_hooks->global_decl(vec[i]);
 
 #if D_GCC_VER == 40
@@ -832,6 +855,11 @@ d_parse_file (int /*set_yydebug*/)
     char * p, * e;
     char * name;
     unsigned i;
+    
+    if (global.params.verbose)
+    {   printf("binary    %s\n", global.params.argv0);
+        printf("version   %s\n", global.version);
+    }
 
     if (global.params.verbose && asm_out_file == stdout)
     {
@@ -871,7 +899,7 @@ d_parse_file (int /*set_yydebug*/)
     Type::tdchar->toCtype();
     
     for (TY ty = (TY) 0; ty < TMAX; ty = (TY)(ty + 1)) {
-	if (Type::basic[ty])
+	if (Type::basic[ty] && ty != Terror)
 	    nametype(Type::basic[ty]);
     }
 
@@ -890,7 +918,10 @@ d_parse_file (int /*set_yydebug*/)
     an_output_module = NULL;
     Array modules; // vs. outmodules... = [an_output_module] or modules
     modules.reserve(num_in_fnames);
-    
+#if ! V2
+    AsyncRead *aw;
+#endif
+
     // %% FIX
     if ( ! main_input_filename ) {
 	::error("input file name required; cannot use stdin");
@@ -974,7 +1005,32 @@ d_parse_file (int /*set_yydebug*/)
 
     //global.params.verbose = 1;
     
-    // Read files, parse them
+    // Read files
+#if ! V2
+    aw = AsyncRead::create(modules.dim);
+    for (i = 0; i < modules.dim; i++)
+    {
+	m = (Module *)modules.data[i];
+	aw->addFile(m->srcfile);
+    }
+    aw->start();
+    for (i = 0; i < modules.dim; i++)
+    {
+	if (aw->read(i))
+	{
+	    error("cannot read file %s", m->srcfile->name->toChars());
+	    goto had_errors;
+	}
+    }
+    AsyncRead::dispose(aw);
+#else
+    for (i = 0; i < modules.dim; i++)
+    {
+	m = (Module *)modules.data[i];
+	m->read(0);
+    }
+#endif
+    // Parse files
     for (i = 0; i < modules.dim; i++)
     {
 	m = (Module *)modules.data[i];
@@ -984,7 +1040,6 @@ d_parse_file (int /*set_yydebug*/)
 	    Module::rootModule = m;
 	m->importedFrom = m;
 	//m->deleteObjFile(); // %% driver does this
-	m->read(0);
 	m->parse(global.params.dump_source);
 	d_gcc_magic_module(m);
 	if (m->isDocFile)
@@ -1020,7 +1075,20 @@ d_parse_file (int /*set_yydebug*/)
      if (global.errors)
 	 fatal();
 #endif
-	 
+
+#if ! V2
+    // load all unconditional imports for better symbol resolving
+    for (i = 0; i < modules.dim; i++)
+    {
+	m = (Module *)modules.data[i];
+	if (global.params.verbose)
+	    printf("importall %s\n", m->toChars());
+	m->importAll(0);
+    }
+    if (global.errors)
+	goto had_errors;
+#endif
+
     // Do semantic analysis
     for (i = 0; i < modules.dim; i++)
     {
@@ -1031,6 +1099,11 @@ d_parse_file (int /*set_yydebug*/)
     }
     if (global.errors)
 	goto had_errors;
+	
+#if V1
+    Module::dprogress = 1;
+    Module::runDeferredSemantic();
+#endif
 
     // Do pass 2 semantic analysis
     for (i = 0; i < modules.dim; i++)
@@ -1054,9 +1127,41 @@ d_parse_file (int /*set_yydebug*/)
     if (global.errors)
 	goto had_errors;
 
+#if ! V2
+    if (global.params.moduleDeps != NULL)
+    {
+	assert(global.params.moduleDepsFile != NULL);
+
+	File deps(global.params.moduleDepsFile);
+	OutBuffer* ob = global.params.moduleDeps;
+	deps.setbuffer((void*)ob->data, ob->offset);
+	deps.writev();
+    }
+#endif
+
     // Scan for functions to inline
     if (global.params.useInline)
     {
+	/* The problem with useArrayBounds and useAssert is that the
+	 * module being linked to may not have generated them, so if
+	 * we inline functions from those modules, the symbols for them will
+	 * not be found at link time.
+	 */
+	if (!global.params.useArrayBounds && !global.params.useAssert)
+	{
+	    // Do pass 3 semantic analysis on all imported modules,
+	    // since otherwise functions in them cannot be inlined
+	    for (i = 0; i < Module::amodules.dim; i++)
+	    {
+		m = (Module *)Module::amodules.data[i];
+		if (global.params.verbose)
+		    printf("semantic3 %s\n", m->toChars());
+		m->semantic3();
+	    }
+	    if (global.errors)
+		goto had_errors;
+	}
+
 	for (i = 0; i < modules.dim; i++)
 	{
 	    m = (Module *)modules.data[i];
@@ -1065,8 +1170,16 @@ d_parse_file (int /*set_yydebug*/)
 	    m->inlineScan();
 	}
     }
-    if (global.errors)
+    
+    // Do not attempt to generate output files if errors or warnings occurred
+    if (global.errors
+#if V1
+	|| global.warnings
+#endif
+       )
+    {
 	fatal();
+    }
 
     g.ofile = new ObjectFile();
     if (fonly_arg)
@@ -1076,6 +1189,12 @@ d_parse_file (int /*set_yydebug*/)
     g.irs = & gen; // needed for FuncDeclaration::toObjFile shouldDefer check
     
     // Generate output files
+    
+#if V1
+    if (global.params.doXGeneration)
+	json_generate(&modules);
+#endif
+
     for (i = 0; i < modules.dim; i++)
     {
 	m = (Module *)modules.data[i];
@@ -1442,7 +1561,7 @@ tree
 d_signed_or_unsigned_type (int unsignedp, tree type)
 {
   if (! INTEGRAL_TYPE_P (type)
-      || TREE_UNSIGNED (type) == (unsigned) unsignedp)
+      || TYPE_UNSIGNED (type) == (unsigned) unsignedp)
     return type;
 
   if (TYPE_PRECISION (type) == TYPE_PRECISION (signed_char_type_node))
@@ -1477,10 +1596,6 @@ d_signed_or_unsigned_type (int unsignedp, tree type)
   return type;
 }
 
-#if D_GCC_VER < 40
-#define TYPE_UNSIGNED TREE_UNSIGNED
-#endif
-
 /* Type promotion for variable arguments.  */
 tree
 d_type_promotes_to (tree type)
@@ -1748,21 +1863,31 @@ d_expand_function(tree fndecl)
 
 #if D_GCC_VER >= 40
 static int
-d_types_compatible_p (tree x, tree y)
+d_types_compatible_p (tree t1, tree t2)
 {
-    if (lhd_types_compatible_p(x, y))
+    tree d_va_list = NULL;
+
+    /* Is compatible if types are equivalent */
+    if (lhd_types_compatible_p (t1, t2))
 	return 1;
-    else if (d_gcc_builtin_va_list_d_type &&
-	(
-	 ( x == d_gcc_builtin_va_list_d_type->ctype &&
-	     y == va_list_type_node ) ||
-	 ( y == d_gcc_builtin_va_list_d_type->ctype &&
-	     x == va_list_type_node ) ))
+
+    if (d_gcc_builtin_va_list_d_type)
+	d_va_list = d_gcc_builtin_va_list_d_type->ctype;
+
+    /* Is compatible if we are dealing with C <-> D va_list nodes */
+    if ((t1 == d_va_list && t2 == va_list_type_node)
+	|| (t2 == d_va_list && t1 == va_list_type_node))
 	return 1;
-    else
-	return 0;
-	
-    return TYPE_MAIN_VARIANT (x) == TYPE_MAIN_VARIANT (y);
+
+    /* Is compatible if aggregates are same type and share the same
+       attributes. The frontend should have already ensured that types
+       aren't wildly different anyway... */
+    if (AGGREGATE_TYPE_P (t1) && AGGREGATE_TYPE_P (t2)
+	&& TREE_CODE (t1) == TREE_CODE (t2)
+	&& TYPE_ATTRIBUTES (t1) == TYPE_ATTRIBUTES (t2))
+	return 1;
+
+    return 0;
 }
 
 #if V2
