@@ -1,3 +1,25 @@
+/* GDC -- D front-end for GCC
+   Copyright (C) 2004 David Friedman
+
+   Modified by
+    Michael Parrott, (C) 2009
+    Iain Buclaw, (C) 2010
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+ 
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+ 
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
+
 #include "d-gcc-includes.h"
 #include "d-lang.h"
 #include "d-codegen.h"
@@ -109,7 +131,7 @@ ObjectFile::doLineNote(const Loc & loc)
     }
 }
 
-#ifdef USE_MAPPED_LOCATION
+#ifdef D_USE_MAPPED_LOCATION
 static location_t
 cvtLocToloc_t(const Loc loc)
 {
@@ -148,7 +170,7 @@ void
 ObjectFile::setLoc(const Loc & loc)
 {
     if (loc.filename) {
-#ifdef USE_MAPPED_LOCATION
+#ifdef D_USE_MAPPED_LOCATION
 	input_location = cvtLocToloc_t(loc);
 #else
 	location_t gcc_loc = { loc.filename, loc.linnum };
@@ -165,7 +187,7 @@ ObjectFile::setDeclLoc(tree t, const Loc & loc)
     // DWARF2 will often crash if the DECL_SOURCE_FILE is not set.  It's
     // easier the error here.
     assert(loc.filename);
-#ifdef USE_MAPPED_LOCATION
+#ifdef D_USE_MAPPED_LOCATION
     DECL_SOURCE_LOCATION (t) = cvtLocToloc_t(loc);
 #else
     DECL_SOURCE_FILE (t) = loc.filename;
@@ -204,7 +226,7 @@ void
 ObjectFile::setCfunEndLoc(const Loc & loc)
 {
     tree fn_decl = cfun->decl;
-# ifdef USE_MAPPED_LOCATION
+# ifdef D_USE_MAPPED_LOCATION
     if (loc.filename)
 	cfun->function_end_locus = cvtLocToloc_t(loc);
     else
@@ -763,7 +785,7 @@ ObjectFile::outputThunk(tree thunk_decl, tree target_decl, target_ptrdiff_t offs
 	
 	fnname = XSTR(XEXP(DECL_RTL(thunk_decl), 0), 0);
 	gen.initFunctionStart(thunk_decl, 0);
-	current_function_is_thunk = 1;
+	cfun->is_thunk = 1;
 	assemble_start_function (thunk_decl, fnname);
 	targetm.asm_out.output_mi_thunk (asm_out_file, thunk_decl,
 	    delta, 0, alias);
@@ -1067,6 +1089,7 @@ dt2tree_list_of_elems(dt_t * dt)
 	DECL_CONTEXT(field) = aggtype;
 	DECL_FIELD_OFFSET(field) = offset;
 	DECL_FIELD_BIT_OFFSET(field) = bitsize_zero_node;
+	SET_DECL_OFFSET_ALIGN(field, TYPE_ALIGN( TREE_TYPE( field )));
 	DECL_ARTIFICIAL(field) = 1;
 	DECL_IGNORED_P(field) = 1;
 	layout_decl(field, 0);
