@@ -31,14 +31,17 @@ struct HdrGenState;
 
 struct AttribDeclaration : Dsymbol
 {
-    Array *decl;	// array of Dsymbol's
+    Array *decl;        // array of Dsymbol's
 
     AttribDeclaration(Array *decl);
     virtual Array *include(Scope *sc, ScopeDsymbol *s);
     int addMember(Scope *sc, ScopeDsymbol *s, int memnum);
+    void setScopeNewSc(Scope *sc,
+        StorageClass newstc, enum LINK linkage, enum PROT protection, int explictProtection,
+        unsigned structalign);
     void semanticNewSc(Scope *sc,
- 	 unsigned newstc, enum LINK linkage, enum PROT protection, int explictProtection,
- 	 	unsigned structalign);
+        StorageClass newstc, enum LINK linkage, enum PROT protection, int explictProtection,
+        unsigned structalign);
     void semantic(Scope *sc);
     void semantic2(Scope *sc);
     void semantic3(Scope *sc);
@@ -51,22 +54,24 @@ struct AttribDeclaration : Dsymbol
     void checkCtorConstInit();
     void addLocalClass(ClassDeclarations *);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+    void toJsonBuffer(OutBuffer *buf);
     AttribDeclaration *isAttribDeclaration() { return this; }
 
-    void toObjFile(int multiobj);			// compile to .obj file
+    void toObjFile(int multiobj);                       // compile to .obj file
     int cvMember(unsigned char *p);
 };
 
 struct StorageClassDeclaration: AttribDeclaration
 {
-    unsigned stc;
+    StorageClass stc;
 
-    StorageClassDeclaration(unsigned stc, Array *decl);
+    StorageClassDeclaration(StorageClass stc, Array *decl);
     Dsymbol *syntaxCopy(Dsymbol *s);
+    void setScope(Scope *sc);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-    
-    static void stcToCBuffer(OutBuffer *buf, int stc);
+
+    static void stcToCBuffer(OutBuffer *buf, StorageClass stc);
 };
 
 struct LinkDeclaration : AttribDeclaration
@@ -75,6 +80,7 @@ struct LinkDeclaration : AttribDeclaration
 
     LinkDeclaration(enum LINK p, Array *decl);
     Dsymbol *syntaxCopy(Dsymbol *s);
+    void setScope(Scope *sc);
     void semantic(Scope *sc);
     void semantic3(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -87,9 +93,11 @@ struct ProtDeclaration : AttribDeclaration
 
     ProtDeclaration(enum PROT p, Array *decl);
     Dsymbol *syntaxCopy(Dsymbol *s);
+    void importAll(Scope *sc);
+    void setScope(Scope *sc);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-    
+
     static void protectionToCBuffer(OutBuffer *buf, enum PROT protection);
 };
 
@@ -99,6 +107,7 @@ struct AlignDeclaration : AttribDeclaration
 
     AlignDeclaration(unsigned sa, Array *decl);
     Dsymbol *syntaxCopy(Dsymbol *s);
+    void setScope(Scope *sc);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 };
@@ -106,7 +115,7 @@ struct AlignDeclaration : AttribDeclaration
 struct AnonDeclaration : AttribDeclaration
 {
     int isunion;
-    int sem;			// 1 if successful semantic()
+    int sem;                    // 1 if successful semantic()
 
     AnonDeclaration(Loc loc, int isunion, Array *decl);
     Dsymbol *syntaxCopy(Dsymbol *s);
@@ -117,21 +126,22 @@ struct AnonDeclaration : AttribDeclaration
 
 struct PragmaDeclaration : AttribDeclaration
 {
-    Expressions *args;		// array of Expression's
+    Expressions *args;          // array of Expression's
 
     PragmaDeclaration(Loc loc, Identifier *ident, Expressions *args, Array *decl);
     Dsymbol *syntaxCopy(Dsymbol *s);
     void semantic(Scope *sc);
+    void setScope(Scope *sc);
     int oneMember(Dsymbol **ps);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     const char *kind();
-    void toObjFile(int multiobj);			// compile to .obj file
+    void toObjFile(int multiobj);                       // compile to .obj file
 };
 
 struct ConditionalDeclaration : AttribDeclaration
 {
     Condition *condition;
-    Array *elsedecl;	// array of Dsymbol's for else block
+    Array *elsedecl;    // array of Dsymbol's for else block
 
     ConditionalDeclaration(Condition *condition, Array *decl, Array *elsedecl);
     Dsymbol *syntaxCopy(Dsymbol *s);
@@ -140,6 +150,9 @@ struct ConditionalDeclaration : AttribDeclaration
     Array *include(Scope *sc, ScopeDsymbol *s);
     void addComment(unsigned char *comment);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+    void toJsonBuffer(OutBuffer *buf);
+    void importAll(Scope *sc);
+    void setScope(Scope *sc);
 };
 
 struct StaticIfDeclaration : ConditionalDeclaration
@@ -151,6 +164,8 @@ struct StaticIfDeclaration : ConditionalDeclaration
     Dsymbol *syntaxCopy(Dsymbol *s);
     int addMember(Scope *sc, ScopeDsymbol *s, int memnum);
     void semantic(Scope *sc);
+    void importAll(Scope *sc);
+    void setScope(Scope *sc);
     const char *kind();
 };
 
