@@ -1,7 +1,6 @@
 
-
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2009 by Digital Mars
+// Copyright (c) 1999-2010 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -49,67 +48,73 @@ enum MATCH;
 enum STC
 {
     STCundefined    = 0,
-    STCstatic	    = 1,
-    STCextern	    = 2,
-    STCconst	    = 4,
-    STCfinal	    = 8,
+    STCstatic       = 1,
+    STCextern       = 2,
+    STCconst        = 4,
+    STCfinal        = 8,
     STCabstract     = 0x10,
     STCparameter    = 0x20,
-    STCfield	    = 0x40,
-    STCoverride	    = 0x80,
+    STCfield        = 0x40,
+    STCoverride     = 0x80,
     STCauto         = 0x100,
     STCsynchronized = 0x200,
     STCdeprecated   = 0x400,
-    STCin           = 0x800,		// in parameter
-    STCout          = 0x1000,		// out parameter
-    STClazy	    = 0x2000,		// lazy parameter
-    STCforeach      = 0x4000,		// variable for foreach loop
-    STCcomdat       = 0x8000,		// should go into COMDAT record
-    STCvariadic     = 0x10000,		// variadic function argument
-    STCctorinit     = 0x20000,		// can only be set inside constructor
-    STCtemplateparameter = 0x40000,	// template parameter
-    STCscope	    = 0x80000,		// template parameter
-    STCinvariant    = 0x100000,
+    STCin           = 0x800,            // in parameter
+    STCout          = 0x1000,           // out parameter
+    STClazy         = 0x2000,           // lazy parameter
+    STCforeach      = 0x4000,           // variable for foreach loop
+    STCcomdat       = 0x8000,           // should go into COMDAT record
+    STCvariadic     = 0x10000,          // variadic function argument
+    STCctorinit     = 0x20000,          // can only be set inside constructor
+    STCtemplateparameter = 0x40000,     // template parameter
+    STCscope        = 0x80000,          // template parameter
     STCimmutable    = 0x100000,
-    STCref	    = 0x200000,
-    STCinit	    = 0x400000,		// has explicit initializer
-    STCmanifest	    = 0x800000,		// manifest constant
-    STCnodtor	    = 0x1000000,	// don't run destructor
-    STCnothrow	    = 0x2000000,	// never throws exceptions
-    STCpure	    = 0x4000000,	// pure function
-    STCtls	    = 0x8000000,	// thread local
-    STCalias	    = 0x10000000,	// alias parameter
-    STCshared       = 0x20000000,	// accessible from multiple threads
-    STCgshared      = 0x40000000,	// accessible from multiple threads
-					// but not typed as "shared"
-    STC_TYPECTOR    = (STCconst | STCimmutable | STCshared),
+    STCref          = 0x200000,
+    STCinit         = 0x400000,         // has explicit initializer
+    STCmanifest     = 0x800000,         // manifest constant
+    STCnodtor       = 0x1000000,        // don't run destructor
+    STCnothrow      = 0x2000000,        // never throws exceptions
+    STCpure         = 0x4000000,        // pure function
+    STCtls          = 0x8000000,        // thread local
+    STCalias        = 0x10000000,       // alias parameter
+    STCshared       = 0x20000000,       // accessible from multiple threads
+    STCgshared      = 0x40000000,       // accessible from multiple threads
+                                        // but not typed as "shared"
+    STCwild         = 0x80000000,       // for "wild" type constructor
+    STC_TYPECTOR    = (STCconst | STCimmutable | STCshared | STCwild),
 };
 
+#define STCproperty     0x100000000LL
+#define STCsafe         0x200000000LL
+#define STCtrusted      0x400000000LL
+#define STCsystem       0x800000000LL
+#define STCctfe         0x1000000000LL  // can be used in CTFE, even if it is static
+
 struct Match
 {
-    int count;			// number of matches found
-    MATCH last;			// match level of lastf
-    FuncDeclaration *lastf;	// last matching function we found
-    FuncDeclaration *nextf;	// current matching function
-    FuncDeclaration *anyf;	// pick a func, any func, to use for error recovery
+    int count;                  // number of matches found
+    MATCH last;                 // match level of lastf
+    FuncDeclaration *lastf;     // last matching function we found
+    FuncDeclaration *nextf;     // current matching function
+    FuncDeclaration *anyf;      // pick a func, any func, to use for error recovery
 };
 
 void overloadResolveX(Match *m, FuncDeclaration *f,
- 	Expression *ethis, Expressions *arguments);
+        Expression *ethis, Expressions *arguments);
 int overloadApply(FuncDeclaration *fstart,
-	int (*fp)(void *, FuncDeclaration *),
-	void *param);
+        int (*fp)(void *, FuncDeclaration *),
+        void *param);
 
 /**************************************************************/
 
 struct Declaration : Dsymbol
 {
     Type *type;
-    Type *originalType;		// before semantic analysis
-    unsigned storage_class;
+    Type *originalType;         // before semantic analysis
+    StorageClass storage_class;
     enum PROT protection;
     enum LINK linkage;
-    int inuse;			// used to detect cycles
+    int inuse;                  // used to detect cycles
     Expressions * attributes; // GCC decl/type attributes
 
     Declaration(Identifier *id);
@@ -119,6 +124,7 @@ struct Declaration : Dsymbol
     void checkModify(Loc loc, Scope *sc, Type *t);
 
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
     void toDocBuffer(OutBuffer *buf);
 
     char *mangle();
@@ -133,7 +139,7 @@ struct Declaration : Dsymbol
     int isFinal()        { return storage_class & STCfinal; }
     int isAbstract()     { return storage_class & STCabstract; }
     int isConst()        { return storage_class & STCconst; }
-    int isInvariant()    { return 0; }
+    int isImmutable()    { return storage_class & STCimmutable; }
     int isAuto()         { return storage_class & STCauto; }
     int isScope()        { return storage_class & (STCscope | STCauto); }
     int isSynchronized() { return storage_class & STCsynchronized; }
@@ -155,9 +161,9 @@ struct Declaration : Dsymbol
 struct TupleDeclaration : Declaration
 {
     Objects *objects;
-    int isexp;			// 1: expression tuple
+    int isexp;                  // 1: expression tuple
 
-    TypeTuple *tupletype;	// !=NULL if this is a type tuple
+    TypeTuple *tupletype;       // !=NULL if this is a type tuple
 
     TupleDeclaration(Loc loc, Identifier *ident, Objects *objects);
     Dsymbol *syntaxCopy(Dsymbol *);
@@ -174,10 +180,10 @@ struct TypedefDeclaration : Declaration
 {
     Type *basetype;
     Initializer *init;
-    int sem;			// 0: semantic() has not been run
-				// 1: semantic() is in progress
-				// 2: semantic() has been run
-				// 3: semantic2() has been run
+    int sem;                    // 0: semantic() has not been run
+                                // 1: semantic() is in progress
+                                // 2: semantic() has been run
+                                // 3: semantic2() has been run
 
     TypedefDeclaration(Loc loc, Identifier *ident, Type *basetype, Initializer *init);
     Dsymbol *syntaxCopy(Dsymbol *);
@@ -194,7 +200,7 @@ struct TypedefDeclaration : Declaration
 
     void toDocBuffer(OutBuffer *buf);
 
-    void toObjFile(int multiobj);			// compile to .obj file
+    void toObjFile(int multiobj);                       // compile to .obj file
     void toDebug();
     int cvMember(unsigned char *p);
 
@@ -209,7 +215,7 @@ struct TypedefDeclaration : Declaration
 struct AliasDeclaration : Declaration
 {
     Dsymbol *aliassym;
-    Dsymbol *overnext;		// next in overload list
+    Dsymbol *overnext;          // next in overload list
     int inSemantic;
 
     AliasDeclaration(Loc loc, Identifier *ident, Type *type);
@@ -237,15 +243,25 @@ struct VarDeclaration : Declaration
 {
     Initializer *init;
     target_size_t offset;
-    int noauto;			// no auto semantics
-    int nestedref;		// referenced by a lexically nested function
-    int ctorinit;		// it has been initialized in a ctor
-    int onstack;		// 1: it has been allocated on the stack
-				// 2: on stack, run destructor anyway
-    int canassign;		// it can be assigned to
-    Dsymbol *aliassym;		// if redone as alias to another symbol
-    Expression *value;		// when interpreting, this is the value
-				// (NULL if value not determinable)
+    int noauto;                 // no auto semantics
+#if DMDV2
+    FuncDeclarations nestedrefs; // referenced by these lexically nested functions
+    bool isargptr;              // if parameter that _argptr points to
+#else
+    int nestedref;              // referenced by a lexically nested function
+#endif
+    int ctorinit;               // it has been initialized in a ctor
+    int onstack;                // 1: it has been allocated on the stack
+                                // 2: on stack, run destructor anyway
+    int canassign;              // it can be assigned to
+    Dsymbol *aliassym;          // if redone as alias to another symbol
+    Expression *value;          // when interpreting, this is the value
+                                // (NULL if value not determinable)
+#if DMDV2
+    VarDeclaration *rundtor;    // if !NULL, rundtor is tested at runtime to see
+                                // if the destructor should be run. Used to prevent
+                                // dtor calls on postblitted vars
+#endif
 
     VarDeclaration(Loc loc, Type *t, Identifier *id, Initializer *init);
     Dsymbol *syntaxCopy(Dsymbol *);
@@ -260,15 +276,22 @@ struct VarDeclaration : Declaration
     int needThis();
     int isImportedSymbol();
     int isDataseg();
+    int isThreadlocal();
+    int isCTFE();
     int hasPointers();
-    Expression *callAutoDtor();
+#if DMDV2
+    int canTakeAddressOf();
+    int needsAutoDtor();
+#endif
+    Expression *callAutoDtor(Scope *sc);
     ExpInitializer *getExpInitializer();
+    Expression *getConstInitializer();
     void checkCtorConstInit();
     void checkNestedReference(Scope *sc, Loc loc);
     Dsymbol *toAlias();
 
     Symbol *toSymbol();
-    void toObjFile(int multiobj);			// compile to .obj file
+    void toObjFile(int multiobj);                       // compile to .obj file
     int cvMember(unsigned char *p);
 
     // Eliminate need for dynamic_cast
@@ -301,6 +324,7 @@ struct ClassInfoDeclaration : VarDeclaration
     void semantic(Scope *sc);
 
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
 
     Symbol *toSymbol();
 };
@@ -314,6 +338,7 @@ struct ModuleInfoDeclaration : VarDeclaration
     void semantic(Scope *sc);
 
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
 
     Symbol *toSymbol();
 };
@@ -327,9 +352,10 @@ struct TypeInfoDeclaration : VarDeclaration
     void semantic(Scope *sc);
 
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
 
     Symbol *toSymbol();
-    void toObjFile(int multiobj);			// compile to .obj file
+    void toObjFile(int multiobj);                       // compile to .obj file
     virtual void toDt(dt_t **pdt);
 };
 
@@ -451,9 +477,9 @@ struct ThisDeclaration : VarDeclaration
 
 enum ILS
 {
-    ILSuninitialized,	// not computed yet
-    ILSno,		// cannot inline
-    ILSyes,		// can inline
+    ILSuninitialized,   // not computed yet
+    ILSno,              // cannot inline
+    ILSyes,             // can inline
 };
 
 /**************************************************************/
@@ -461,13 +487,13 @@ enum ILS
 
 enum BUILTIN
 {
-    BUILTINunknown = -1,	// not known if this is a builtin
-    BUILTINnot,			// this is not a builtin
-    BUILTINsin,			// std.math.sin
-    BUILTINcos,			// std.math.cos
-    BUILTINtan,			// std.math.tan
-    BUILTINsqrt,		// std.math.sqrt
-    BUILTINfabs,		// std.math.fabs
+    BUILTINunknown = -1,        // not known if this is a builtin
+    BUILTINnot,                 // this is not a builtin
+    BUILTINsin,                 // std.math.sin
+    BUILTINcos,                 // std.math.cos
+    BUILTINtan,                 // std.math.tan
+    BUILTINsqrt,                // std.math.sqrt
+    BUILTINfabs,                // std.math.fabs
 };
 
 Expression *eval_builtin(enum BUILTIN builtin, Expressions *arguments);
@@ -478,72 +504,76 @@ enum BUILTIN { };
 
 struct FuncDeclaration : Declaration
 {
-    Array *fthrows;			// Array of Type's of exceptions (not used)
+    Array *fthrows;                     // Array of Type's of exceptions (not used)
     Statement *frequire;
     Statement *fensure;
     Statement *fbody;
 
-    Identifier *outId;			// identifier for out statement
-    VarDeclaration *vresult;		// variable corresponding to outId
-    LabelDsymbol *returnLabel;		// where the return goes
+    FuncDeclarations foverrides;        // functions this function overrides
+    FuncDeclaration *fdrequire;         // function that does the in contract
+    FuncDeclaration *fdensure;          // function that does the out contract
 
-    DsymbolTable *localsymtab;		// used to prevent symbols in different
-					// scopes from having the same name
-    VarDeclaration *vthis;		// 'this' parameter (member and nested)
-    VarDeclaration *v_arguments;	// '_arguments' parameter
+    Identifier *outId;                  // identifier for out statement
+    VarDeclaration *vresult;            // variable corresponding to outId
+    LabelDsymbol *returnLabel;          // where the return goes
+
+    DsymbolTable *localsymtab;          // used to prevent symbols in different
+                                        // scopes from having the same name
+    VarDeclaration *vthis;              // 'this' parameter (member and nested)
+    VarDeclaration *v_arguments;        // '_arguments' parameter
 #if IN_GCC
-    VarDeclaration *v_arguments_var;	// '_arguments' variable
-    VarDeclaration *v_argptr;	        // '_argptr' variable
+    VarDeclaration *v_arguments_var;    // '_arguments' variable
+    VarDeclaration *v_argptr;           // '_argptr' variable
 #endif
-    Dsymbols *parameters;		// Array of VarDeclaration's for parameters
-    DsymbolTable *labtab;		// statement label symbol table
-    Declaration *overnext;		// next in overload list
-    Loc endloc;				// location of closing curly bracket
-    int vtblIndex;			// for member functions, index into vtbl[]
-    int naked;				// !=0 if naked
-    int inlineAsm;			// !=0 if has inline assembler
+    Dsymbols *parameters;               // Array of VarDeclaration's for parameters
+    DsymbolTable *labtab;               // statement label symbol table
+    Declaration *overnext;              // next in overload list
+    Loc endloc;                         // location of closing curly bracket
+    int vtblIndex;                      // for member functions, index into vtbl[]
+    int naked;                          // !=0 if naked
+    int inlineAsm;                      // !=0 if has inline assembler
     ILS inlineStatus;
-    int inlineNest;			// !=0 if nested inline
-    int cantInterpret;			// !=0 if cannot interpret function
-    int semanticRun;			// 1 semantic() run
- 					// 2 semantic2() run
- 					// 3 semantic3() started
- 					// 4 semantic3() done
- 					// 5 toObjFile() run
-					// this function's frame ptr
-    ForeachStatement *fes;		// if foreach body, this is the foreach
-    int introducing;			// !=0 if 'introducing' function
-    Type *tintro;			// if !=NULL, then this is the type
-					// of the 'introducing' function
-					// this one is overriding
-    int inferRetType;			// !=0 if return type is to be inferred
+    int inlineNest;                     // !=0 if nested inline
+    int cantInterpret;                  // !=0 if cannot interpret function
+    int semanticRun;                    // 1 semantic() run
+                                        // 2 semantic2() run
+                                        // 3 semantic3() started
+                                        // 4 semantic3() done
+                                        // 5 toObjFile() run
+                                        // this function's frame ptr
+    ForeachStatement *fes;              // if foreach body, this is the foreach
+    int introducing;                    // !=0 if 'introducing' function
+    Type *tintro;                       // if !=NULL, then this is the type
+                                        // of the 'introducing' function
+                                        // this one is overriding
+    int inferRetType;                   // !=0 if return type is to be inferred
 
     // Things that should really go into Scope
-    int hasReturnExp;			// 1 if there's a return exp; statement
-					// 2 if there's a throw statement
-					// 4 if there's an assert(0)
-					// 8 if there's inline asm
+    int hasReturnExp;                   // 1 if there's a return exp; statement
+                                        // 2 if there's a throw statement
+                                        // 4 if there's an assert(0)
+                                        // 8 if there's inline asm
 
     // Support for NRVO (named return value optimization)
-    int nrvo_can;			// !=0 means we can do it
-    VarDeclaration *nrvo_var;		// variable to replace with shidden
-    Symbol *shidden;			// hidden pointer passed to function
+    int nrvo_can;                       // !=0 means we can do it
+    VarDeclaration *nrvo_var;           // variable to replace with shidden
+    Symbol *shidden;                    // hidden pointer passed to function
 
 #if DMDV2
-    enum BUILTIN builtin;		// set if this is a known, builtin
-					// function we can evaluate at compile
-					// time
+    enum BUILTIN builtin;               // set if this is a known, builtin
+                                        // function we can evaluate at compile
+                                        // time
 
-    int tookAddressOf;			// set if someone took the address of
-					// this function
-    Dsymbols closureVars;		// local variables in this function
-					// which are referenced by nested
-					// functions
+    int tookAddressOf;                  // set if someone took the address of
+                                        // this function
+    Dsymbols closureVars;               // local variables in this function
+                                        // which are referenced by nested
+                                        // functions
 #else
-    int nestedFrameRef;			// !=0 if nested variables referenced
+    int nestedFrameRef;                 // !=0 if nested variables referenced
 #endif
 
-    FuncDeclaration(Loc loc, Loc endloc, Identifier *id, enum STC storage_class, Type *type);
+    FuncDeclaration(Loc loc, Loc endloc, Identifier *id, StorageClass storage_class, Type *type);
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
     void semantic2(Scope *sc);
@@ -562,10 +592,11 @@ struct FuncDeclaration : Declaration
     LabelDsymbol *searchLabel(Identifier *ident);
     AggregateDeclaration *isThis();
     AggregateDeclaration *isMember2();
-    int getLevel(Loc loc, FuncDeclaration *fd);	// lexical nesting level difference
+    int getLevel(Loc loc, FuncDeclaration *fd); // lexical nesting level difference
     void appendExp(Expression *e);
     void appendState(Statement *s);
     char *mangle();
+    const char *toPrettyChars();
     int isMain();
     int isWinMain();
     int isDllMain();
@@ -582,26 +613,40 @@ struct FuncDeclaration : Declaration
     virtual int isFinal();
     virtual int addPreInvariant();
     virtual int addPostInvariant();
-    Expression *interpret(InterState *istate, Expressions *arguments);
+    Expression *interpret(InterState *istate, Expressions *arguments, Expression *thisexp = NULL);
     void inlineScan();
     int canInline(int hasthis, int hdrscan = 0);
     Expression *doInline(InlineScanState *iss, Expression *ethis, Array *arguments);
     const char *kind();
     void toDocBuffer(OutBuffer *buf);
+    FuncDeclaration *isUnique();
+    int needsClosure();
+    Statement *mergeFrequire(Statement *);
+    Statement *mergeFensure(Statement *);
+    Parameters *getParameters(int *pvarargs);
 
     static FuncDeclaration *genCfunc(Type *treturn, const char *name,
-	Type *t1 = 0, Type *t2 = 0, Type *t3 = 0);
+        Type *t1 = 0, Type *t2 = 0, Type *t3 = 0);
     static FuncDeclaration *genCfunc(Type *treturn, Identifier *id,
-	Type *t1 = 0, Type *t2 = 0, Type *t3 = 0);
+        Type *t1 = 0, Type *t2 = 0, Type *t3 = 0);
 
     Symbol *toSymbol();
-    Symbol *toThunkSymbol(target_ptrdiff_t offset);	// thunk version
-    void toObjFile(int multiobj);			// compile to .obj file
+    Symbol *toThunkSymbol(target_ptrdiff_t offset);     // thunk version
+    void toObjFile(int multiobj);                       // compile to .obj file
     int cvMember(unsigned char *p);
+    void buildClosure(IRState *irs);
 
     FuncDeclaration *isFuncDeclaration() { return this; }
 };
 
+#if DMDV2
+FuncDeclaration *resolveFuncCall(Scope *sc, Loc loc, Dsymbol *s,
+        Objects *tiargs,
+        Expression *ethis,
+        Expressions *arguments,
+        int flags);
+#endif
+
 struct FuncAliasDeclaration : FuncDeclaration
 {
     FuncDeclaration *funcalias;
@@ -615,10 +660,10 @@ struct FuncAliasDeclaration : FuncDeclaration
 
 struct FuncLiteralDeclaration : FuncDeclaration
 {
-    enum TOK tok;			// TOKfunction or TOKdelegate
+    enum TOK tok;                       // TOKfunction or TOKdelegate
 
     FuncLiteralDeclaration(Loc loc, Loc endloc, Type *type, enum TOK tok,
-	ForeachStatement *fes);
+        ForeachStatement *fes);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Dsymbol *syntaxCopy(Dsymbol *);
     int isNested();
@@ -629,10 +674,10 @@ struct FuncLiteralDeclaration : FuncDeclaration
 };
 
 struct CtorDeclaration : FuncDeclaration
-{   Arguments *arguments;
+{   Parameters *arguments;
     int varargs;
 
-    CtorDeclaration(Loc loc, Loc endloc, Arguments *arguments, int varargs);
+    CtorDeclaration(Loc loc, Loc endloc, Parameters *arguments, int varargs);
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -647,22 +692,23 @@ struct CtorDeclaration : FuncDeclaration
 };
 
 #if DMDV2
- struct PostBlitDeclaration : FuncDeclaration
- {
-     PostBlitDeclaration(Loc loc, Loc endloc);
-    PostBlitDeclaration(Loc loc, Loc endloc, Identifier *id);
-     Dsymbol *syntaxCopy(Dsymbol *);
-     void semantic(Scope *sc);
-     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-     int isVirtual();
-     int addPreInvariant();
-     int addPostInvariant();
-     int overloadInsert(Dsymbol *s);
-     void emitComment(Scope *sc);
- 
-     PostBlitDeclaration *isPostBlitDeclaration() { return this; }
- };
- #endif
+struct PostBlitDeclaration : FuncDeclaration
+{
+    PostBlitDeclaration(Loc loc, Loc endloc);
+   PostBlitDeclaration(Loc loc, Loc endloc, Identifier *id);
+    Dsymbol *syntaxCopy(Dsymbol *);
+    void semantic(Scope *sc);
+    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+    int isVirtual();
+    int addPreInvariant();
+    int addPostInvariant();
+    int overloadInsert(Dsymbol *s);
+    void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
+
+    PostBlitDeclaration *isPostBlitDeclaration() { return this; }
+};
+#endif
 
 struct DtorDeclaration : FuncDeclaration
 {
@@ -678,6 +724,7 @@ struct DtorDeclaration : FuncDeclaration
     int addPostInvariant();
     int overloadInsert(Dsymbol *s);
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
 
     DtorDeclaration *isDtorDeclaration() { return this; }
 };
@@ -693,13 +740,14 @@ struct StaticCtorDeclaration : FuncDeclaration
     int addPreInvariant();
     int addPostInvariant();
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     StaticCtorDeclaration *isStaticCtorDeclaration() { return this; }
 };
 
 struct StaticDtorDeclaration : FuncDeclaration
-{   VarDeclaration *vgate;	// 'gate' variable
+{   VarDeclaration *vgate;      // 'gate' variable
     StaticDtorDeclaration(Loc loc, Loc endloc);
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
@@ -709,6 +757,7 @@ struct StaticDtorDeclaration : FuncDeclaration
     int addPreInvariant();
     int addPostInvariant();
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     StaticDtorDeclaration *isStaticDtorDeclaration() { return this; }
@@ -723,6 +772,7 @@ struct InvariantDeclaration : FuncDeclaration
     int addPreInvariant();
     int addPostInvariant();
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     InvariantDeclaration *isInvariantDeclaration() { return this; }
@@ -739,15 +789,16 @@ struct UnitTestDeclaration : FuncDeclaration
     int addPreInvariant();
     int addPostInvariant();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+    void toJsonBuffer(OutBuffer *buf);
 
     UnitTestDeclaration *isUnitTestDeclaration() { return this; }
 };
 
 struct NewDeclaration : FuncDeclaration
-{   Arguments *arguments;
+{   Parameters *arguments;
     int varargs;
 
-    NewDeclaration(Loc loc, Loc endloc, Arguments *arguments, int varargs);
+    NewDeclaration(Loc loc, Loc endloc, Parameters *arguments, int varargs);
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -761,9 +812,9 @@ struct NewDeclaration : FuncDeclaration
 
 
 struct DeleteDeclaration : FuncDeclaration
-{   Arguments *arguments;
+{   Parameters *arguments;
 
-    DeleteDeclaration(Loc loc, Loc endloc, Arguments *arguments);
+    DeleteDeclaration(Loc loc, Loc endloc, Parameters *arguments);
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
