@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2009 by Digital Mars
+// Copyright (c) 1999-2010 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -48,7 +48,7 @@ struct ClassDeclaration;
 struct HdrGenState;
 struct BinExp;
 struct InterState;
-struct Symbol;		// back end symbol
+struct Symbol;          // back end symbol
 struct OverloadSet;
 
 enum TOK;
@@ -68,7 +68,7 @@ void initPrecedence();
 Expression *resolveProperties(Scope *sc, Expression *e);
 void accessCheck(Loc loc, Scope *sc, Expression *e, Declaration *d);
 Dsymbol *search_function(AggregateDeclaration *ad, Identifier *funcid);
-void inferApplyArgTypes(enum TOK op, Arguments *arguments, Expression *aggr);
+void inferApplyArgTypes(enum TOK op, Parameters *arguments, Expression *aggr);
 void argExpTypesToCBuffer(OutBuffer *buf, Expressions *arguments, HdrGenState *hgs);
 void argsToCBuffer(OutBuffer *buf, Expressions *arguments, HdrGenState *hgs);
 void expandTuples(Expressions *exps);
@@ -76,19 +76,24 @@ FuncDeclaration *hasThis(Scope *sc);
 Expression *fromConstInitializer(int result, Expression *e);
 int arrayExpressionCanThrow(Expressions *exps);
 
+struct IntRange
+{   uinteger_t imin;
+    uinteger_t imax;
+};
+
 struct Expression : Object
 {
-    Loc loc;			// file location
-    enum TOK op;		// handy to minimize use of dynamic_cast
-    Type *type;			// !=NULL means that semantic() has been run
-    int size;			// # of bytes in Expression so we can copy() it
+    Loc loc;                    // file location
+    enum TOK op;                // handy to minimize use of dynamic_cast
+    Type *type;                 // !=NULL means that semantic() has been run
+    int size;                   // # of bytes in Expression so we can copy() it
 
     Expression(Loc loc, enum TOK op, int size);
     Expression *copy();
     virtual Expression *syntaxCopy();
     virtual Expression *semantic(Scope *sc);
 
-    int dyncast() { return DYNCAST_EXPRESSION; }	// kludge for template.isExpression()
+    int dyncast() { return DYNCAST_EXPRESSION; }        // kludge for template.isExpression()
 
     void print();
     char *toChars();
@@ -109,10 +114,11 @@ struct Expression : Object
     virtual void toMangleBuffer(OutBuffer *buf);
     virtual Expression *toLvalue(Scope *sc, Expression *e);
     virtual Expression *modifiableLvalue(Scope *sc, Expression *e);
-    Expression *implicitCastTo(Scope *sc, Type *t);
+    virtual Expression *implicitCastTo(Scope *sc, Type *t);
     virtual MATCH implicitConvTo(Type *t);
     virtual Expression *castTo(Scope *sc, Type *t);
     virtual void checkEscape();
+    virtual void checkEscapeRef();
     void checkScalar();
     void checkNoBool();
     Expression *checkIntegral();
@@ -128,8 +134,8 @@ struct Expression : Object
     virtual void scanForNestedRef(Scope *sc);
 
     virtual Expression *optimize(int result);
-    #define WANTflags	1
-    #define WANTvalue	2
+    #define WANTflags   1
+    #define WANTvalue   2
     #define WANTinterpret 4
 
     virtual Expression *interpret(InterState *istate);
@@ -143,15 +149,17 @@ struct Expression : Object
     virtual int inlineCost(InlineCostState *ics);
     virtual Expression *doInline(InlineDoState *ids);
     virtual Expression *inlineScan(InlineScanState *iss);
+    Expression *inlineCopy(Scope *sc);
 
     // For operator overloading
     virtual int isCommutative();
     virtual Identifier *opId();
     virtual Identifier *opId_r();
-    
+
     // For array ops
     virtual void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
-    virtual Expression *buildArrayLoop(Arguments *fparams);
+    virtual Expression *buildArrayLoop(Parameters *fparams);
+    int isArrayOperand();
 
     // Back end
     virtual elem *toElem(IRState *irs);
@@ -187,6 +195,7 @@ struct ErrorExp : IntegerExp
 {
     ErrorExp();
 
+    Expression *implicitCastTo(Scope *sc, Type *t);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 };
 
@@ -261,6 +270,7 @@ struct DollarExp : IdentifierExp
 struct DsymbolExp : Expression
 {
     Dsymbol *s;
+    int hasOverloads;
 
     DsymbolExp(Loc loc, Dsymbol *s);
     Expression *semantic(Scope *sc);
@@ -276,6 +286,7 @@ struct ThisExp : Expression
 
     ThisExp(Loc loc);
     Expression *semantic(Scope *sc);
+    Expression *interpret(InterState *istate);
     int isBool(int result);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Expression *toLvalue(Scope *sc, Expression *e);
@@ -302,11 +313,12 @@ struct SuperExp : ThisExp
 
 struct NullExp : Expression
 {
-    unsigned char committed;	// !=0 if type is committed
+    unsigned char committed;    // !=0 if type is committed
 
-    NullExp(Loc loc);
+    NullExp(Loc loc, Type *t = NULL);
     Expression *semantic(Scope *sc);
     int isBool(int result);
+    int isConst();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     void toMangleBuffer(OutBuffer *buf);
     MATCH implicitConvTo(Type *t);
@@ -318,11 +330,11 @@ struct NullExp : Expression
 
 struct StringExp : Expression
 {
-    void *string;	// char, wchar, or dchar data
-    size_t len;		// number of chars, wchars, or dchars
-    unsigned char sz;	// 1: char, 2: wchar, 4: dchar
-    unsigned char committed;	// !=0 if type is committed
-    unsigned char postfix;	// 'c', 'w', 'd'
+    void *string;       // char, wchar, or dchar data
+    size_t len;         // number of chars, wchars, or dchars
+    unsigned char sz;   // 1: char, 2: wchar, 4: dchar
+    unsigned char committed;    // !=0 if type is committed
+    unsigned char postfix;      // 'c', 'w', 'd'
 
     StringExp(Loc loc, char *s);
     StringExp(Loc loc, void *s, size_t len);
@@ -422,13 +434,13 @@ struct AssocArrayLiteralExp : Expression
 
 struct StructLiteralExp : Expression
 {
-    StructDeclaration *sd;		// which aggregate this is for
-    Expressions *elements;	// parallels sd->fields[] with
-				// NULL entries for fields to skip
+    StructDeclaration *sd;      // which aggregate this is for
+    Expressions *elements;      // parallels sd->fields[] with
+                                // NULL entries for fields to skip
 
-    Symbol *sym;		// back end symbol to initialize with literal
-    size_t soffset;		// offset from start of s
-    int fillHoles;		// fill alignment 'holes' with zero
+    Symbol *sym;                // back end symbol to initialize with literal
+    size_t soffset;             // offset from start of s
+    int fillHoles;              // fill alignment 'holes' with zero
 
     StructLiteralExp(Loc loc, StructDeclaration *sd, Expressions *elements);
 
@@ -487,19 +499,21 @@ struct NewExp : Expression
 {
     /* thisexp.new(newargs) newtype(arguments)
      */
-    Expression *thisexp;	// if !NULL, 'this' for class being allocated
-    Expressions *newargs;	// Array of Expression's to call new operator
+    Expression *thisexp;        // if !NULL, 'this' for class being allocated
+    Expressions *newargs;       // Array of Expression's to call new operator
     Type *newtype;
-    Expressions *arguments;	// Array of Expression's
+    Expressions *arguments;     // Array of Expression's
 
-    CtorDeclaration *member;	// constructor function
-    NewDeclaration *allocator;	// allocator function
-    int onstack;		// allocate on stack
+    CtorDeclaration *member;    // constructor function
+    NewDeclaration *allocator;  // allocator function
+    int onstack;                // allocate on stack
 
     NewExp(Loc loc, Expression *thisexp, Expressions *newargs,
-	Type *newtype, Expressions *arguments);
+        Type *newtype, Expressions *arguments);
     Expression *syntaxCopy();
     Expression *semantic(Scope *sc);
+    Expression *optimize(int result);
+    Expression *interpret(InterState *istate);
     elem *toElem(IRState *irs);
     int checkSideEffect(int flag);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -514,19 +528,31 @@ struct NewAnonClassExp : Expression
 {
     /* thisexp.new(newargs) class baseclasses { } (arguments)
      */
-    Expression *thisexp;	// if !NULL, 'this' for class being allocated
-    Expressions *newargs;	// Array of Expression's to call new operator
-    ClassDeclaration *cd;	// class being instantiated
-    Expressions *arguments;	// Array of Expression's to call class constructor
+    Expression *thisexp;        // if !NULL, 'this' for class being allocated
+    Expressions *newargs;       // Array of Expression's to call new operator
+    ClassDeclaration *cd;       // class being instantiated
+    Expressions *arguments;     // Array of Expression's to call class constructor
 
     NewAnonClassExp(Loc loc, Expression *thisexp, Expressions *newargs,
-	ClassDeclaration *cd, Expressions *arguments);
+        ClassDeclaration *cd, Expressions *arguments);
     Expression *syntaxCopy();
     Expression *semantic(Scope *sc);
     int checkSideEffect(int flag);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 };
 
+#if DMDV2
+struct SymbolExp : Expression
+{
+    Declaration *var;
+    int hasOverloads;
+
+    SymbolExp(Loc loc, enum TOK op, int size, Declaration *var, int hasOverloads);
+
+    elem *toElem(IRState *irs);
+};
+#endif
+
 // Offset from symbol
 
 struct SymOffExp : Expression
@@ -536,6 +562,7 @@ struct SymOffExp : Expression
 
     SymOffExp(Loc loc, Declaration *var, target_size_t offset);
     Expression *semantic(Scope *sc);
+    Expression *interpret(InterState *istate);
     void checkEscape();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     int isConst();
@@ -564,6 +591,8 @@ struct VarExp : Expression
     char *toChars();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     void checkEscape();
+    void checkEscapeRef();
+    int isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
     elem *toElem(IRState *irs);
@@ -576,17 +605,17 @@ struct VarExp : Expression
 };
 
 #if DMDV2
- // Overload Set
- 
- struct OverExp : Expression
- {
-     OverloadSet *vars;
- 
-     OverExp(OverloadSet *s);
-     int isLvalue();
-     Expression *toLvalue(Scope *sc, Expression *e);
- };
- #endif
+// Overload Set
+
+struct OverExp : Expression
+{
+    OverloadSet *vars;
+
+    OverExp(OverloadSet *s);
+    int isLvalue();
+    Expression *toLvalue(Scope *sc, Expression *e);
+};
+#endif
 
 // Function/Delegate literal
 
@@ -597,6 +626,7 @@ struct FuncExp : Expression
     FuncExp(Loc loc, FuncLiteralDeclaration *fd);
     Expression *syntaxCopy();
     Expression *semantic(Scope *sc);
+    Expression *interpret(InterState *istate);
     void scanForNestedRef(Scope *sc);
     char *toChars();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -638,17 +668,17 @@ struct TypeidExp : Expression
 };
 
 #if DMDV2
- struct TraitsExp : Expression
- {
-     Identifier *ident;
-     Objects *args;
- 
-     TraitsExp(Loc loc, Identifier *ident, Objects *args);
-     Expression *syntaxCopy();
-     Expression *semantic(Scope *sc);
-     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
- };
- #endif
+struct TraitsExp : Expression
+{
+    Identifier *ident;
+    Objects *args;
+
+    TraitsExp(Loc loc, Identifier *ident, Objects *args);
+    Expression *syntaxCopy();
+    Expression *semantic(Scope *sc);
+    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+};
+#endif
 
 struct HaltExp : Expression
 {
@@ -666,10 +696,10 @@ struct IsExp : Expression
      * is(targ id == tok2)
      */
     Type *targ;
-    Identifier *id;	// can be NULL
-    enum TOK tok;	// ':' or '=='
-    Type *tspec;	// can be NULL
-    enum TOK tok2;	// 'struct', 'union', 'typedef', etc.
+    Identifier *id;     // can be NULL
+    enum TOK tok;       // ':' or '=='
+    Type *tspec;        // can be NULL
+    enum TOK tok2;      // 'struct', 'union', 'typedef', etc.
 
     IsExp(Loc loc, Type *targ, Identifier *id, enum TOK tok, Type *tspec, enum TOK tok2);
     Expression *syntaxCopy();
@@ -696,7 +726,7 @@ struct UnaExp : Expression
     Expression *doInline(InlineDoState *ids);
     Expression *inlineScan(InlineScanState *iss);
 
-    Expression *op_overload(Scope *sc);	// doesn't need to be virtual
+    Expression *op_overload(Scope *sc); // doesn't need to be virtual
 };
 
 struct BinExp : Expression
@@ -711,6 +741,8 @@ struct BinExp : Expression
     Expression *commonSemanticAssign(Scope *sc);
     Expression *commonSemanticAssignIntegral(Scope *sc);
     int checkSideEffect(int flag);
+    void checkComplexMulAssign();
+    void checkComplexAddAssign();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Expression *scaleFactor(Scope *sc);
     Expression *typeCombine(Scope *sc);
@@ -786,7 +818,7 @@ struct DotIdExp : UnaExp
 struct DotTemplateExp : UnaExp
 {
     TemplateDeclaration *td;
-    
+
     DotTemplateExp(Loc loc, Expression *e, TemplateDeclaration *td);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 };
@@ -823,6 +855,7 @@ struct DelegateExp : UnaExp
 
     DelegateExp(Loc loc, Expression *e, FuncDeclaration *func);
     Expression *semantic(Scope *sc);
+    Expression *interpret(InterState *istate);
     MATCH implicitConvTo(Type *t);
     Expression *castTo(Scope *sc, Type *t);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -834,7 +867,7 @@ struct DelegateExp : UnaExp
 
 struct DotTypeExp : UnaExp
 {
-    Dsymbol *sym;		// symbol that represents a type
+    Dsymbol *sym;               // symbol that represents a type
 
     DotTypeExp(Loc loc, Expression *e, Dsymbol *sym);
     Expression *semantic(Scope *sc);
@@ -844,7 +877,7 @@ struct DotTypeExp : UnaExp
 
 struct CallExp : UnaExp
 {
-    Expressions *arguments;	// function arguments
+    Expressions *arguments;     // function arguments
 
     CallExp(Loc loc, Expression *e, Expressions *exps);
     CallExp(Loc loc, Expression *e);
@@ -872,6 +905,7 @@ struct AddrExp : UnaExp
 {
     AddrExp(Loc loc, Expression *e);
     Expression *semantic(Scope *sc);
+    void checkEscape();
     elem *toElem(IRState *irs);
     MATCH implicitConvTo(Type *t);
     Expression *castTo(Scope *sc, Type *t);
@@ -883,6 +917,7 @@ struct PtrExp : UnaExp
     PtrExp(Loc loc, Expression *e);
     PtrExp(Loc loc, Expression *e, Type *t);
     Expression *semantic(Scope *sc);
+    void checkEscapeRef();
     Expression *toLvalue(Scope *sc, Expression *e);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     elem *toElem(IRState *irs);
@@ -897,7 +932,7 @@ struct NegExp : UnaExp
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
     void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
-    Expression *buildArrayLoop(Arguments *fparams);
+    Expression *buildArrayLoop(Parameters *fparams);
 
     // For operator overloading
     Identifier *opId();
@@ -921,7 +956,7 @@ struct ComExp : UnaExp
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
     void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
-    Expression *buildArrayLoop(Arguments *fparams);
+    Expression *buildArrayLoop(Parameters *fparams);
 
     // For operator overloading
     Identifier *opId();
@@ -962,7 +997,7 @@ struct DeleteExp : UnaExp
 struct CastExp : UnaExp
 {
     // Possible to cast to one type while painting to another type
-    Type *to;			// type to cast to
+    Type *to;                   // type to cast to
 
     CastExp(Loc loc, Expression *e, Type *t);
     Expression *syntaxCopy();
@@ -972,6 +1007,8 @@ struct CastExp : UnaExp
     int checkSideEffect(int flag);
     void checkEscape();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+    void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
+    Expression *buildArrayLoop(Parameters *fparams);
     elem *toElem(IRState *irs);
 
     // For operator overloading
@@ -981,14 +1018,15 @@ struct CastExp : UnaExp
 
 struct SliceExp : UnaExp
 {
-    Expression *upr;		// NULL if implicit 0
-    Expression *lwr;		// NULL if implicit [length - 1]
+    Expression *upr;            // NULL if implicit 0
+    Expression *lwr;            // NULL if implicit [length - 1]
     VarDeclaration *lengthVar;
 
     SliceExp(Loc loc, Expression *e1, Expression *lwr, Expression *upr);
     Expression *syntaxCopy();
     Expression *semantic(Scope *sc);
     void checkEscape();
+    void checkEscapeRef();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -998,7 +1036,7 @@ struct SliceExp : UnaExp
     elem *toElem(IRState *irs);
     void scanForNestedRef(Scope *sc);
     void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
-    Expression *buildArrayLoop(Arguments *fparams);
+    Expression *buildArrayLoop(Parameters *fparams);
 
     int inlineCost(InlineCostState *ics);
     Expression *doInline(InlineDoState *ids);
@@ -1019,7 +1057,7 @@ struct ArrayLengthExp : UnaExp
 
 struct ArrayExp : UnaExp
 {
-    Expressions *arguments;		// Array of Expression's
+    Expressions *arguments;             // Array of Expression's
 
     ArrayExp(Loc loc, Expression *e1, Expressions *arguments);
     Expression *syntaxCopy();
@@ -1049,6 +1087,7 @@ struct CommaExp : BinExp
     CommaExp(Loc loc, Expression *e1, Expression *e2);
     Expression *semantic(Scope *sc);
     void checkEscape();
+    void checkEscapeRef();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
     int isBool(int result);
@@ -1089,7 +1128,7 @@ struct PostExp : BinExp
 };
 
 struct AssignExp : BinExp
-{   int ismemset;	// !=0 if setting the contents of an array
+{   int ismemset;       // !=0 if setting the contents of an array
 
     AssignExp(Loc loc, Expression *e1, Expression *e2);
     Expression *semantic(Scope *sc);
@@ -1097,22 +1136,22 @@ struct AssignExp : BinExp
     Expression *interpret(InterState *istate);
     Identifier *opId();    // For operator overloading
     void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
-    Expression *buildArrayLoop(Arguments *fparams);
+    Expression *buildArrayLoop(Parameters *fparams);
     elem *toElem(IRState *irs);
 };
 
-#define ASSIGNEXP(op)	\
-struct op##AssignExp : BinExp					\
-{								\
-    op##AssignExp(Loc loc, Expression *e1, Expression *e2);	\
-    Expression *semantic(Scope *sc);				\
-    Expression *interpret(InterState *istate);			\
+#define ASSIGNEXP(op)   \
+struct op##AssignExp : BinExp                                   \
+{                                                               \
+    op##AssignExp(Loc loc, Expression *e1, Expression *e2);     \
+    Expression *semantic(Scope *sc);                            \
+    Expression *interpret(InterState *istate);                  \
     X(void buildArrayIdent(OutBuffer *buf, Expressions *arguments);) \
-    X(Expression *buildArrayLoop(Arguments *fparams);)		\
-								\
-    Identifier *opId();    /* For operator overloading */	\
-								\
-    elem *toElem(IRState *irs);					\
+    X(Expression *buildArrayLoop(Parameters *fparams);)         \
+                                                                \
+    Identifier *opId();    /* For operator overloading */       \
+                                                                \
+    elem *toElem(IRState *irs);                                 \
 };
 
 #define X(a) a
@@ -1127,13 +1166,13 @@ ASSIGNEXP(Xor)
 #undef X
 
 #define X(a)
- 
- ASSIGNEXP(Shl)
- ASSIGNEXP(Shr)
- ASSIGNEXP(Ushr)
- ASSIGNEXP(Cat)
- 
- #undef X
+
+ASSIGNEXP(Shl)
+ASSIGNEXP(Shr)
+ASSIGNEXP(Ushr)
+ASSIGNEXP(Cat)
+
+#undef X
 #undef ASSIGNEXP
 
 struct AddExp : BinExp
@@ -1143,7 +1182,7 @@ struct AddExp : BinExp
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
     void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
-    Expression *buildArrayLoop(Arguments *fparams);
+    Expression *buildArrayLoop(Parameters *fparams);
 
     // For operator overloading
     int isCommutative();
@@ -1160,7 +1199,7 @@ struct MinExp : BinExp
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
     void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
-    Expression *buildArrayLoop(Arguments *fparams);
+    Expression *buildArrayLoop(Parameters *fparams);
 
     // For operator overloading
     Identifier *opId();
@@ -1190,7 +1229,7 @@ struct MulExp : BinExp
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
     void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
-    Expression *buildArrayLoop(Arguments *fparams);
+    Expression *buildArrayLoop(Parameters *fparams);
 
     // For operator overloading
     int isCommutative();
@@ -1207,7 +1246,7 @@ struct DivExp : BinExp
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
     void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
-    Expression *buildArrayLoop(Arguments *fparams);
+    Expression *buildArrayLoop(Parameters *fparams);
 
     // For operator overloading
     Identifier *opId();
@@ -1223,7 +1262,7 @@ struct ModExp : BinExp
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
     void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
-    Expression *buildArrayLoop(Arguments *fparams);
+    Expression *buildArrayLoop(Parameters *fparams);
 
     // For operator overloading
     Identifier *opId();
@@ -1232,6 +1271,18 @@ struct ModExp : BinExp
     elem *toElem(IRState *irs);
 };
 
+#if DMDV2
+struct PowExp : BinExp
+{
+    PowExp(Loc loc, Expression *e1, Expression *e2);
+    Expression *semantic(Scope *sc);
+
+    // For operator overloading
+    Identifier *opId();
+    Identifier *opId_r();
+};
+#endif
+
 struct ShlExp : BinExp
 {
     ShlExp(Loc loc, Expression *e1, Expression *e2);
@@ -1281,7 +1332,7 @@ struct AndExp : BinExp
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
     void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
-    Expression *buildArrayLoop(Arguments *fparams);
+    Expression *buildArrayLoop(Parameters *fparams);
 
     // For operator overloading
     int isCommutative();
@@ -1298,7 +1349,7 @@ struct OrExp : BinExp
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
     void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
-    Expression *buildArrayLoop(Arguments *fparams);
+    Expression *buildArrayLoop(Parameters *fparams);
 
     // For operator overloading
     int isCommutative();
@@ -1315,7 +1366,7 @@ struct XorExp : BinExp
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
     void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
-    Expression *buildArrayLoop(Arguments *fparams);
+    Expression *buildArrayLoop(Parameters *fparams);
 
     // For operator overloading
     int isCommutative();
@@ -1424,6 +1475,7 @@ struct CondExp : BinExp
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
     void checkEscape();
+    void checkEscapeRef();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
     Expression *checkToBoolean();
@@ -1441,42 +1493,42 @@ struct CondExp : BinExp
 };
 
 #if DMDV2
- /****************************************************************/
- 
- struct DefaultInitExp : Expression
- {
-     enum TOK subop;		// which of the derived classes this is
- 
-     DefaultInitExp(Loc loc, enum TOK subop, int size);
-     virtual Expression *resolve(Loc loc, Scope *sc) = 0;
-     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
- };
- 
- struct FileInitExp : DefaultInitExp
- {
-     FileInitExp(Loc loc);
-     Expression *semantic(Scope *sc);
-     Expression *resolve(Loc loc, Scope *sc);
- };
- 
- struct LineInitExp : DefaultInitExp
- {
-     LineInitExp(Loc loc);
-     Expression *semantic(Scope *sc);
-     Expression *resolve(Loc loc, Scope *sc);
- };
- #endif
+/****************************************************************/
+
+struct DefaultInitExp : Expression
+{
+    enum TOK subop;             // which of the derived classes this is
+
+    DefaultInitExp(Loc loc, enum TOK subop, int size);
+    virtual Expression *resolve(Loc loc, Scope *sc) = 0;
+    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+};
+
+struct FileInitExp : DefaultInitExp
+{
+    FileInitExp(Loc loc);
+    Expression *semantic(Scope *sc);
+    Expression *resolve(Loc loc, Scope *sc);
+};
+
+struct LineInitExp : DefaultInitExp
+{
+    LineInitExp(Loc loc);
+    Expression *semantic(Scope *sc);
+    Expression *resolve(Loc loc, Scope *sc);
+};
+#endif
 
 
 /****************************************************************/
 
 /* Special values used by the interpreter
  */
-#define EXP_CANT_INTERPRET	((Expression *)1)
-#define EXP_CONTINUE_INTERPRET	((Expression *)2)
-#define EXP_BREAK_INTERPRET	((Expression *)3)
-#define EXP_GOTO_INTERPRET	((Expression *)4)
-#define EXP_VOID_INTERPRET	((Expression *)5)
+#define EXP_CANT_INTERPRET      ((Expression *)1)
+#define EXP_CONTINUE_INTERPRET  ((Expression *)2)
+#define EXP_BREAK_INTERPRET     ((Expression *)3)
+#define EXP_GOTO_INTERPRET      ((Expression *)4)
+#define EXP_VOID_INTERPRET      ((Expression *)5)
 
 Expression *expType(Type *type, Expression *e);
 
