@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2007 by Digital Mars
+// Copyright (c) 1999-2010 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -45,7 +45,7 @@ enum LINK;
 
 struct TypeBasic;
 struct HdrGenState;
-struct Argument;
+struct Parameter;
 
 // Back end
 #if IN_GCC
@@ -60,9 +60,9 @@ struct Symbol;
 
 enum TY
 {
-    Tarray,		// dynamic array
-    Tsarray,		// static array
-    Taarray,		// associative array
+    Tarray,             // dynamic array
+    Tsarray,            // static array
+    Taarray,            // associative array
     Tpointer,
     Treference,
     Tfunction,
@@ -121,58 +121,58 @@ extern int Tindex;
 struct Type : Object
 {
     TY ty;
-    unsigned char mod;	// modifiers MODxxxx
- 	/* pick this order of numbers so switch statements work better
-! 	 */
-	#define MODconst     1	// type is const
-	#define MODinvariant 4	// type is invariant
- 	#define MODshared    2	// type is shared
+    unsigned char mod;  // modifiers MODxxxx
+        /* pick this order of numbers so switch statements work better
+         */
+        #define MODconst     1  // type is const
+        #define MODimmutable 4  // type is invariant
+        #define MODshared    2  // type is shared
     char *deco;
-    Type *pto;		// merged pointer to this type
-    Type *rto;		// reference to this type
-    Type *arrayof;	// array of this type
-    TypeInfoDeclaration *vtinfo;	// TypeInfo object for this Type
+    Type *pto;          // merged pointer to this type
+    Type *rto;          // reference to this type
+    Type *arrayof;      // array of this type
+    TypeInfoDeclaration *vtinfo;        // TypeInfo object for this Type
 
-    type *ctype;	// for back end
+    type *ctype;        // for back end
 
-    #define tvoid	basic[Tvoid]
-    #define tint8	basic[Tint8]
-    #define tuns8	basic[Tuns8]
-    #define tint16	basic[Tint16]
-    #define tuns16	basic[Tuns16]
-    #define tint32	basic[Tint32]
-    #define tuns32	basic[Tuns32]
-    #define tint64	basic[Tint64]
-    #define tuns64	basic[Tuns64]
-    #define tfloat32	basic[Tfloat32]
-    #define tfloat64	basic[Tfloat64]
-    #define tfloat80	basic[Tfloat80]
+    #define tvoid       basic[Tvoid]
+    #define tint8       basic[Tint8]
+    #define tuns8       basic[Tuns8]
+    #define tint16      basic[Tint16]
+    #define tuns16      basic[Tuns16]
+    #define tint32      basic[Tint32]
+    #define tuns32      basic[Tuns32]
+    #define tint64      basic[Tint64]
+    #define tuns64      basic[Tuns64]
+    #define tfloat32    basic[Tfloat32]
+    #define tfloat64    basic[Tfloat64]
+    #define tfloat80    basic[Tfloat80]
 
     #define timaginary32 basic[Timaginary32]
     #define timaginary64 basic[Timaginary64]
     #define timaginary80 basic[Timaginary80]
 
-    #define tcomplex32	basic[Tcomplex32]
-    #define tcomplex64	basic[Tcomplex64]
-    #define tcomplex80	basic[Tcomplex80]
+    #define tcomplex32  basic[Tcomplex32]
+    #define tcomplex64  basic[Tcomplex64]
+    #define tcomplex80  basic[Tcomplex80]
 
-    #define tbit	basic[Tbit]
-    #define tbool	basic[Tbool]
-    #define tchar	basic[Tchar]
-    #define twchar	basic[Twchar]
-    #define tdchar	basic[Tdchar]
+    #define tbit        basic[Tbit]
+    #define tbool       basic[Tbool]
+    #define tchar       basic[Tchar]
+    #define twchar      basic[Twchar]
+    #define tdchar      basic[Tdchar]
 
     // Some special types
-    #define tshiftcnt	tint32		// right side of shift expression
-//    #define tboolean	tint32		// result of boolean expression
-    #define tboolean	tbool		// result of boolean expression
-    #define tindex	basic[Tindex]	// array/ptr index
-    static Type *tvoidptr;		// void*
-    #define terror	basic[Terror]	// for error recovery
+    #define tshiftcnt   tint32          // right side of shift expression
+//    #define tboolean  tint32          // result of boolean expression
+    #define tboolean    tbool           // result of boolean expression
+    #define tindex      basic[Tindex]   // array/ptr index
+    static Type *tvoidptr;              // void*
+    #define terror      basic[Terror]   // for error recovery
 
-    #define tsize_t	basic[Tsize_t]		// matches size_t alias
-    #define tptrdiff_t	basic[Tptrdiff_t]	// matches ptrdiff_t alias
-    #define thash_t	tsize_t			// matches hash_t alias
+    #define tsize_t     basic[Tsize_t]          // matches size_t alias
+    #define tptrdiff_t  basic[Tptrdiff_t]       // matches ptrdiff_t alias
+    #define thash_t     tsize_t                 // matches hash_t alias
 
     static ClassDeclaration *typeinfo;
     static ClassDeclaration *typeinfoclass;
@@ -221,7 +221,7 @@ struct Type : Object
     void toCBuffer3(OutBuffer *buf, HdrGenState *hgs, int mod);
     virtual int isbit();
     virtual int isintegral();
-    virtual int isfloating();	// real, imaginary, or complex
+    virtual int isfloating();   // real, imaginary, or complex
     virtual int isreal();
     virtual int isimaginary();
     virtual int iscomplex();
@@ -229,7 +229,7 @@ struct Type : Object
     virtual int isunsigned();
     virtual int isauto();
     virtual int isString();
-    virtual int checkBoolean();	// if can be converted to boolean value
+    virtual int checkBoolean(); // if can be converted to boolean value
     void checkDeprecated(Loc loc, Scope *sc);
     Type *pointerTo();
     Type *referenceTo();
@@ -243,7 +243,8 @@ struct Type : Object
     virtual Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     virtual unsigned memalign(unsigned salign);
     virtual Expression *defaultInit(Loc loc = 0);
-    virtual int isZeroInit(Loc loc = 0);		// if initializer is 0
+    virtual Expression *defaultInitLiteral(Loc loc = 0);
+    virtual int isZeroInit(Loc loc = 0);                // if initializer is 0
     virtual dt_t **toDt(dt_t **pdt);
     Identifier *getTypeInfoIdent(int internal);
     virtual MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
@@ -327,6 +328,7 @@ struct TypeSArray : TypeArray
     unsigned memalign(unsigned salign);
     MATCH implicitConvTo(Type *to);
     Expression *defaultInit(Loc loc);
+    Expression *defaultInitLiteral(Loc loc);
     dt_t **toDt(dt_t **pdt);
     dt_t **toDtElem(dt_t **pdt, Expression *e);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
@@ -363,8 +365,8 @@ struct TypeDArray : TypeArray
 
 struct TypeAArray : TypeArray
 {
-    Type *index;		// key type for type checking
-    Type *key;			// actual key type
+    Type *index;                // key type for type checking
+    Type *key;                  // actual key type
 
     TypeAArray(Type *t, Type *index);
     Type *syntaxCopy();
@@ -417,20 +419,20 @@ struct TypeReference : Type
 
 enum RET
 {
-    RETregs	= 1,	// returned in registers
-    RETstack	= 2,	// returned on stack
+    RETregs     = 1,    // returned in registers
+    RETstack    = 2,    // returned on stack
 };
 
 struct TypeFunction : Type
 {
-    Arguments *parameters;	// function parameters
-    int varargs;	// 1: T t, ...) style for variable number of arguments
-			// 2: T t ...) style for variable number of arguments
-    enum LINK linkage;	// calling convention
+    Parameters *parameters;     // function parameters
+    int varargs;        // 1: T t, ...) style for variable number of arguments
+                        // 2: T t ...) style for variable number of arguments
+    enum LINK linkage;  // calling convention
 
     int inuse;
 
-    TypeFunction(Arguments *parameters, Type *treturn, int varargs, enum LINK linkage);
+    TypeFunction(Parameters *parameters, Type *treturn, int varargs, enum LINK linkage);
     Type *syntaxCopy();
     Type *semantic(Loc loc, Scope *sc);
     void toDecoBuffer(OutBuffer *buf);
@@ -467,7 +469,7 @@ struct TypeDelegate : Type
 struct TypeQualified : Type
 {
     Loc loc;
-    Array idents;	// array of Identifier's representing ident.ident.ident etc.
+    Array idents;       // array of Identifier's representing ident.ident.ident etc.
 
     TypeQualified(TY ty, Loc loc);
     void syntaxCopyHelper(TypeQualified *t);
@@ -475,7 +477,7 @@ struct TypeQualified : Type
     void toCBuffer2Helper(OutBuffer *buf, HdrGenState *hgs);
     d_uns64 size(Loc loc);
     void resolveHelper(Loc loc, Scope *sc, Dsymbol *s, Dsymbol *scopesym,
-	Expression **pe, Type **pt, Dsymbol **ps);
+        Expression **pe, Type **pt, Dsymbol **ps);
 };
 
 struct TypeIdentifier : TypeQualified
@@ -520,6 +522,7 @@ struct TypeTypeof : TypeQualified
     Type *syntaxCopy();
     Dsymbol *toDsymbol(Scope *sc);
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
+    void toDecoBuffer(OutBuffer *buf);
     Type *semantic(Loc loc, Scope *sc);
     d_uns64 size(Loc loc);
 };
@@ -540,6 +543,7 @@ struct TypeStruct : Type
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     unsigned memalign(unsigned salign);
     Expression *defaultInit(Loc loc);
+    Expression *defaultInitLiteral(Loc loc);
     int isZeroInit(Loc loc);
     int checkBoolean();
     dt_t **toDt(dt_t **pdt);
@@ -576,6 +580,9 @@ struct TypeEnum : Type
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
     TypeInfoDeclaration *getTypeInfoDeclaration();
     int hasPointers();
+#if CPP_MANGLE
+    void toCppMangle(OutBuffer *buf, CppMangleState *cms);
+#endif
 
     type *toCtype();
 };
@@ -644,7 +651,7 @@ struct TypeClass : Type
 #if DMDV2
     Type *toHeadMutable();
     MATCH constConv(Type *to);
-#if TARGET_LINUX || TARGET_OSX
+#if CPP_MANGLE
     void toCppMangle(OutBuffer *buf, CppMangleState *cms);
 #endif
 #endif
@@ -656,9 +663,9 @@ struct TypeClass : Type
 
 struct TypeTuple : Type
 {
-    Arguments *arguments;	// types making up the tuple
+    Parameters *arguments;      // types making up the tuple
 
-    TypeTuple(Arguments *arguments);
+    TypeTuple(Parameters *arguments);
     TypeTuple(Expressions *exps);
     Type *syntaxCopy();
     Type *semantic(Loc loc, Scope *sc);
@@ -686,24 +693,26 @@ struct TypeSlice : Type
 
 //enum InOut { None, In, Out, InOut, Lazy };
 
-struct Argument : Object
+struct Parameter : Object
 {
     //enum InOut inout;
-    unsigned storageClass;
+    StorageClass storageClass;
     Type *type;
     Identifier *ident;
     Expression *defaultArg;
 
-    Argument(unsigned storageClass, Type *type, Identifier *ident, Expression *defaultArg);
-    Argument *syntaxCopy();
+    Parameter(StorageClass storageClass, Type *type, Identifier *ident, Expression *defaultArg);
+    Parameter *syntaxCopy();
     Type *isLazyArray();
     void toDecoBuffer(OutBuffer *buf);
-    static Arguments *arraySyntaxCopy(Arguments *args);
-    static char *argsTypesToChars(Arguments *args, int varargs);
-    static void argsToCBuffer(OutBuffer *buf, HdrGenState *hgs, Arguments *arguments, int varargs);
-    static void argsToDecoBuffer(OutBuffer *buf, Arguments *arguments);
-    static size_t dim(Arguments *arguments);
-    static Argument *getNth(Arguments *arguments, size_t nth, size_t *pn = NULL);
+    static Parameters *arraySyntaxCopy(Parameters *args);
+    static char *argsTypesToChars(Parameters *args, int varargs);
+    static void argsCppMangle(OutBuffer *buf, CppMangleState *cms, Parameters *arguments, int varargs);
+    static void argsToCBuffer(OutBuffer *buf, HdrGenState *hgs, Parameters *arguments, int varargs);
+    static void argsToDecoBuffer(OutBuffer *buf, Parameters *arguments);
+    static int isTPL(Parameters *arguments);
+    static size_t dim(Parameters *arguments);
+    static Parameter *getNth(Parameters *arguments, size_t nth, size_t *pn = NULL);
 };
 
 extern int PTRSIZE;
@@ -714,4 +723,8 @@ extern int Tptrdiff_t;
 
 #endif
 
+#ifndef IN_GCC
+int arrayTypeCompatible(Loc loc, Type *t1, Type *t2);
+#endif
+
 #endif /* DMD_MTYPE_H */
