@@ -38,7 +38,7 @@ struct InlineDoState;
 struct InlineScanState;
 struct ReturnStatement;
 struct CompoundStatement;
-struct Argument;
+struct Parameter;
 struct StaticAssert;
 struct AsmStatement;
 struct GotoStatement;
@@ -63,19 +63,19 @@ struct elem;
 struct code;
 
 /* How a statement exits; this is returned by blockExit()
-+  */
- enum BE
- {
-     BEnone =	 0,
-     BEfallthru = 1,
-     BEthrow =    2,
-     BEreturn =   4,
-     BEgoto =     8,
-     BEhalt =	 0x10,
-     BEbreak =	 0x20,
-     BEcontinue = 0x40,
-     BEany = (BEfallthru | BEthrow | BEreturn | BEgoto | BEhalt),
- };
+ */
+enum BE
+{
+    BEnone =     0,
+    BEfallthru = 1,
+    BEthrow =    2,
+    BEreturn =   4,
+    BEgoto =     8,
+    BEhalt =     0x10,
+    BEbreak =    0x20,
+    BEcontinue = 0x40,
+    BEany = (BEfallthru | BEthrow | BEreturn | BEgoto | BEhalt),
+};
 
 struct Statement : Object
 {
@@ -104,7 +104,8 @@ struct Statement : Object
     virtual int usesEH();
     virtual int blockExit();
     virtual int comeFrom();
-    virtual void scopeCode(Statement **sentry, Statement **sexit, Statement **sfinally);
+    virtual int isEmpty();
+    virtual void scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
     virtual Statements *flatten(Scope *sc);
     virtual Expression *interpret(InterState *istate);
 
@@ -122,6 +123,14 @@ struct Statement : Object
     virtual IfStatement *isIfStatement() { return NULL; }
 };
 
+struct PeelStatement : Statement
+{
+    Statement *s;
+
+    PeelStatement(Statement *s);
+    Statement *semantic(Scope *sc);
+};
+
 struct ExpStatement : Statement
 {
     Expression *exp;
@@ -132,6 +141,7 @@ struct ExpStatement : Statement
     Statement *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
     int blockExit();
+    int isEmpty();
 
     int inlineCost(InlineCostState *ics);
     Expression *doInline(InlineDoState *ids);
@@ -160,7 +170,7 @@ struct DeclarationStatement : ExpStatement
     DeclarationStatement(Loc loc, Expression *exp);
     Statement *syntaxCopy();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-    void scopeCode(Statement **sentry, Statement **sexit, Statement **sfinally);
+    void scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
 
     DeclarationStatement *isDeclarationStatement() { return this; }
 };
@@ -177,6 +187,7 @@ struct CompoundStatement : Statement
     int usesEH();
     int blockExit();
     int comeFrom();
+    int isEmpty();
     Statements *flatten(Scope *sc);
     ReturnStatement *isReturnStatement();
     Expression *interpret(InterState *istate);
@@ -236,6 +247,7 @@ struct ScopeStatement : Statement
     int usesEH();
     int blockExit();
     int comeFrom();
+    int isEmpty();
     Expression *interpret(InterState *istate);
 
     Statement *inlineScan(InlineScanState *iss);
@@ -295,7 +307,7 @@ struct ForStatement : Statement
     ForStatement(Loc loc, Statement *init, Expression *condition, Expression *increment, Statement *body);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
-    void scopeCode(Statement **sentry, Statement **sexit, Statement **sfinally);
+    void scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
     int hasBreak();
     int hasContinue();
     int usesEH();
@@ -311,22 +323,23 @@ struct ForStatement : Statement
 
 struct ForeachStatement : Statement
 {
-    enum TOK op;	// TOKforeach or TOKforeach_reverse
-    Arguments *arguments;	// array of Argument*'s
+    enum TOK op;                // TOKforeach or TOKforeach_reverse
+    Parameters *arguments;      // array of Parameter*'s
     Expression *aggr;
     Statement *body;
 
     VarDeclaration *key;
     VarDeclaration *value;
 
-    FuncDeclaration *func;	// function we're lexically in
+    FuncDeclaration *func;      // function we're lexically in
 
-    Array cases;	// put breaks, continues, gotos and returns here
-    Array gotos;	// forward referenced goto's go here
+    Array cases;        // put breaks, continues, gotos and returns here
+    Array gotos;        // forward referenced goto's go here
 
-    ForeachStatement(Loc loc, enum TOK op, Arguments *arguments, Expression *aggr, Statement *body);
+    ForeachStatement(Loc loc, enum TOK op, Parameters *arguments, Expression *aggr, Statement *body);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
+    bool checkForArgTypes();
     int hasBreak();
     int hasContinue();
     int usesEH();
@@ -341,45 +354,45 @@ struct ForeachStatement : Statement
 };
 
 #if DMDV2
- struct ForeachRangeStatement : Statement
- {
-     enum TOK op;		// TOKforeach or TOKforeach_reverse
-     Argument *arg;		// loop index variable
-     Expression *lwr;
-     Expression *upr;
-     Statement *body;
- 
-     VarDeclaration *key;
- 
-     ForeachRangeStatement(Loc loc, enum TOK op, Argument *arg,
- 	Expression *lwr, Expression *upr, Statement *body);
-     Statement *syntaxCopy();
-     Statement *semantic(Scope *sc);
-     int hasBreak();
-     int hasContinue();
-     int usesEH();
-     int blockExit();
+struct ForeachRangeStatement : Statement
+{
+    enum TOK op;                // TOKforeach or TOKforeach_reverse
+    Parameter *arg;             // loop index variable
+    Expression *lwr;
+    Expression *upr;
+    Statement *body;
 
-     int comeFrom();
-     Expression *interpret(InterState *istate);
-     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
- 
-     Statement *inlineScan(InlineScanState *iss);
- 
-     void toIR(IRState *irs);
- };
- #endif
+    VarDeclaration *key;
+
+    ForeachRangeStatement(Loc loc, enum TOK op, Parameter *arg,
+        Expression *lwr, Expression *upr, Statement *body);
+    Statement *syntaxCopy();
+    Statement *semantic(Scope *sc);
+    int hasBreak();
+    int hasContinue();
+    int usesEH();
+    int blockExit();
+
+    int comeFrom();
+    Expression *interpret(InterState *istate);
+    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+
+    Statement *inlineScan(InlineScanState *iss);
+
+    void toIR(IRState *irs);
+};
+#endif
 
 struct IfStatement : Statement
 {
-    Argument *arg;
+    Parameter *arg;
     Expression *condition;
     Statement *ifbody;
     Statement *elsebody;
 
-    VarDeclaration *match;	// for MatchExpression results
+    VarDeclaration *match;      // for MatchExpression results
 
-    IfStatement(Loc loc, Argument *arg, Expression *condition, Statement *ifbody, Statement *elsebody);
+    IfStatement(Loc loc, Parameter *arg, Expression *condition, Statement *ifbody, Statement *elsebody);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
@@ -414,7 +427,7 @@ struct ConditionalStatement : Statement
 struct PragmaStatement : Statement
 {
     Identifier *ident;
-    Expressions *args;		// array of Expression's
+    Expressions *args;          // array of Expression's
     Statement *body;
 
     PragmaStatement(Loc loc, Identifier *ident, Expressions *args, Statement *body);
@@ -433,6 +446,7 @@ struct StaticAssertStatement : Statement
     StaticAssertStatement(StaticAssert *sa);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
+    int blockExit();
 
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 };
@@ -443,9 +457,9 @@ struct SwitchStatement : Statement
     Statement *body;
     DefaultStatement *sdefault;
 
-    Array gotoCases;		// array of unresolved GotoCaseStatement's
-    Array *cases;		// array of CaseStatement's
-    int hasNoDefault;		// !=0 if no default statement
+    Array gotoCases;            // array of unresolved GotoCaseStatement's
+    Array *cases;               // array of CaseStatement's
+    int hasNoDefault;           // !=0 if no default statement
 
     SwitchStatement(Loc loc, Expression *c, Statement *b);
     Statement *syntaxCopy();
@@ -465,8 +479,9 @@ struct CaseStatement : Statement
 {
     Expression *exp;
     Statement *statement;
-    int index;		// which case it is (since we sort this)
-    block *cblock;	// back end: label for the block
+
+    int index;          // which case it is (since we sort this)
+    block *cblock;      // back end: label for the block
 
     CaseStatement(Loc loc, Expression *exp, Statement *s);
     Statement *syntaxCopy();
@@ -483,11 +498,27 @@ struct CaseStatement : Statement
     void toIR(IRState *irs);
 };
 
+#if DMDV2
+
+struct CaseRangeStatement : Statement
+{
+    Expression *first;
+    Expression *last;
+    Statement *statement;
+
+    CaseRangeStatement(Loc loc, Expression *first, Expression *last, Statement *s);
+    Statement *syntaxCopy();
+    Statement *semantic(Scope *sc);
+    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+};
+
+#endif
+
 struct DefaultStatement : Statement
 {
     Statement *statement;
 #if IN_GCC
-    block *cblock;	// back end: label for the block
+    block *cblock;      // back end: label for the block
 #endif
 
     DefaultStatement(Loc loc, Statement *s);
@@ -520,8 +551,8 @@ struct GotoDefaultStatement : Statement
 
 struct GotoCaseStatement : Statement
 {
-    Expression *exp;		// NULL, or which case to goto
-    CaseStatement *cs;		// case statement it resolves to
+    Expression *exp;            // NULL, or which case to goto
+    CaseStatement *cs;          // case statement it resolves to
 
     GotoCaseStatement(Loc loc, Expression *exp);
     Statement *syntaxCopy();
@@ -624,6 +655,7 @@ struct WithStatement : Statement
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     int usesEH();
     int blockExit();
+    Expression *interpret(InterState *istate);
 
     Statement *inlineScan(InlineScanState *iss);
 
@@ -641,6 +673,7 @@ struct TryCatchStatement : Statement
     int hasBreak();
     int usesEH();
     int blockExit();
+    Expression *interpret(InterState *istate);
 
     Statement *inlineScan(InlineScanState *iss);
 
@@ -677,6 +710,7 @@ struct TryFinallyStatement : Statement
     int hasContinue();
     int usesEH();
     int blockExit();
+    Expression *interpret(InterState *istate);
 
     Statement *inlineScan(InlineScanState *iss);
 
@@ -694,7 +728,8 @@ struct OnScopeStatement : Statement
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Statement *semantic(Scope *sc);
     int usesEH();
-    void scopeCode(Statement **sentry, Statement **sexit, Statement **sfinally);
+    void scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
+    Expression *interpret(InterState *istate);
 
     void toIR(IRState *irs);
 };
@@ -708,6 +743,7 @@ struct ThrowStatement : Statement
     Statement *semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     int blockExit();
+    Expression *interpret(InterState *istate);
 
     Statement *inlineScan(InlineScanState *iss);
 
@@ -752,7 +788,7 @@ struct LabelStatement : Statement
     Identifier *ident;
     Statement *statement;
     TryFinallyStatement *tf;
-    block *lblock;		// back end
+    block *lblock;              // back end
     int isReturnLabel;
 
     LabelStatement(Loc loc, Identifier *ident, Statement *statement);
@@ -785,16 +821,17 @@ struct AsmStatement : Statement
 {
     Token *tokens;
     code *asmcode;
-    unsigned asmalign;		// alignment of this statement
-    unsigned refparam;		// !=0 if function parameter is referenced
-    unsigned naked;		// !=0 if function is to be naked
-    unsigned regs;		// mask of registers modified
+    unsigned asmalign;          // alignment of this statement
+    unsigned refparam;          // !=0 if function parameter is referenced
+    unsigned naked;             // !=0 if function is to be naked
+    unsigned regs;              // mask of registers modified
 
     AsmStatement(Loc loc, Token *tokens);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     int blockExit();
     int comeFrom();
+    Expression *interpret(InterState *istate);
 
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     virtual AsmStatement *isAsmStatement() { return this; }
@@ -809,16 +846,16 @@ struct AsmStatement : Statement
 struct ExtAsmStatement : Statement
 {
     Expression *insnTemplate;
-    Expressions *args;            
+    Expressions *args;
     Array *argNames;        // of NULL or Identifier*
     Expressions *argConstraints;  // of StringExp*
     unsigned nOutputArgs;
     Expressions *clobbers;        // of StringExp*
-    
+
     int blockExit();
 
     ExtAsmStatement(Loc loc, Expression *insnTemplate, Expressions *args, Array *argNames,
-	Expressions *argConstraints, int nOutputArgs, Expressions *clobbers);
+        Expressions *argConstraints, int nOutputArgs, Expressions *clobbers);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     void toIR(IRState *irs);
