@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2009 by Digital Mars
+// Copyright (c) 1999-2010 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -33,8 +33,11 @@
 #include "import.h"
 #include "aggregate.h"
 
+#ifndef TARGET_NET
 #include "rmem.h"
+#endif
 #ifndef IN_GCC
+#ifndef TARGET_NET
 #include "cc.h"
 #include "global.h"
 #include "oper.h"
@@ -43,6 +46,7 @@
 #include "dt.h"
 #include "cgcv.h"
 #include "outbuf.h"
+#endif // TARGET_NET
 #include "irstate.h"
 #else
 #include "symbol.h"
@@ -70,38 +74,38 @@ Expression *Type::getInternalTypeInfo(Scope *sc)
     t = toBasetype();
     switch (t->ty)
     {
-	case Tsarray:
+        case Tsarray:
 #if 0
-	    t = t->next->arrayOf();	// convert to corresponding dynamic array type
+            t = t->next->arrayOf();     // convert to corresponding dynamic array type
 #endif
-	    break;
+            break;
 
-	case Tclass:
-	    if (((TypeClass *)t)->sym->isInterfaceDeclaration())
-		break;
-	    goto Linternal;
+        case Tclass:
+            if (((TypeClass *)t)->sym->isInterfaceDeclaration())
+                break;
+            goto Linternal;
 
-	case Tarray:
-	    if (t->next->ty != Tclass)
-		break;
-	    goto Linternal;
+        case Tarray:
+            if (t->next->ty != Tclass)
+                break;
+            goto Linternal;
 
-	case Tfunction:
-	case Tdelegate:
-	case Tpointer:
-	Linternal:
-	    tid = internalTI[t->ty];
-	    if (!tid)
-	    {	tid = new TypeInfoDeclaration(t, 1);
-		internalTI[t->ty] = tid;
-	    }
-	    e = new VarExp(0, tid);
-	    e = e->addressOf(sc);
-	    e->type = tid->type;	// do this so we don't get redundant dereference
-	    return e;
+        case Tfunction:
+        case Tdelegate:
+        case Tpointer:
+        Linternal:
+            tid = internalTI[t->ty];
+            if (!tid)
+            {   tid = new TypeInfoDeclaration(t, 1);
+                internalTI[t->ty] = tid;
+            }
+            e = new VarExp(0, tid);
+            e = e->addressOf(sc);
+            e->type = tid->type;        // do this so we don't get redundant dereference
+            return e;
 
-	default:
-	    break;
+        default:
+            break;
     }
     //printf("\tcalling getTypeInfo() %s\n", t->toChars());
     return t->getTypeInfo(sc);
@@ -114,45 +118,48 @@ Expression *Type::getInternalTypeInfo(Scope *sc)
 
 Expression *Type::getTypeInfo(Scope *sc)
 {
-    Expression *e;
-    Type *t;
-
     //printf("Type::getTypeInfo() %p, %s\n", this, toChars());
-    t = merge2();	// do this since not all Type's are merge'd
+    if (!Type::typeinfo)
+    {
+        error(0, "TypeInfo not found. object.d may be incorrectly installed or corrupt, compile with -v switch");
+        fatal();
+    }
+
+    Type *t = merge2(); // do this since not all Type's are merge'd
     if (!t->vtinfo)
     {
 #if DMDV2
-	if (t->isShared())	// does both 'shared' and 'shared const'
-	    t->vtinfo = new TypeInfoSharedDeclaration(t);
-	else if (t->isConst())
-	    t->vtinfo = new TypeInfoConstDeclaration(t);
-	else if (t->isInvariant())
-	    t->vtinfo = new TypeInfoInvariantDeclaration(t);
-	else
+        if (t->isShared())      // does both 'shared' and 'shared const'
+            t->vtinfo = new TypeInfoSharedDeclaration(t);
+        else if (t->isConst())
+            t->vtinfo = new TypeInfoConstDeclaration(t);
+        else if (t->isImmutable())
+            t->vtinfo = new TypeInfoInvariantDeclaration(t);
+        else
 #endif
-	    t->vtinfo = t->getTypeInfoDeclaration();
-	assert(t->vtinfo);
-	vtinfo = t->vtinfo;
+            t->vtinfo = t->getTypeInfoDeclaration();
+        assert(t->vtinfo);
+        vtinfo = t->vtinfo;
 
-	/* If this has a custom implementation in std/typeinfo, then
-	 * do not generate a COMDAT for it.
-	 */
-	if (!t->builtinTypeInfo())
-	{   // Generate COMDAT
-	    if (sc)			// if in semantic() pass
-	    {	// Find module that will go all the way to an object file
-		Module *m = sc->module->importedFrom;
-		m->members->push(t->vtinfo);
-	    }
-	    else			// if in obj generation pass
-	    {
-		t->vtinfo->toObjFile(global.params.multiobj);
-	    }
-	}
+        /* If this has a custom implementation in std/typeinfo, then
+         * do not generate a COMDAT for it.
+         */
+        if (!t->builtinTypeInfo())
+        {   // Generate COMDAT
+            if (sc)                     // if in semantic() pass
+            {   // Find module that will go all the way to an object file
+                Module *m = sc->module->importedFrom;
+                m->members->push(t->vtinfo);
+            }
+            else                        // if in obj generation pass
+            {
+                t->vtinfo->toObjFile(global.params.multiobj);
+            }
+        }
     }
-    e = new VarExp(0, t->vtinfo);
+    Expression *e = new VarExp(0, t->vtinfo);
     e = e->addressOf(sc);
-    e->type = t->vtinfo->type;		// do this so we don't get redundant dereference
+    e->type = t->vtinfo->type;          // do this so we don't get redundant dereference
     return e;
 }
 
@@ -195,9 +202,9 @@ TypeInfoDeclaration *TypeStruct::getTypeInfoDeclaration()
 TypeInfoDeclaration *TypeClass::getTypeInfoDeclaration()
 {
     if (sym->isInterfaceDeclaration())
-	return new TypeInfoInterfaceDeclaration(this);
+        return new TypeInfoInterfaceDeclaration(this);
     else
-	return new TypeInfoClassDeclaration(this);
+        return new TypeInfoClassDeclaration(this);
 }
 
 TypeInfoDeclaration *TypeEnum::getTypeInfoDeclaration()
@@ -220,7 +227,7 @@ TypeInfoDeclaration *TypeTuple::getTypeInfoDeclaration()
     return new TypeInfoTupleDeclaration(this);
 }
 
-
+#ifndef TARGET_NET
 /****************************************************
  */
 
@@ -230,46 +237,46 @@ void TypeInfoDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoDeclaration::toDt() %s\n", toChars());
     dtxoff(pdt, Type::typeinfo->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo
-    dtdword(pdt, 0);			    // monitor
+    dtdword(pdt, 0);                        // monitor
 }
 
 #if DMDV2
- void TypeInfoConstDeclaration::toDt(dt_t **pdt)
- {
-     //printf("TypeInfoConstDeclaration::toDt() %s\n", toChars());
-     dtxoff(pdt, Type::typeinfoconst->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Const
-     dtdword(pdt, 0);			    // monitor
-     Type *tm = tinfo->mutableOf();
-     tm = tm->merge();
-     tm->getTypeInfo(NULL);
-     dtxoff(pdt, tm->vtinfo->toSymbol(), 0, TYnptr);
- }
- 
- void TypeInfoInvariantDeclaration::toDt(dt_t **pdt)
- {
-     //printf("TypeInfoInvariantDeclaration::toDt() %s\n", toChars());
-     dtxoff(pdt, Type::typeinfoinvariant->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Invariant
-     dtdword(pdt, 0);			    // monitor
-     Type *tm = tinfo->mutableOf();
-     tm = tm->merge();
-     tm->getTypeInfo(NULL);
-     dtxoff(pdt, tm->vtinfo->toSymbol(), 0, TYnptr);
- }
- 
- void TypeInfoSharedDeclaration::toDt(dt_t **pdt)
- {
-     //printf("TypeInfoSharedDeclaration::toDt() %s\n", toChars());
-     dtxoff(pdt, Type::typeinfoshared->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Shared
-     dtdword(pdt, 0);			    // monitor
-     Type *tm;
-     if (tinfo->isConst())		// it was 'shared const'
- 	tm = tinfo->constOf();
-     else				// it was just 'shared'
- 	tm = tinfo->mutableOf();
-     tm = tm->merge();
+void TypeInfoConstDeclaration::toDt(dt_t **pdt)
+{
+    //printf("TypeInfoConstDeclaration::toDt() %s\n", toChars());
+    dtxoff(pdt, Type::typeinfoconst->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Const
+    dtdword(pdt, 0);                        // monitor
+    Type *tm = tinfo->mutableOf();
+    tm = tm->merge();
     tm->getTypeInfo(NULL);
-     dtxoff(pdt, tm->vtinfo->toSymbol(), 0, TYnptr);
- }
+    dtxoff(pdt, tm->vtinfo->toSymbol(), 0, TYnptr);
+}
+
+void TypeInfoInvariantDeclaration::toDt(dt_t **pdt)
+{
+    //printf("TypeInfoInvariantDeclaration::toDt() %s\n", toChars());
+    dtxoff(pdt, Type::typeinfoinvariant->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Invariant
+    dtdword(pdt, 0);                        // monitor
+    Type *tm = tinfo->mutableOf();
+    tm = tm->merge();
+    tm->getTypeInfo(NULL);
+    dtxoff(pdt, tm->vtinfo->toSymbol(), 0, TYnptr);
+}
+
+void TypeInfoSharedDeclaration::toDt(dt_t **pdt)
+{
+    //printf("TypeInfoSharedDeclaration::toDt() %s\n", toChars());
+    dtxoff(pdt, Type::typeinfoshared->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Shared
+    dtdword(pdt, 0);                        // monitor
+    Type *tm;
+    if (tinfo->isConst())               // it was 'shared const'
+        tm = tinfo->constOf();
+    else                                // it was just 'shared'
+        tm = tinfo->mutableOf();
+    tm = tm->merge();
+    tm->getTypeInfo(NULL);
+    dtxoff(pdt, tm->vtinfo->toSymbol(), 0, TYnptr);
+}
 #endif
 
 void TypeInfoTypedefDeclaration::toDt(dt_t **pdt)
@@ -277,7 +284,7 @@ void TypeInfoTypedefDeclaration::toDt(dt_t **pdt)
     //printf("TypeInfoTypedefDeclaration::toDt() %s\n", toChars());
 
     dtxoff(pdt, Type::typeinfotypedef->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Typedef
-    dtdword(pdt, 0);			    // monitor
+    dtdword(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Ttypedef);
 
@@ -286,31 +293,31 @@ void TypeInfoTypedefDeclaration::toDt(dt_t **pdt)
     //printf("basetype = %s\n", sd->basetype->toChars());
 
     /* Put out:
-     *	TypeInfo base;
-     *	char[] name;
-     *	void[] m_init;
+     *  TypeInfo base;
+     *  char[] name;
+     *  void[] m_init;
      */
 
     sd->basetype = sd->basetype->merge();
-    sd->basetype->getTypeInfo(NULL);		// generate vtinfo
+    sd->basetype->getTypeInfo(NULL);            // generate vtinfo
     assert(sd->basetype->vtinfo);
-    dtxoff(pdt, sd->basetype->vtinfo->toSymbol(), 0, TYnptr);	// TypeInfo for basetype
+    dtxoff(pdt, sd->basetype->vtinfo->toSymbol(), 0, TYnptr);   // TypeInfo for basetype
 
-    char *name = sd->toPrettyChars();
+    const char *name = sd->toPrettyChars();
     size_t namelen = strlen(name);
     dtdword(pdt, namelen);
     dtabytes(pdt, TYnptr, 0, namelen + 1, name);
 
     // void[] init;
     if (tinfo->isZeroInit() || !sd->init)
-    {	// 0 initializer, or the same as the base type
-	dtdword(pdt, 0);	// init.length
-	dtdword(pdt, 0);	// init.ptr
+    {   // 0 initializer, or the same as the base type
+        dtdword(pdt, 0);        // init.length
+        dtdword(pdt, 0);        // init.ptr
     }
     else
     {
-	dtdword(pdt, sd->type->size());	// init.length
-	dtxoff(pdt, sd->toInitializer(), 0, TYnptr);	// init.ptr
+        dtdword(pdt, sd->type->size()); // init.length
+        dtxoff(pdt, sd->toInitializer(), 0, TYnptr);    // init.ptr
     }
 }
 
@@ -318,7 +325,7 @@ void TypeInfoEnumDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoEnumDeclaration::toDt()\n");
     dtxoff(pdt, Type::typeinfoenum->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Enum
-    dtdword(pdt, 0);			    // monitor
+    dtdword(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tenum);
 
@@ -326,29 +333,29 @@ void TypeInfoEnumDeclaration::toDt(dt_t **pdt)
     EnumDeclaration *sd = tc->sym;
 
     /* Put out:
-     *	TypeInfo base;
-     *	char[] name;
-     *	void[] m_init;
+     *  TypeInfo base;
+     *  char[] name;
+     *  void[] m_init;
      */
 
     sd->memtype->getTypeInfo(NULL);
-    dtxoff(pdt, sd->memtype->vtinfo->toSymbol(), 0, TYnptr);	// TypeInfo for enum members
+    dtxoff(pdt, sd->memtype->vtinfo->toSymbol(), 0, TYnptr);    // TypeInfo for enum members
 
-    char *name = sd->toPrettyChars();
+    const char *name = sd->toPrettyChars();
     size_t namelen = strlen(name);
     dtdword(pdt, namelen);
     dtabytes(pdt, TYnptr, 0, namelen + 1, name);
 
     // void[] init;
     if (!sd->defaultval || tinfo->isZeroInit())
-    {	// 0 initializer, or the same as the base type
-	dtdword(pdt, 0);	// init.length
-	dtdword(pdt, 0);	// init.ptr
+    {   // 0 initializer, or the same as the base type
+        dtdword(pdt, 0);        // init.length
+        dtdword(pdt, 0);        // init.ptr
     }
     else
     {
-	dtdword(pdt, sd->type->size());	// init.length
-	dtxoff(pdt, sd->toInitializer(), 0, TYnptr);	// init.ptr
+        dtdword(pdt, sd->type->size()); // init.length
+        dtxoff(pdt, sd->toInitializer(), 0, TYnptr);    // init.ptr
     }
 }
 
@@ -356,7 +363,7 @@ void TypeInfoPointerDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoPointerDeclaration::toDt()\n");
     dtxoff(pdt, Type::typeinfopointer->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Pointer
-    dtdword(pdt, 0);			    // monitor
+    dtdword(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tpointer);
 
@@ -370,7 +377,7 @@ void TypeInfoArrayDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoArrayDeclaration::toDt()\n");
     dtxoff(pdt, Type::typeinfoarray->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Array
-    dtdword(pdt, 0);			    // monitor
+    dtdword(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tarray);
 
@@ -384,7 +391,7 @@ void TypeInfoStaticArrayDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoStaticArrayDeclaration::toDt()\n");
     dtxoff(pdt, Type::typeinfostaticarray->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_StaticArray
-    dtdword(pdt, 0);			    // monitor
+    dtdword(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tsarray);
 
@@ -393,14 +400,14 @@ void TypeInfoStaticArrayDeclaration::toDt(dt_t **pdt)
     tc->next->getTypeInfo(NULL);
     dtxoff(pdt, tc->next->vtinfo->toSymbol(), 0, TYnptr); // TypeInfo for array of type
 
-    dtdword(pdt, tc->dim->toInteger());		// length
+    dtdword(pdt, tc->dim->toInteger());         // length
 }
 
 void TypeInfoAssociativeArrayDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoAssociativeArrayDeclaration::toDt()\n");
     dtxoff(pdt, Type::typeinfoassociativearray->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_AssociativeArray
-    dtdword(pdt, 0);			    // monitor
+    dtdword(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Taarray);
 
@@ -417,7 +424,7 @@ void TypeInfoFunctionDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoFunctionDeclaration::toDt()\n");
     dtxoff(pdt, Type::typeinfofunction->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Function
-    dtdword(pdt, 0);			    // monitor
+    dtdword(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tfunction);
 
@@ -431,14 +438,14 @@ void TypeInfoDelegateDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoDelegateDeclaration::toDt()\n");
     dtxoff(pdt, Type::typeinfodelegate->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Delegate
-    dtdword(pdt, 0);			    // monitor
+    dtdword(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tdelegate);
 
     TypeDelegate *tc = (TypeDelegate *)tinfo;
 
-    tc->next->next->getTypeInfo(NULL);
-    dtxoff(pdt, tc->next->next->vtinfo->toSymbol(), 0, TYnptr); // TypeInfo for delegate return value
+    tc->next->nextOf()->getTypeInfo(NULL);
+    dtxoff(pdt, tc->next->nextOf()->vtinfo->toSymbol(), 0, TYnptr); // TypeInfo for delegate return value
 }
 
 void TypeInfoStructDeclaration::toDt(dt_t **pdt)
@@ -448,7 +455,7 @@ void TypeInfoStructDeclaration::toDt(dt_t **pdt)
     unsigned offset = Type::typeinfostruct->structsize;
 
     dtxoff(pdt, Type::typeinfostruct->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Struct
-    dtdword(pdt, 0);			    // monitor
+    dtdword(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tstruct);
 
@@ -456,18 +463,18 @@ void TypeInfoStructDeclaration::toDt(dt_t **pdt)
     StructDeclaration *sd = tc->sym;
 
     /* Put out:
-     *	char[] name;
-     *	void[] init;
-     *	hash_t function(void*) xtoHash;
-     *	int function(void*,void*) xopEquals;
-     *	int function(void*,void*) xopCmp;
-     *	char[] function(void*) xtoString;
-     *	uint m_flags;
+     *  char[] name;
+     *  void[] init;
+     *  hash_t function(void*) xtoHash;
+     *  int function(void*,void*) xopEquals;
+     *  int function(void*,void*) xopCmp;
+     *  char[] function(void*) xtoString;
+     *  uint m_flags;
      *
-     *	name[]
+     *  name[]
      */
 
-    char *name = sd->toPrettyChars();
+    const char *name = sd->toPrettyChars();
     size_t namelen = strlen(name);
     dtdword(pdt, namelen);
     //dtabytes(pdt, TYnptr, 0, namelen + 1, name);
@@ -475,11 +482,11 @@ void TypeInfoStructDeclaration::toDt(dt_t **pdt)
     offset += namelen + 1;
 
     // void[] init;
-    dtdword(pdt, sd->structsize);	// init.length
+    dtdword(pdt, sd->structsize);       // init.length
     if (sd->zeroInit)
-	dtdword(pdt, 0);		// NULL for 0 initialization
+        dtdword(pdt, 0);                // NULL for 0 initialization
     else
-	dtxoff(pdt, sd->toInitializer(), 0, TYnptr);	// init.ptr
+        dtxoff(pdt, sd->toInitializer(), 0, TYnptr);    // init.ptr
 
     FuncDeclaration *fd;
     FuncDeclaration *fdx;
@@ -492,109 +499,116 @@ void TypeInfoStructDeclaration::toDt(dt_t **pdt)
 
     if (!tftohash)
     {
-	Scope sc;
+        Scope sc;
 
-	tftohash = new TypeFunction(NULL, Type::thash_t, 0, LINKd);
-	tftohash = (TypeFunction *)tftohash->semantic(0, &sc);
+        tftohash = new TypeFunction(NULL, Type::thash_t, 0, LINKd);
+        tftohash = (TypeFunction *)tftohash->semantic(0, &sc);
 
-	tftostring = new TypeFunction(NULL, Type::tchar->arrayOf(), 0, LINKd);
-	tftostring = (TypeFunction *)tftostring->semantic(0, &sc);
+        tftostring = new TypeFunction(NULL, Type::tchar->arrayOf(), 0, LINKd);
+        tftostring = (TypeFunction *)tftostring->semantic(0, &sc);
     }
 
     TypeFunction *tfeqptr;
     {
-	Scope sc;
-	Arguments *arguments = new Arguments;
-	Argument *arg = new Argument(STCin, tc->pointerTo(), NULL, NULL);
+        Scope sc;
+        Parameters *arguments = new Parameters;
+#if STRUCTTHISREF
+        // arg type is ref const T
+        Parameter *arg = new Parameter(STCref, tc->constOf(), NULL, NULL);
+#else
+        // arg type is const T*
+        Parameter *arg = new Parameter(STCin, tc->pointerTo(), NULL, NULL);
+#endif
 
-	arguments->push(arg);
-	tfeqptr = new TypeFunction(arguments, Type::tint32, 0, LINKd);
-	tfeqptr = (TypeFunction *)tfeqptr->semantic(0, &sc);
+        arguments->push(arg);
+        tfeqptr = new TypeFunction(arguments, Type::tint32, 0, LINKd);
+        tfeqptr = (TypeFunction *)tfeqptr->semantic(0, &sc);
     }
 
 #if 0
     TypeFunction *tfeq;
     {
-	Scope sc;
-	Array *arguments = new Array;
-	Argument *arg = new Argument(In, tc, NULL, NULL);
+        Scope sc;
+        Array *arguments = new Array;
+        Parameter *arg = new Parameter(In, tc, NULL, NULL);
 
-	arguments->push(arg);
-	tfeq = new TypeFunction(arguments, Type::tint32, 0, LINKd);
-	tfeq = (TypeFunction *)tfeq->semantic(0, &sc);
+        arguments->push(arg);
+        tfeq = new TypeFunction(arguments, Type::tint32, 0, LINKd);
+        tfeq = (TypeFunction *)tfeq->semantic(0, &sc);
     }
 #endif
 
     s = search_function(sd, Id::tohash);
     fdx = s ? s->isFuncDeclaration() : NULL;
     if (fdx)
-    {	fd = fdx->overloadExactMatch(tftohash);
-	if (fd)
-	    dtxoff(pdt, fd->toSymbol(), 0, TYnptr);
-	else
-	    //fdx->error("must be declared as extern (D) uint toHash()");
-	    dtdword(pdt, 0);
+    {   fd = fdx->overloadExactMatch(tftohash);
+        if (fd)
+            dtxoff(pdt, fd->toSymbol(), 0, TYnptr);
+        else
+            //fdx->error("must be declared as extern (D) uint toHash()");
+            dtdword(pdt, 0);
     }
     else
-	dtdword(pdt, 0);
+        dtdword(pdt, 0);
 
     s = search_function(sd, Id::eq);
     fdx = s ? s->isFuncDeclaration() : NULL;
     for (int i = 0; i < 2; i++)
     {
-	if (fdx)
-	{   fd = fdx->overloadExactMatch(tfeqptr);
-	    if (fd)
-		dtxoff(pdt, fd->toSymbol(), 0, TYnptr);
-	    else
-		//fdx->error("must be declared as extern (D) int %s(%s*)", fdx->toChars(), sd->toChars());
-		dtdword(pdt, 0);
-	}
-	else
-	    dtdword(pdt, 0);
+        if (fdx)
+        {   fd = fdx->overloadExactMatch(tfeqptr);
+            if (fd)
+                dtxoff(pdt, fd->toSymbol(), 0, TYnptr);
+            else
+                //fdx->error("must be declared as extern (D) int %s(%s*)", fdx->toChars(), sd->toChars());
+                dtdword(pdt, 0);
+        }
+        else
+            //fdx->error("must be declared as extern (D) int %s(%s*)", fdx->toChars(), sd->toChars());
+            dtdword(pdt, 0);
 
-	s = search_function(sd, Id::cmp);
-	fdx = s ? s->isFuncDeclaration() : NULL;
+        s = search_function(sd, Id::cmp);
+        fdx = s ? s->isFuncDeclaration() : NULL;
     }
 
     s = search_function(sd, Id::tostring);
     fdx = s ? s->isFuncDeclaration() : NULL;
     if (fdx)
-    {	fd = fdx->overloadExactMatch(tftostring);
-	if (fd)
-	    dtxoff(pdt, fd->toSymbol(), 0, TYnptr);
-	else
-	    //fdx->error("must be declared as extern (D) char[] toString()");
-	    dtdword(pdt, 0);
+    {   fd = fdx->overloadExactMatch(tftostring);
+        if (fd)
+            dtxoff(pdt, fd->toSymbol(), 0, TYnptr);
+        else
+            //fdx->error("must be declared as extern (D) char[] toString()");
+            dtdword(pdt, 0);
     }
     else
-	dtdword(pdt, 0);
+        dtdword(pdt, 0);
 
     // uint m_flags;
     dti32(pdt, tc->hasPointers(), false);
-    
-     #if DMDV2
-     // xgetMembers
-     FuncDeclaration *sgetmembers = sd->findGetMembers();
-     if (sgetmembers)
- 	dtxoff(pdt, sgetmembers->toSymbol(), 0, TYnptr);
-     else
- 	dtdword(pdt, 0);			// xgetMembers
- 
-     // xdtor
-     FuncDeclaration *sdtor = sd->dtor;
-     if (sdtor)
- 	dtxoff(pdt, sdtor->toSymbol(), 0, TYnptr);
-     else
- 	dtdword(pdt, 0);			// xdtor
- 
-     // xpostblit
-     FuncDeclaration *spostblit = sd->postblit;
-     if (spostblit)
- 	dtxoff(pdt, spostblit->toSymbol(), 0, TYnptr);
-     else
- 	dtdword(pdt, 0);			// xpostblit
- 	#endif
+
+#if DMDV2
+    // xgetMembers
+    FuncDeclaration *sgetmembers = sd->findGetMembers();
+    if (sgetmembers)
+        dtxoff(pdt, sgetmembers->toSymbol(), 0, TYnptr);
+    else
+        dtdword(pdt, 0);                        // xgetMembers
+
+    // xdtor
+    FuncDeclaration *sdtor = sd->dtor;
+    if (sdtor)
+        dtxoff(pdt, sdtor->toSymbol(), 0, TYnptr);
+    else
+        dtdword(pdt, 0);                        // xdtor
+
+    // xpostblit
+    FuncDeclaration *spostblit = sd->postblit;
+    if (spostblit)
+        dtxoff(pdt, spostblit->toSymbol(), 0, TYnptr);
+    else
+        dtdword(pdt, 0);                        // xpostblit
+#endif
 
     // name[]
     dtnbytes(pdt, namelen + 1, name);
@@ -604,7 +618,7 @@ void TypeInfoClassDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoClassDeclaration::toDt() %s\n", tinfo->toChars());
     dtxoff(pdt, Type::typeinfoclass->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfoClass
-    dtdword(pdt, 0);			    // monitor
+    dtdword(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tclass);
 
@@ -612,16 +626,16 @@ void TypeInfoClassDeclaration::toDt(dt_t **pdt)
     Symbol *s;
 
     if (!tc->sym->vclassinfo)
-	tc->sym->vclassinfo = new ClassInfoDeclaration(tc->sym);
+        tc->sym->vclassinfo = new ClassInfoDeclaration(tc->sym);
     s = tc->sym->vclassinfo->toSymbol();
-    dtxoff(pdt, s, 0, TYnptr);		// ClassInfo for tinfo
+    dtxoff(pdt, s, 0, TYnptr);          // ClassInfo for tinfo
 }
 
 void TypeInfoInterfaceDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoInterfaceDeclaration::toDt() %s\n", tinfo->toChars());
     dtxoff(pdt, Type::typeinfointerface->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfoInterface
-    dtdword(pdt, 0);			    // monitor
+    dtdword(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tclass);
 
@@ -629,30 +643,30 @@ void TypeInfoInterfaceDeclaration::toDt(dt_t **pdt)
     Symbol *s;
 
     if (!tc->sym->vclassinfo)
-	tc->sym->vclassinfo = new ClassInfoDeclaration(tc->sym);
+        tc->sym->vclassinfo = new ClassInfoDeclaration(tc->sym);
     s = tc->sym->vclassinfo->toSymbol();
-    dtxoff(pdt, s, 0, TYnptr);		// ClassInfo for tinfo
+    dtxoff(pdt, s, 0, TYnptr);          // ClassInfo for tinfo
 }
 
 void TypeInfoTupleDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoTupleDeclaration::toDt() %s\n", tinfo->toChars());
     dtxoff(pdt, Type::typeinfotypelist->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfoInterface
-    dtdword(pdt, 0);			    // monitor
+    dtdword(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Ttuple);
 
     TypeTuple *tu = (TypeTuple *)tinfo;
 
     size_t dim = tu->arguments->dim;
-    dtdword(pdt, dim);			    // elements.length
+    dtdword(pdt, dim);                      // elements.length
 
     dt_t *d = NULL;
     for (size_t i = 0; i < dim; i++)
-    {	Argument *arg = (Argument *)tu->arguments->data[i];
-	Expression *e = arg->type->getTypeInfo(NULL);
-	e = e->optimize(WANTvalue);
-	e->toDt(&d);
+    {   Parameter *arg = (Parameter *)tu->arguments->data[i];
+        Expression *e = arg->type->getTypeInfo(NULL);
+        e = e->optimize(WANTvalue);
+        e->toDt(&d);
     }
 
     Symbol *s;
@@ -660,7 +674,7 @@ void TypeInfoTupleDeclaration::toDt(dt_t **pdt)
     s->Sdt = d;
     outdata(s);
 
-    dtxoff(pdt, s, 0, TYnptr);		    // elements.ptr
+    dtxoff(pdt, s, 0, TYnptr);              // elements.ptr
 }
 
 void TypeInfoDeclaration::toObjFile(int multiobj)
@@ -673,8 +687,8 @@ void TypeInfoDeclaration::toObjFile(int multiobj)
 
     if (multiobj)
     {
-	obj_append(this);
-	return;
+        obj_append(this);
+        return;
     }
 
     s = toSymbol();
@@ -691,25 +705,26 @@ void TypeInfoDeclaration::toObjFile(int multiobj)
     // See if we can convert a comdat to a comdef,
     // which saves on exe file space.
     if (s->Sclass == SCcomdat &&
-	s->Sdt->dt == DT_azeros &&
-	s->Sdt->DTnext == NULL)
+        s->Sdt->dt == DT_azeros &&
+        s->Sdt->DTnext == NULL)
     {
-	s->Sclass = SCglobal;
-	s->Sdt->dt = DT_common;
+        s->Sclass = SCglobal;
+        s->Sdt->dt = DT_common;
     }
 
 #if ELFOBJ || MACHOBJ // Burton
     if (s->Sdt && s->Sdt->dt == DT_azeros && s->Sdt->DTnext == NULL)
-	s->Sseg = UDATA;
+        s->Sseg = UDATA;
     else
-	s->Sseg = DATA;
+        s->Sseg = DATA;
 #endif
     outdata(s);
     if (isExport())
-	obj_export(s,0);
+        obj_export(s,0);
 }
 
 #endif
+#endif // TARGET_NET
 
 /* ========================================================================= */
 
@@ -724,22 +739,22 @@ int Type::builtinTypeInfo()
 
 int TypeBasic::builtinTypeInfo()
 {
-    #if DMDV2
+#if DMDV2
     return mod ? 0 : 1;
-	 #else
+#else
     return 1;
-	 #endif
+#endif
 }
 
 int TypeDArray::builtinTypeInfo()
 {
-    #if DMDV2
+#if DMDV2
     return !mod && (next->isTypeBasic() != NULL && !next->mod ||
-	 // strings are so common, make them builtin
-	 next->ty == Tchar && next->mod == MODinvariant);
-	 #else
+        // strings are so common, make them builtin
+        next->ty == Tchar && next->mod == MODimmutable);
+#else
     return next->isTypeBasic() != NULL;
-	 #endif
+#endif
 }
 
 int TypeClass::builtinTypeInfo()
@@ -747,7 +762,11 @@ int TypeClass::builtinTypeInfo()
     /* This is statically put out with the ClassInfo, so
      * claim it is built in so it isn't regenerated by each module.
      */
+#if DMDV2
+    return mod ? 0 : 1;
+#else
     return 1;
+#endif
 }
 
 /* ========================================================================= */
@@ -767,16 +786,16 @@ Expression *createTypeInfoArray(Scope *sc, Expression *exps[], int dim)
 
     /* Create the TypeTuple corresponding to the types of args[]
      */
-    Arguments *args = new Arguments;
+    Parameters *args = new Parameters;
     args->setDim(dim);
     for (size_t i = 0; i < dim; i++)
-    {	Argument *arg = new Argument(STCin, exps[i]->type, NULL, NULL);
-	args->data[i] = (void *)arg;
+    {   Parameter *arg = new Parameter(STCin, exps[i]->type, NULL, NULL);
+        args->data[i] = (void *)arg;
     }
     TypeTuple *tup = new TypeTuple(args);
     Expression *e = tup->getTypeInfo(sc);
     e = e->optimize(WANTvalue);
-    assert(e->op == TOKsymoff);		// should be SymOffExp
+    assert(e->op == TOKsymoff);         // should be SymOffExp
 
 #if BREAKABI
     /*
@@ -818,8 +837,8 @@ Expression *createTypeInfoArray(Scope *sc, Expression *exps[], int dim)
     // Generate identifier for _arguments[]
     buf.writestring("_arguments_");
     for (int i = 0; i < dim; i++)
-    {	t = exps[i]->type;
-	t->toDecoBuffer(&buf);
+    {   t = exps[i]->type;
+        t->toDecoBuffer(&buf);
     }
     buf.writeByte(0);
     id = Lexer::idPool((char *)buf.data);
@@ -832,31 +851,31 @@ Expression *createTypeInfoArray(Scope *sc, Expression *exps[], int dim)
     Dsymbol *s = m->symtab->lookup(id);
 
     if (s && s->parent == m)
-    {	// Use existing one
-	v = s->isVarDeclaration();
-	assert(v);
+    {   // Use existing one
+        v = s->isVarDeclaration();
+        assert(v);
     }
     else
-    {	// Generate new one
+    {   // Generate new one
 
-	for (int i = 0; i < dim; i++)
-	{   t = exps[i]->type;
-	    e = t->getTypeInfo(sc);
-	    ai->addInit(new IntegerExp(i), new ExpInitializer(0, e));
-	}
+        for (int i = 0; i < dim; i++)
+        {   t = exps[i]->type;
+            e = t->getTypeInfo(sc);
+            ai->addInit(new IntegerExp(i), new ExpInitializer(0, e));
+        }
 
-	t = Type::typeinfo->type->arrayOf();
-	ai->type = t;
-	v = new VarDeclaration(0, t, id, ai);
-	m->members->push(v);
-	m->symtab->insert(v);
-	sc = sc->push();
-	sc->linkage = LINKc;
-	sc->stc = STCstatic | STCcomdat;
-	ai->semantic(sc, t);
-	v->semantic(sc);
-	v->parent = m;
-	sc = sc->pop();
+        t = Type::typeinfo->type->arrayOf();
+        ai->type = t;
+        v = new VarDeclaration(0, t, id, ai);
+        m->members->push(v);
+        m->symtabInsert(v);
+        sc = sc->push();
+        sc->linkage = LINKc;
+        sc->stc = STCstatic | STCcomdat;
+        ai->semantic(sc, t);
+        v->semantic(sc);
+        v->parent = m;
+        sc = sc->pop();
     }
     e = new VarExp(0, v);
     e = e->semantic(sc);
