@@ -63,6 +63,11 @@ struct AggregateDeclaration : ScopeDsymbol
     NewDeclaration *aggNew;		// allocator
     DeleteDeclaration *aggDelete;	// deallocator
 
+#if V2
+    CtorDeclaration *ctor;
+    CtorDeclaration *defaultCtor;	// default constructor
+#endif
+
     FuncDeclarations dtors;	// Array of destructors
     FuncDeclaration *dtor;	// aggregate destructor
 
@@ -127,7 +132,7 @@ struct StructDeclaration : AggregateDeclaration
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     char *mangle();
-    char *kind();
+    const char *kind();
     int needOpAssign();
     FuncDeclaration *buildOpAssign(Scope *sc);
     FuncDeclaration *buildPostBlit(Scope *sc);
@@ -147,7 +152,7 @@ struct UnionDeclaration : StructDeclaration
 {
     UnionDeclaration(Loc loc, Identifier *id);
     Dsymbol *syntaxCopy(Dsymbol *s);
-    char *kind();
+    const char *kind();
 
     UnionDeclaration *isUnionDeclaration() { return this; }
 };
@@ -181,8 +186,6 @@ struct ClassDeclaration : AggregateDeclaration
     static ClassDeclaration *classinfo;
 
     ClassDeclaration *baseClass;	// NULL only if this is Object
-    CtorDeclaration *ctor;
-    CtorDeclaration *defaultCtor;	// default constructor
     FuncDeclaration *staticCtor;
     FuncDeclaration *staticDtor;
     Array vtbl;				// Array of FuncDeclaration's making up the vtbl[]
@@ -232,7 +235,7 @@ struct ClassDeclaration : AggregateDeclaration
 #endif
     int isAbstract();
     virtual int vtblOffset();
-    char *kind();
+    const char *kind();
     char *mangle();
     void toDocBuffer(OutBuffer *buf);
 
@@ -264,7 +267,7 @@ struct InterfaceDeclaration : ClassDeclaration
     void semantic(Scope *sc);
     int isBaseOf(ClassDeclaration *cd, target_ptrdiff_t *poffset);
     int isBaseOf(BaseClass *bc, target_ptrdiff_t *poffset);
-    char *kind();
+    const char *kind();
     int vtblOffset();
 #if V2
     int isCPPinterface();
