@@ -63,9 +63,9 @@ int AttribDeclaration::addMember(Scope *sc, ScopeDsymbol *sd, int memnum)
     if (d)
     {
     	for (unsigned i = 0; i < d->dim; i++)
-			{   Dsymbol *s = (Dsymbol *)d->data[i];
-    	  	    m |= s->addMember(sc, sd, m | memnum);
-    	  	}
+	{   Dsymbol *s = (Dsymbol *)d->data[i];
+	    m |= s->addMember(sc, sd, m | memnum);
+	}
     }
     return m;
 }
@@ -92,10 +92,10 @@ void AttribDeclaration::semantic2(Scope *sc)
 
     if (d)
     {
-    	for (unsigned i = 0; i < d->dim; i++)
-			{   Dsymbol *s = (Dsymbol *)d->data[i];
-    	  	    s->semantic2(sc);
-    	  	}
+	for (unsigned i = 0; i < d->dim; i++)
+	{   Dsymbol *s = (Dsymbol *)d->data[i];
+	    s->semantic2(sc);
+	}
     }
 }
 
@@ -105,10 +105,10 @@ void AttribDeclaration::semantic3(Scope *sc)
 
     if (d)
     {
-    	for (unsigned i = 0; i < d->dim; i++)
-		{   Dsymbol *s = (Dsymbol *)d->data[i];
-    	  	    s->semantic3(sc);
-    	}
+	for (unsigned i = 0; i < d->dim; i++)
+	{   Dsymbol *s = (Dsymbol *)d->data[i];
+	    s->semantic3(sc);
+	}
     }
 }
 
@@ -118,12 +118,12 @@ void AttribDeclaration::inlineScan()
 
     if (d)
     {
-    	for (unsigned i = 0; i < d->dim; i++)
-		{   Dsymbol *s = (Dsymbol *)d->data[i];
+	for (unsigned i = 0; i < d->dim; i++)
+	{   Dsymbol *s = (Dsymbol *)d->data[i];
 
 	    //printf("AttribDeclaration::inlineScan %s\n", s->toChars());
-			s->inlineScan();
-		}
+	    s->inlineScan();
+	}
     }
 }
 
@@ -131,14 +131,14 @@ void AttribDeclaration::addComment(unsigned char *comment)
 {
     if (comment)
     {
-    	Array *d = include(NULL, NULL);
+	Array *d = include(NULL, NULL);
 
 	if (d)
 	{
-		for (unsigned i = 0; i < d->dim; i++)
-		{   Dsymbol *s = (Dsymbol *)d->data[i];
+	    for (unsigned i = 0; i < d->dim; i++)
+	    {   Dsymbol *s = (Dsymbol *)d->data[i];
 		//printf("AttribDeclaration::addComment %s\n", s->toChars());
-			s->addComment(comment);
+		s->addComment(comment);
 	    }
 	}
     }
@@ -158,11 +158,11 @@ void AttribDeclaration::emitComment(Scope *sc)
 
     if (d)
     {
-    	for (unsigned i = 0; i < d->dim; i++)
-    	{   Dsymbol *s = (Dsymbol *)d->data[i];
+	for (unsigned i = 0; i < d->dim; i++)
+	{   Dsymbol *s = (Dsymbol *)d->data[i];
 	    //printf("AttribDeclaration::emitComment %s\n", s->toChars());
-			s->emitComment(sc);
-    	}
+	    s->emitComment(sc);
+	}
     }
 }
 
@@ -174,7 +174,7 @@ void AttribDeclaration::toObjFile(int multiobj)
     {
     	for (unsigned i = 0; i < d->dim; i++)
     	{   Dsymbol *s = (Dsymbol *)d->data[i];
-			s->toObjFile(multiobj);
+	    s->toObjFile(multiobj);
     	}
     }
 }
@@ -187,13 +187,13 @@ int AttribDeclaration::cvMember(unsigned char *p)
 
     if (d)
     {
-    	for (unsigned i = 0; i < d->dim; i++)
-    	{   Dsymbol *s = (Dsymbol *)d->data[i];
-			n = s->cvMember(p);
-			if (p)
-			p += n;
-			nwritten += n;
-    	}
+	for (unsigned i = 0; i < d->dim; i++)
+	{   Dsymbol *s = (Dsymbol *)d->data[i];
+	    n = s->cvMember(p);
+	    if (p)
+		p += n;
+	    nwritten += n;
+	}
     }
     return nwritten;
 }
@@ -214,7 +214,7 @@ int AttribDeclaration::hasPointers()
     return 0;
 }
 
-char *AttribDeclaration::kind()
+const char *AttribDeclaration::kind()
 {
     return "attribute";
 }
@@ -232,10 +232,10 @@ void AttribDeclaration::checkCtorConstInit()
 
     if (d)
     {
-    	for (unsigned i = 0; i < d->dim; i++)
-    	{   Dsymbol *s = (Dsymbol *)d->data[i];
-			s->checkCtorConstInit();
-    	}
+	for (unsigned i = 0; i < d->dim; i++)
+	{   Dsymbol *s = (Dsymbol *)d->data[i];
+	    s->checkCtorConstInit();
+	}
     }
 }
 
@@ -248,10 +248,10 @@ void AttribDeclaration::addLocalClass(ClassDeclarations *aclasses)
 
     if (d)
     {
-    	for (unsigned i = 0; i < d->dim; i++)
-    	{   Dsymbol *s = (Dsymbol *)d->data[i];
-			s->addLocalClass(aclasses);
-    	}
+	for (unsigned i = 0; i < d->dim; i++)
+	{   Dsymbol *s = (Dsymbol *)d->data[i];
+	    s->addLocalClass(aclasses);
+	}
     }
 }
 
@@ -336,7 +336,8 @@ void StorageClassDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 	{ STCstatic,       TOKstatic },
 	{ STCextern,       TOKextern },
 	{ STCconst,        TOKconst },
-	{ STCinvariant,    TOKinvariant },
+	{ STCinvariant,    TOKimmutable },
+	{ STCshared,       TOKshared },
 	{ STCfinal,        TOKfinal },
 	{ STCabstract,     TOKabstract },
 	{ STCsynchronized, TOKsynchronized },
@@ -344,6 +345,7 @@ void StorageClassDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 	{ STCoverride,     TOKoverride },
 	{ STCnothrow,      TOKnothrow },
 	{ STCpure,         TOKpure },
+	{ STCref,          TOKref },
 	{ STCtls,          TOKtls },
     };
 
@@ -423,7 +425,7 @@ void LinkDeclaration::semantic3(Scope *sc)
 }
 
 void LinkDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
-{   char *p;
+{   const char *p;
 
     switch (linkage)
     {
@@ -444,7 +446,7 @@ void LinkDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 
 char *LinkDeclaration::toChars()
 {
-    return "extern ()";
+    return (char *)"extern ()";
 }
 
 /********************************* ProtDeclaration ****************************/
@@ -489,7 +491,7 @@ void ProtDeclaration::semantic(Scope *sc)
 }
 
 void ProtDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
-{   char *p;
+{   const char *p;
 
     switch (protection)
     {
@@ -711,9 +713,9 @@ void AnonDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
     buf->writestring("}\n");
 }
 
-char *AnonDeclaration::kind()
+const char *AnonDeclaration::kind()
 {
-    return (char *)(isunion ? "anonymous union" : "anonymous struct");
+    return (isunion ? "anonymous union" : "anonymous struct");
 }
 
 /********************************* PragmaDeclaration ****************************/
@@ -1018,7 +1020,7 @@ int PragmaDeclaration::oneMember(Dsymbol **ps)
     return TRUE;
 }
 
-char *PragmaDeclaration::kind()
+const char *PragmaDeclaration::kind()
 {
     return "pragma";
 }
@@ -1214,7 +1216,7 @@ Dsymbol *StaticIfDeclaration::syntaxCopy(Dsymbol *s)
 
 int StaticIfDeclaration::addMember(Scope *sc, ScopeDsymbol *sd, int memnum)
 {
-	//printf("StaticIfDeclaration::addMember() '%s'\n",toChars());
+    //printf("StaticIfDeclaration::addMember() '%s'\n",toChars());
     /* This is deferred until semantic(), so that
      * expressions in the condition can refer to declarations
      * in the same scope, such as:
@@ -1258,7 +1260,7 @@ void StaticIfDeclaration::semantic(Scope *sc)
     }
 }
 
-char *StaticIfDeclaration::kind()
+const char *StaticIfDeclaration::kind()
 {
     return "static if";
 }
@@ -1269,6 +1271,8 @@ char *StaticIfDeclaration::kind()
 CompileDeclaration::CompileDeclaration(Loc loc, Expression *exp)
     : AttribDeclaration(NULL)
 {
+    //printf("CompileDeclaration(loc = %d)\n", loc.linnum);
+    this->loc = loc;
     this->exp = exp;
     this->sd = NULL;
     this->compiled = 0;
@@ -1283,36 +1287,36 @@ Dsymbol *CompileDeclaration::syntaxCopy(Dsymbol *s)
 
 int CompileDeclaration::addMember(Scope *sc, ScopeDsymbol *sd, int memnum)
 {
-	//printf("CompileDeclaration::addMember(sc = %p)\n", sc);
-	this->sd = sd;
-	if (memnum == 0)
-	{	/* No members yet, so parse the mixin now */
-	 	compileIt(sc);
- 		memnum |= AttribDeclaration::addMember(sc, sd, memnum);
-		compiled = 1;
-     }
+    //printf("CompileDeclaration::addMember(sc = %p, memnum = %d)\n", sc, memnum);
+    this->sd = sd;
+    if (memnum == 0)
+    {	/* No members yet, so parse the mixin now */
+	compileIt(sc);
+ 	memnum |= AttribDeclaration::addMember(sc, sd, memnum);
+	compiled = 1;
+    }
     return memnum;
 }
 
 void CompileDeclaration::compileIt(Scope *sc)
 {
-	//printf("CompileDeclaration::compileIt()\n");
+    //printf("CompileDeclaration::compileIt(loc = %d)\n", loc.linnum);
     exp = exp->semantic(sc);
     exp = resolveProperties(sc, exp);
     exp = exp->optimize(WANTvalue | WANTinterpret);
     if (exp->op != TOKstring)
-    {	error("argument to mixin must be a string, not (%s)", exp->toChars());
+    {	exp->error("argument to mixin must be a string, not (%s)", exp->toChars());
     }
     else
     {
-    StringExp *se = (StringExp *)exp;
-    se = se->toUTF8(sc);
-    Parser p(sc->module, (unsigned char *)se->string, se->len, 0);
-    p.loc = loc;
-    p.nextToken();
-    decl = p.parseDeclDefs(0);
-    if (p.token.value != TOKeof)
-    	error("incomplete mixin declaration (%s)", se->toChars());
+	StringExp *se = (StringExp *)exp;
+	se = se->toUTF8(sc);
+	Parser p(sc->module, (unsigned char *)se->string, se->len, 0);
+	p.loc = loc;
+	p.nextToken();
+	decl = p.parseDeclDefs(0);
+	if (p.token.value != TOKeof)
+	    exp->error("incomplete mixin declaration (%s)", se->toChars());
     }
 }
 
