@@ -58,8 +58,6 @@ ClassDeclaration::ClassDeclaration(Loc loc, Identifier *id, BaseClasses *basecla
     type = new TypeClass(this);
     handle = type;
 
-    ctor = NULL;
-    defaultCtor = NULL;
     staticCtor = NULL;
     staticDtor = NULL;
 
@@ -229,7 +227,7 @@ void ClassDeclaration::semantic(Scope *sc)
     //{ static int n;  if (++n == 20) *(char*)0=0; }
 
     if (!ident)		// if anonymous class
-    {	char *id = "__anonclass";
+    {	const char *id = "__anonclass";
 
 	ident = Identifier::generateId(id);
     }
@@ -490,10 +488,22 @@ void ClassDeclaration::semantic(Scope *sc)
 	    if (storage_class & STCstatic)
 		error("static class cannot inherit from nested class %s", baseClass->toChars());
 	    if (toParent2() != baseClass->toParent2())
-		error("super class %s is nested within %s, not %s",
+	    {
+		if (toParent2())
+		{
+		    error("is nested within %s, but super class %s is nested within %s",
+			toParent2()->toChars(),
 			baseClass->toChars(),
-			baseClass->toParent2()->toChars(),
-			toParent2()->toChars());
+			baseClass->toParent2()->toChars());
+		}
+		else
+		{
+		    error("is not nested, but super class %s is nested within %s",
+			baseClass->toChars(),
+			baseClass->toParent2()->toChars());
+		}
+		isnested = 0;
+	    }
 	}
 	else if (!(storage_class & STCstatic))
 	{   Dsymbol *s = toParent2();
@@ -598,7 +608,7 @@ void ClassDeclaration::semantic(Scope *sc)
      * They must be in this class, not in a base class.
      */
     ctor = (CtorDeclaration *)search(0, Id::ctor, 0);
-    if (ctor && ctor->toParent() != this)
+    if (ctor && (ctor->toParent() != this || !ctor->isCtorDeclaration()))
 	ctor = NULL;
 
 //    dtor = (DtorDeclaration *)search(Id::dtor, 0);
@@ -619,7 +629,7 @@ void ClassDeclaration::semantic(Scope *sc)
     if (!ctor && baseClass && baseClass->ctor)
     {
 	//printf("Creating default this(){} for class %s\n", toChars());
-	ctor = new CtorDeclaration(0, 0, NULL, 0);
+	ctor = new CtorDeclaration(loc, 0, NULL, 0);
 	ctor->fbody = new CompoundStatement(0, new Statements());
 	members->push(ctor);
 	ctor->addMember(sc, this, 1);
@@ -981,7 +991,7 @@ int ClassDeclaration::vtblOffset()
 /****************************************
  */
 
-char *ClassDeclaration::kind()
+const char *ClassDeclaration::kind()
 {
     return "class";
 }
@@ -1308,7 +1318,7 @@ int InterfaceDeclaration::isCPPinterface()
 /*******************************************
  */
 
-char *InterfaceDeclaration::kind()
+const char *InterfaceDeclaration::kind()
 {
     return "interface";
 }
