@@ -463,7 +463,7 @@ Expression *Div(Type *type, Expression *e1, Expression *e2)
 	n2 = e2->toInteger();
 	if (n2 == 0)
 	{   e2->error("divide by 0");
-	    e2 = new IntegerExp(0, 1, e2->type);
+	    e2 = new IntegerExp(loc, 1, e2->type);
 	    n2 = 1;
 	}
 	if (e1->type->isunsigned() || e2->type->isunsigned())
@@ -526,7 +526,7 @@ Expression *Mod(Type *type, Expression *e1, Expression *e2)
 	n2 = e2->toInteger();
 	if (n2 == 0)
 	{   e2->error("divide by 0");
-	    e2 = new IntegerExp(0, 1, e2->type);
+	    e2 = new IntegerExp(loc, 1, e2->type);
 	    n2 = 1;
 	}
 	if (e1->type->isunsigned() || e2->type->isunsigned())
@@ -635,26 +635,21 @@ Expression *Ushr(Type *type, Expression *e1, Expression *e2)
 }
 
 Expression *And(Type *type, Expression *e1, Expression *e2)
-{   Expression *e;
-    Loc loc = e1->loc;
-
-    e = new IntegerExp(loc, e1->toInteger() & e2->toInteger(), type);
+{
+    Expression *e;
+    e = new IntegerExp(e1->loc, e1->toInteger() & e2->toInteger(), type);
     return e;
 }
 
 Expression *Or(Type *type, Expression *e1, Expression *e2)
 {   Expression *e;
-    Loc loc = e1->loc;
-
-    e = new IntegerExp(loc, e1->toInteger() | e2->toInteger(), type);
+    e = new IntegerExp(e1->loc, e1->toInteger() | e2->toInteger(), type);
     return e;
 }
 
 Expression *Xor(Type *type, Expression *e1, Expression *e2)
 {   Expression *e;
-    Loc loc = e1->loc;
-
-    e = new IntegerExp(loc, e1->toInteger() ^ e2->toInteger(), type);
+    e = new IntegerExp(e1->loc, e1->toInteger() ^ e2->toInteger(), type);
     return e;
 }
 
@@ -703,7 +698,11 @@ Expression *Equal(enum TOK op, Type *type, Expression *e1, Expression *e2)
     {	StringExp *es1 = (StringExp *)e1;
 	StringExp *es2 = (StringExp *)e2;
 
-	assert(es1->sz == es2->sz);
+	if (es1->sz != es2->sz)
+	{
+	    assert(global.errors);
+	    return EXP_CANT_INTERPRET;
+	}
 	if (es1->len == es2->len &&
 	    memcmp(es1->string, es2->string, es1->sz * es1->len) == 0)
 	    cmp = 1;
@@ -1132,18 +1131,23 @@ Expression *Cast(Type *type, Type *to, Expression *e1)
 	if (e1->type->isfloating())
 	{   integer_t result;
 #ifdef IN_GCC
+	    d_int64 r;
 	    Type * rt = e1->type;
 	    if (rt->iscomplex())
 	    {
 		switch (rt->toBasetype()->ty)
 		{
-		case Tcomplex32: rt = Type::tfloat32; break;
-		case Tcomplex64: rt = Type::tfloat64; break;
-		case Tcomplex80: rt = Type::tfloat80; break;
-		default: assert(0);
+    		    case Tcomplex32: rt = Type::tfloat32; break;
+    		    case Tcomplex64: rt = Type::tfloat64; break;
+    		    case Tcomplex80: rt = Type::tfloat80; break;
+    		    default: assert(0);
 		}
+    		r = e1->toReal().toInt(rt, type);
+	    }
+	    else
+	    {
+    		r = e1->toInteger();
 	    }
-	    d_int64 r = e1->toReal().toInt(rt, type);
 #else
 	    real_t r = e1->toReal();
 #endif
@@ -1213,7 +1217,7 @@ Expression *Cast(Type *type, Type *to, Expression *e1)
     else
     {
 	error(loc, "cannot cast %s to %s", e1->type->toChars(), type->toChars());
-	e = new IntegerExp(loc, 0, type);
+	e = new IntegerExp(loc, 0, Type::tint32);
     }
     return e;
 }
@@ -1444,7 +1448,14 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
 	size_t len = es1->len + es2->len;
 	int sz = es1->sz;
 
-	assert(sz == es2->sz);
+	if (sz != es2->sz)
+	{
+	    /* Can happen with:
+	     *   auto s = "foo"d ~ "bar"c;
+	     */
+	    assert(global.errors);
+	    return e;
+	}
 	s = mem.malloc((len + 1) * sz);
 	memcpy(s, es1->string, es1->len * sz);
 	memcpy((unsigned char *)s + es1->len * sz, es2->string, es2->len * sz);
@@ -1539,7 +1550,32 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
 
 	if (type->toBasetype()->ty == Tsarray)
 	{
-	    e->type = new TypeSArray(t1->nextOf(), new IntegerExp(0, es1->elements->dim, Type::tindex));
+	    e->type = new TypeSArray(t1->nextOf(), new IntegerExp(loc, es1->elements->dim, Type::tindex));
+	    e->type = e->type->semantic(loc, NULL);
+	}
+	else
+	    e->type = type;
+    }
+    else if (e1->op == TOKarrayliteral && e2->op == TOKnull &&
+	t1->nextOf()->equals(t2->nextOf()))
+    {
+	e = e1;
+	goto L3;
+    }
+    else if (e1->op == TOKnull && e2->op == TOKarrayliteral &&
+	t1->nextOf()->equals(t2->nextOf()))
+    {
+	e = e2;
+     L3:
+	// Concatenate the array with null
+	ArrayLiteralExp *es = (ArrayLiteralExp *)e;
+
+	es = new ArrayLiteralExp(es->loc, (Expressions *)es->elements->copy());
+	e = es;
+
+	if (type->toBasetype()->ty == Tsarray)
+	{
+	    e->type = new TypeSArray(t1->nextOf(), new IntegerExp(loc, es->elements->dim, Type::tindex));
 	    e->type = e->type->semantic(loc, NULL);
 	}
 	else
@@ -1562,7 +1598,7 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
 
 	if (type->toBasetype()->ty == Tsarray)
 	{
-	    e->type = new TypeSArray(e2->type, new IntegerExp(0, es1->elements->dim, Type::tindex));
+	    e->type = new TypeSArray(e2->type, new IntegerExp(loc, es1->elements->dim, Type::tindex));
 	    e->type = e->type->semantic(loc, NULL);
 	}
 	else
@@ -1579,7 +1615,7 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
 
 	if (type->toBasetype()->ty == Tsarray)
 	{
-	    e->type = new TypeSArray(e1->type, new IntegerExp(0, es2->elements->dim, Type::tindex));
+	    e->type = new TypeSArray(e1->type, new IntegerExp(loc, es2->elements->dim, Type::tindex));
 	    e->type = e->type->semantic(loc, NULL);
 	}
 	else
