@@ -46,7 +46,7 @@ void Declaration::semantic(Scope *sc)
 {
 }
 
-char *Declaration::kind()
+const char *Declaration::kind()
 {
     return "declaration";
 }
@@ -138,7 +138,7 @@ void Declaration::checkModify(Loc loc, Scope *sc, Type *t)
 	VarDeclaration *v = isVarDeclaration();
 	if (v && v->canassign == 0)
 	{
-	    char *p = NULL;
+	    const char *p = NULL;
 	    if (isConst())
 		p = "const";
 	    else if (isInvariant())
@@ -173,7 +173,7 @@ Dsymbol *TupleDeclaration::syntaxCopy(Dsymbol *s)
     return NULL;
 }
 
-char *TupleDeclaration::kind()
+const char *TupleDeclaration::kind()
 {
     return "tuple";
 }
@@ -311,6 +311,7 @@ void TypedefDeclaration::semantic(Scope *sc)
 	    attributes = sc->attributes;
 	if (sc->parent->isFuncDeclaration() && init)
 	    semantic2(sc);
+	storage_class |= sc->stc & STCdeprecated;
     }
     else if (sem == 1)
     {
@@ -337,7 +338,7 @@ void TypedefDeclaration::semantic2(Scope *sc)
     }
 }
 
-char *TypedefDeclaration::kind()
+const char *TypedefDeclaration::kind()
 {
     return "typedef";
 }
@@ -541,7 +542,7 @@ int AliasDeclaration::overloadInsert(Dsymbol *s)
     }
 }
 
-char *AliasDeclaration::kind()
+const char *AliasDeclaration::kind()
 {
     return "alias";
 }
@@ -868,6 +869,9 @@ Lagain:
 	}
     }
 
+    if ((storage_class & (STCref | STCparameter | STCforeach)) == STCref)
+	error("only parameters or foreach declarations can be ref");
+
     if (type->isauto() && !noauto)
     {
 	if (storage_class & (STCfield | STCout | STCref | STCstatic | STCmanifest | STCtls) || !fd)
@@ -937,6 +941,9 @@ Lagain:
 
     if (init)
     {
+	sc = sc->push();
+	sc->stc &= ~(STCconst | STCinvariant | STCpure | STCnothrow | STCref | STCshared);
+
 	ArrayInitializer *ai = init->isArrayInitializer();
 	if (ai && tb->ty == Taarray)
 	{
@@ -946,7 +953,7 @@ Lagain:
 	StructInitializer *si = init->isStructInitializer();
 	ExpInitializer *ei = init->isExpInitializer();
 
-	// See if we can allocate on the stack
+	// See if initializer is a NewExp that can be allocated on the stack
 	if (ei && isScope() && ei->exp->op == TOKnew)
 	{   NewExp *ne = (NewExp *)ei->exp;
 	    if (!(ne->newargs && ne->newargs->dim))
@@ -1006,6 +1013,51 @@ Lagain:
 		else if (t->ty == Tstruct)
 		{
 		    ei->exp = ei->exp->semantic(sc);
+
+		    /* Look to see if initializer is a call to the constructor
+		     */
+		    StructDeclaration *sd = ((TypeStruct *)t)->sym;
+		    if (sd->ctor &&		// there are constructors
+			ei->exp->type->ty == Tstruct &&	// rvalue is the same struct
+			((TypeStruct *)ei->exp->type)->sym == sd &&
+			ei->exp->op == TOKstar)
+		    {
+			/* Look for form of constructor call which is:
+			 *    *__ctmp.ctor(arguments...)
+			 */
+			PtrExp *pe = (PtrExp *)ei->exp;
+			if (pe->e1->op == TOKcall)
+			{   CallExp *ce = (CallExp *)pe->e1;
+			    if (ce->e1->op == TOKdotvar)
+			    {	DotVarExp *dve = (DotVarExp *)ce->e1;
+				if (dve->var->isCtorDeclaration())
+				{   /* It's a constructor call, currently constructing
+				     * a temporary __ctmp.
+				     */
+				    /* Before calling the constructor, initialize
+				     * variable with a bit copy of the default
+				     * initializer
+				     */
+				    Expression *e = new AssignExp(loc, new VarExp(loc, this), t->defaultInit(loc));
+				    e->op = TOKblit;
+				    e->type = t;
+				    ei->exp = new CommaExp(loc, e, ei->exp);
+#if IN_GCC
+				    /* We rely on exp->type being set here, else
+				     * causes problems later in CommaExp::toElem()
+				     */
+				    ei->exp = ei->exp->semantic(sc);
+#endif
+
+				    /* Replace __ctmp being constructed with e1
+				     */
+				    dve->e1 = e1;
+				    return;
+				}
+			    }
+			}
+		    }
+
 		    if (!ei->exp->implicitConvTo(type))
 		    {	Type *ti = ei->exp->type->toBasetype();
 			// Don't cast away invariant or mutability in initializer
@@ -1089,6 +1141,7 @@ Lagain:
 		    init = i2;		// no errors, keep result
 	    }
 	}
+	sc = sc->pop();
     }
 }
 
@@ -1110,7 +1163,7 @@ void VarDeclaration::semantic2(Scope *sc)
     }
 }
 
-char *VarDeclaration::kind()
+const char *VarDeclaration::kind()
 {
     return "variable";
 }
@@ -1173,8 +1226,10 @@ int VarDeclaration::isImportedSymbol()
 
 void VarDeclaration::checkCtorConstInit()
 {
+#if 0 /* doesn't work if more than one static ctor */
     if (ctorinit == 0 && isCtorinit() && !(storage_class & STCfield))
 	error("missing initializer in static constructor for const variable");
+#endif
 }
 
 /************************************
@@ -1232,7 +1287,7 @@ ExpInitializer *VarDeclaration::getExpInitializer()
 	ei = init->isExpInitializer();
     else
     {
-	Expression *e = type->defaultInit();
+	Expression *e = type->defaultInit(loc);
 	if (e)
 	    ei = new ExpInitializer(loc, e);
 	else
