@@ -113,6 +113,7 @@ void FuncDeclaration::semantic(Scope *sc)
     StructDeclaration *sd;
     ClassDeclaration *cd;
     InterfaceDeclaration *id;
+    Dsymbol *pd;
 
 #if 0
     printf("FuncDeclaration::semantic(sc = %p, this = %p, '%s', linkage = %d)\n", sc, this, toPrettyChars(), sc->linkage);
@@ -122,7 +123,7 @@ void FuncDeclaration::semantic(Scope *sc)
     printf("type: %p, %s\n", type, type->toChars());
 #endif
 
-    storage_class |= sc->stc;
+    storage_class |= sc->stc & ~STCref;
     //printf("function storage_class = x%x\n", storage_class);
 
     if (!originalType)
@@ -224,6 +225,11 @@ void FuncDeclaration::semantic(Scope *sc)
     sd = parent->isStructDeclaration();
     if (sd)
     {
+	if (isCtorDeclaration())
+	{
+	    return;
+	}
+#if 0
 	// Verify no constructors, destructors, etc.
 	if (isCtorDeclaration()
 	    //||isDtorDeclaration()
@@ -234,7 +240,6 @@ void FuncDeclaration::semantic(Scope *sc)
 	    error("special member functions not allowed for %ss", sd->kind());
 	}
 
-#if 0
 	if (!sd->inv)
 	    sd->inv = isInvariantDeclaration();
 
@@ -265,6 +270,18 @@ void FuncDeclaration::semantic(Scope *sc)
 	    error("function body is not abstract in interface %s", id->toChars());
     }
 
+    /* Template member functions aren't virtual:
+     *   interface TestInterface { void tpl(T)(); }
+     * and so won't work in interfaces
+     */
+    if ((pd = toParent()) != NULL &&
+	pd->isTemplateInstance() &&
+	(pd = toParent2()) != NULL &&
+	(id = pd->isInterfaceDeclaration()) != NULL)
+    {
+	error("template member function not allowed in interface %s", id->toChars());
+    }
+
     cd = parent->isClassDeclaration();
     if (cd)
     {	int vi;
@@ -354,7 +371,6 @@ void FuncDeclaration::semantic(Scope *sc)
 		    cd->vtbl.push(this);
 		    vtblIndex = vi;
 		}
-
 		break;
 
 	    case -2:	// can't determine because of fwd refs
@@ -367,8 +383,10 @@ void FuncDeclaration::semantic(Scope *sc)
 		if (fdv->isFinal())
 		    error("cannot override final function %s", fdv->toPrettyChars());
 
+#if V2
 		if (!isOverride() && global.params.warnings)
 		    error("overrides base class function %s, but is not marked with 'override'", fdv->toPrettyChars());
+#endif
 
 		if (fdv->toParent() == parent)
 		{
@@ -381,7 +399,9 @@ void FuncDeclaration::semantic(Scope *sc)
 #if !BREAKABI
 			&& !isDtorDeclaration()
 #endif
+#if V2
 			&& !isPostBlitDeclaration()
+#endif
 			)
 			error("multiple overrides of same function");
 		}
@@ -789,11 +809,7 @@ void FuncDeclaration::semantic3(Scope *sc)
 		    /* Generate identifier for un-named parameter,
 		     * because we need it later on.
 		     */
-		    OutBuffer buf;
-		    buf.printf("_param_%"PRIuSIZE, i);
-		    char *name = (char *)buf.extractData();
-		    id = new Identifier(name, TOKidentifier);
-		    arg->ident = id;
+		    arg->ident = id = Identifier::generateId("_param_", i);
 		}
 		VarDeclaration *v = new VarDeclaration(loc, arg->type, id, NULL);
 		//printf("declaring parameter %s of type %s\n", v->toChars(), v->type->toChars());
@@ -1090,7 +1106,7 @@ void FuncDeclaration::semantic3(Scope *sc)
 			    e = new AssertExp(
 				  endloc,
 				  new IntegerExp(0),
-				  new StringExp(loc, "missing return expression")
+				  new StringExp(loc, (char *)"missing return expression")
 				);
 			}
 			else
@@ -1207,7 +1223,7 @@ void FuncDeclaration::semantic3(Scope *sc)
 		{   // Call invariant virtually
 		    ThisExp *v = new ThisExp(0);
 		    v->type = vthis->type;
-		    Expression *se = new StringExp(0, "null this");
+		    Expression *se = new StringExp(0, (char *)"null this");
 		    se = se->semantic(sc);
 		    se->type = Type::tchar->arrayOf();
 		    e = new AssertExp(loc, v, se);
@@ -1452,6 +1468,10 @@ int FuncDeclaration::overloadInsert(Dsymbol *s)
     return TRUE;
 }
 
+/********************************************
+ * Find function in overload list that exactly matches t.
+ */
+
 /***************************************************
  * Visit each overloaded function in turn, and call
  * (*fp)(param, f) on it.
@@ -1991,7 +2011,7 @@ int FuncDeclaration::isVirtual()
 {
 #if 0
     printf("FuncDeclaration::isVirtual(%s)\n", toChars());
-    printf("%p %d %d %d %d\n", isMember(), isStatic(), protection == PROTprivate, isCtorDeclaration(), linkage != LINKd);
+    printf("isMember:%p isStatic:%d private:%d ctor:%d !Dlinkage:%d\n", isMember(), isStatic(), protection == PROTprivate, isCtorDeclaration(), linkage != LINKd);
     printf("result is %d\n",
 	isMember() &&
 	!(isStatic() || protection == PROTprivate || protection == PROTpackage) &&
@@ -2082,7 +2102,7 @@ int FuncDeclaration::addPostInvariant()
  * Generate a FuncDeclaration for a runtime library function.
  */
 
-FuncDeclaration *FuncDeclaration::genCfunc(Type *treturn, char *name,
+FuncDeclaration *FuncDeclaration::genCfunc(Type *treturn, const char *name,
     Type *t1, Type *t2, Type *t3)
 {
     return genCfunc(treturn, Lexer::idPool(name), t1, t2, t3);
@@ -2133,7 +2153,7 @@ FuncDeclaration *FuncDeclaration::genCfunc(Type *treturn, Identifier *id,
     return fd;
 }
 
-char *FuncDeclaration::kind()
+const char *FuncDeclaration::kind()
 {
     return "function";
 }
@@ -2176,7 +2196,7 @@ int FuncDeclaration::needsClosure()
 		goto Lyes;	// assume f escapes this function's scope
 
 	    // Look to see if any parents of f that are below this escape
-	    for (Dsymbol *s = f->parent; s != this; s = s->parent)
+	    for (Dsymbol *s = f->parent; s && s != this; s = s->parent)
 	    {
 		f = s->isFuncDeclaration();
 		if (f && (f->isThis() || f->tookAddressOf))
@@ -2204,7 +2224,7 @@ FuncAliasDeclaration::FuncAliasDeclaration(FuncDeclaration *funcalias)
     this->funcalias = funcalias;
 }
 
-char *FuncAliasDeclaration::kind()
+const char *FuncAliasDeclaration::kind()
 {
     return "function alias";
 }
@@ -2216,7 +2236,7 @@ FuncLiteralDeclaration::FuncLiteralDeclaration(Loc loc, Loc endloc, Type *type,
 	enum TOK tok, ForeachStatement *fes)
     : FuncDeclaration(loc, endloc, NULL, STCundefined, type)
 {
-    char *id;
+    const char *id;
 
     if (fes)
 	id = "__foreachbody";
@@ -2249,7 +2269,12 @@ int FuncLiteralDeclaration::isNested()
     return (tok == TOKdelegate);
 }
 
-char *FuncLiteralDeclaration::kind()
+int FuncLiteralDeclaration::isVirtual()
+{
+    return FALSE;
+}
+
+const char *FuncLiteralDeclaration::kind()
 {
     // GCC requires the (char*) casts
     return (tok == TOKdelegate) ? (char*)"delegate" : (char*)"function";
@@ -2277,7 +2302,7 @@ CtorDeclaration::CtorDeclaration(Loc loc, Loc endloc, Arguments *arguments, int
 {
     this->arguments = arguments;
     this->varargs = varargs;
-    //printf("CtorDeclaration() %s\n", toChars());
+    //printf("CtorDeclaration(loc = %s) %s\n", loc.toChars(), toChars());
 }
 
 Dsymbol *CtorDeclaration::syntaxCopy(Dsymbol *s)
@@ -2299,7 +2324,7 @@ Dsymbol *CtorDeclaration::syntaxCopy(Dsymbol *s)
 
 void CtorDeclaration::semantic(Scope *sc)
 {
-    ClassDeclaration *cd;
+    AggregateDeclaration *ad;
     Type *tret;
 
     //printf("CtorDeclaration::semantic()\n");
@@ -2311,14 +2336,16 @@ void CtorDeclaration::semantic(Scope *sc)
 
     parent = sc->parent;
     Dsymbol *parent = toParent();
-    cd = parent->isClassDeclaration();
-    if (!cd)
+    ad = parent->isAggregateDeclaration();
+    if (!ad || parent->isUnionDeclaration())
     {
-	error("constructors are only for class definitions");
+	error("constructors are only for class or struct definitions");
 	tret = Type::tvoid;
     }
     else
-	tret = cd->type; //->referenceTo();
+    {	tret = ad->handle;
+	assert(tret);
+    }
     type = new TypeFunction(arguments, tret, varargs, LINKd);
     if (!originalType)
     	originalType = type;
@@ -2331,11 +2358,9 @@ void CtorDeclaration::semantic(Scope *sc)
     //	return this;
     // to the function body
     if (fbody)
-    {	Expression *e;
-	Statement *s;
-
-	e = new ThisExp(0);
-	s = new ReturnStatement(0, e);
+    {
+	Expression *e = new ThisExp(0);
+	Statement *s = new ReturnStatement(0, e);
 	fbody = new CompoundStatement(0, fbody, s);
     }
 
@@ -2344,18 +2369,22 @@ void CtorDeclaration::semantic(Scope *sc)
     sc->pop();
 
     // See if it's the default constructor
-    if (cd && varargs == 0 && Argument::dim(arguments) == 0)
-	cd->defaultCtor = this;
+    if (ad && varargs == 0 && Argument::dim(arguments) == 0)
+    {	if (ad->isStructDeclaration())
+	    error("default constructor not allowed for structs");
+	else
+	    ad->defaultCtor = this;
+    }
 }
 
-char *CtorDeclaration::kind()
+const char *CtorDeclaration::kind()
 {
     return "constructor";
 }
 
 char *CtorDeclaration::toChars()
 {
-    return "this";
+    return (char *)"this";
 }
 
 int CtorDeclaration::isVirtual()
@@ -2669,28 +2698,28 @@ void StaticDtorDeclaration::semantic(Scope *sc)
      */
     if (inTemplateInstance())
     {
-     	/* Add this prefix to the function:
-     	 *	static int gate;
-     	 *	if (--gate != 0) return;
-     	 * Increment gate during constructor execution.
-     	 * Note that this is not thread safe; should not have threads
-     	 * during static destruction.
-     	 */
-     	Identifier *id = Lexer::idPool("__gate");
-     	VarDeclaration *v = new VarDeclaration(0, Type::tint32, id, NULL);
-     	v->storage_class = STCstatic;
-     	Statements *sa = new Statements();
-     	Statement *s = new DeclarationStatement(0, v);
-     	sa->push(s);
-     	Expression *e = new IdentifierExp(0, id);
-     	e = new AddAssignExp(0, e, new IntegerExp(-1));
-     	e = new EqualExp(TOKnotequal, 0, e, new IntegerExp(1));
-     	s = new IfStatement(0, NULL, e, new ReturnStatement(0, NULL), NULL);
-     	sa->push(s);
-     	if (fbody)
-     	    sa->push(fbody);
-     	fbody = new CompoundStatement(0, sa);
-     	vgate = v;
+	/* Add this prefix to the function:
+	 *	static int gate;
+	 *	if (--gate != 0) return;
+	 * Increment gate during constructor execution.
+	 * Note that this is not thread safe; should not have threads
+	 * during static destruction.
+	 */
+	Identifier *id = Lexer::idPool("__gate");
+	VarDeclaration *v = new VarDeclaration(0, Type::tint32, id, NULL);
+	v->storage_class = STCstatic;
+	Statements *sa = new Statements();
+	Statement *s = new DeclarationStatement(0, v);
+	sa->push(s);
+	Expression *e = new IdentifierExp(0, id);
+	e = new AddAssignExp(0, e, new IntegerExp(-1));
+	e = new EqualExp(TOKnotequal, 0, e, new IntegerExp(0));
+	s = new IfStatement(0, NULL, e, new ReturnStatement(0, NULL), NULL);
+	sa->push(s);
+	if (fbody)
+	    sa->push(fbody);
+	fbody = new CompoundStatement(0, sa);
+	vgate = v;
     }
 
     FuncDeclaration::semantic(sc);
@@ -2947,7 +2976,7 @@ void NewDeclaration::semantic(Scope *sc)
     FuncDeclaration::semantic(sc);
 }
 
-char *NewDeclaration::kind()
+const char *NewDeclaration::kind()
 {
     return "allocator";
 }
@@ -3031,7 +3060,7 @@ void DeleteDeclaration::semantic(Scope *sc)
     FuncDeclaration::semantic(sc);
 }
 
-char *DeleteDeclaration::kind()
+const char *DeleteDeclaration::kind()
 {
     return "deallocator";
 }
