@@ -28,8 +28,8 @@
 
 struct Msgtable
 {
-	char *ident;	// name to use in DMD source
-	char *name;	// name in D executable
+	const char *ident;	// name to use in DMD source
+	const char *name;	// name in D executable
 };
 
 struct Msgtable msgtable[] =
@@ -40,12 +40,12 @@ struct Msgtable msgtable[] =
     { "max" },
     { "min" },
     { "This", "this" },
-    { "ctor", "_ctor" },
-    { "cpctor", "_cpctor" },
-    { "dtor", "_dtor" },
-    { "_postblit", "_postblit" },
+    { "ctor", "__ctor" },
+    { "dtor", "__dtor" },
+    { "cpctor", "__cpctor" },
+    { "_postblit", "__postblit" },
     { "classInvariant", "__invariant" },
-    { "unitTest", "_unitTest" },
+    { "unitTest", "__unitTest" },
     { "init" },
     { "size" },
     { "__sizeof", "sizeof" },
@@ -74,6 +74,8 @@ struct Msgtable msgtable[] =
     { "empty", "" },
     { "p" },
     { "coverage", "__coverage" },
+    { "__vptr" },
+    { "__monitor" },
 
     { "TypeInfo" },
     { "TypeInfo_Class" },
@@ -295,7 +297,7 @@ int main()
 	fprintf(fp, "{\n");
 
 	for (i = 0; i < sizeof(msgtable) / sizeof(msgtable[0]); i++)
-	{   char *id = msgtable[i].ident;
+	{   const char *id = msgtable[i].ident;
 
 	    fprintf(fp,"    static Identifier *%s;\n", id);
 	}
@@ -320,8 +322,8 @@ int main()
 	fprintf(fp, "#include \"lexer.h\"\n");
 
 	for (i = 0; i < sizeof(msgtable) / sizeof(msgtable[0]); i++)
-	{   char *id = msgtable[i].ident;
-	    char *p = msgtable[i].name;
+	{   const char *id = msgtable[i].ident;
+	    const char *p = msgtable[i].name;
 
 	    if (!p)
 		p = id;
@@ -332,8 +334,8 @@ int main()
 	fprintf(fp, "{\n");
 
 	for (i = 0; i < sizeof(msgtable) / sizeof(msgtable[0]); i++)
-	{   char *id = msgtable[i].ident;
-	    char *p = msgtable[i].name;
+	{   const char *id = msgtable[i].ident;
+	    const char *p = msgtable[i].name;
 
 	    if (!p)
 		p = id;
