@@ -92,7 +92,7 @@ static void cmtable_init()
 
 /************************* Token **********************************************/
 
-char *Token::tochars[TOKMAX];
+const char *Token::tochars[TOKMAX];
 
 void *Token::operator new(size_t size)
 {   Token *t;
@@ -114,9 +114,13 @@ void Token::print()
 }
 #endif
 
-char *Token::toChars()
-{   char *p;
+const char *Token::toChars()
+{   const char *p;
+#if IN_GCC
+    static char buffer[(3 + 3 * sizeof(value) + 1) * 2];
+#else
     static char buffer[3 + 3 * sizeof(value) + 1];
+#endif
 
     p = buffer;
     switch (value)
@@ -244,8 +248,8 @@ char *Token::toChars()
     return p;
 }
 
-char *Token::toChars(enum TOK value)
-{   char *p;
+const char *Token::toChars(enum TOK value)
+{   const char *p;
     static char buffer[3 + 3 * sizeof(value) + 1];
 
     p = tochars[value];
@@ -398,6 +402,15 @@ Token *Lexer::peek(Token *ct)
     return t;
 }
 
+/***********************
+ * Look ahead at next token's value.
+ */
+
+TOK Lexer::peekNext()
+{
+    return peek(&token)->value;
+}
+
 /*********************************
  * tk is on the opening (.
  * Look ahead and return token that is past the closing ).
@@ -472,7 +485,7 @@ int Lexer::isValidIdentifier(char *p)
     while (p[idx])
     {   dchar_t dc;
 
-	char *q = utf_decodeChar((unsigned char *)p, len, &idx, &dc);
+	const char *q = utf_decodeChar((unsigned char *)p, len, &idx, &dc);
 	if (q)
 	    goto Linvalid;
 
@@ -727,7 +740,7 @@ void Lexer::scan(Token *t)
 		    {	unsigned major = 0;
 			unsigned minor = 0;
 
-			for (char *p = global.version + 1; 1; p++)
+			for (const char *p = global.version + 1; 1; p++)
 			{
 			    char c = *p;
 			    if (isdigit(c))
@@ -2195,13 +2208,15 @@ done:
 		break;
 	    if (d >= r)
 		break;
-	    if (n && n * r + d <= n)
+	    uinteger_t n2 = n * r;
+	    //printf("n2 / r = %llx, n = %llx\n", n2/r, n);
+	    if (n2 / r != n || n2 + d < n)
 	    {
 		error ("integer overflow");
 		break;
 	    }
 
-	    n = n * r + d;
+	    n = n2 + d;
 	    p++;
 	}
 #endif
@@ -2612,7 +2627,7 @@ unsigned Lexer::decodeUTF()
     unsigned char *s = p;
     size_t len;
     size_t idx;
-    char *msg;
+    const char *msg;
 
     c = *s;
     assert(c & 0x80);
@@ -2810,7 +2825,7 @@ Identifier *Lexer::uniqueId(const char *s)
  */
 
 struct Keyword
-{   char *name;
+{   const char *name;
     enum TOK value;
 };
 
@@ -2899,6 +2914,7 @@ static Keyword keywords[] =
     {	"static",	TOKstatic	},
     {	"final",	TOKfinal	},
     {	"const",	TOKconst	},
+    {	"immutable",	TOKimmutable	},
     {	"typedef",	TOKtypedef	},
     {	"alias",	TOKalias	},
     {	"override",	TOKoverride	},
@@ -2937,6 +2953,7 @@ static Keyword keywords[] =
     {	"__overloadset", TOKoverloadset	},
     {	"__FILE__",	TOKfile		},
     {	"__LINE__",	TOKline		},
+    {	"shared",	TOKshared	},
 #endif
 };
 
@@ -2962,7 +2979,7 @@ void Lexer::initKeywords()
     cmtable_init();
 
     for (u = 0; u < nkeywords; u++)
-    {	char *s;
+    {	const char *s;
 
 	//printf("keyword[%d] = '%s'\n",u, keywords[u].name);
 	s = keywords[u].name;
