@@ -131,7 +131,7 @@ enum TOK
 	TOKalign, TOKextern, TOKprivate, TOKprotected, TOKpublic, TOKexport,
 	TOKstatic, /*TOKvirtual,*/ TOKfinal, TOKconst, TOKabstract, TOKvolatile,
 	TOKdebug, TOKdeprecated, TOKin, TOKout, TOKinout, TOKlazy,
-	TOKauto, TOKpackage, TOKmanifest,
+	TOKauto, TOKpackage, TOKmanifest, TOKimmutable,
 
 	// Statements
 	TOKif, TOKelse, TOKwhile, TOKfor, TOKdo, TOKswitch,
@@ -158,6 +158,7 @@ enum TOK
 	TOKtls,
 	TOKline,
 	TOKfile,
+	TOKshared,
 #endif
 
 	TOKMAX
@@ -235,13 +236,13 @@ struct Token
     real_t float80value; // can't use this in a union!
 #endif
 
-    static char *tochars[TOKMAX];
+    static const char *tochars[TOKMAX];
     static void *operator new(size_t sz);
 
     int isKeyword();
     void print();
-    char *toChars();
-    static char *toChars(enum TOK);
+    const char *toChars();
+    static const char *toChars(enum TOK);
 };
 
 struct Lexer
@@ -271,6 +272,7 @@ struct Lexer
     static Identifier *uniqueId(const char *s, int num);
 
     TOK nextToken();
+    TOK peekNext();
     void scan(Token *t);
     Token *peek(Token *t);
     Token *peekPastParen(Token *t);
