@@ -45,7 +45,9 @@ long __cdecl __ehfilter(LPEXCEPTION_POINTERS ep);
 #include "cond.h"
 #include "expression.h"
 #include "lexer.h"
+#ifndef IN_GCC
 #include "lib.h"
+#endif
 
 void browse(const char *url);
 void getenv_setargv(const char *envvar, int *pargc, char** *pargv);
@@ -89,7 +91,7 @@ Global::Global()
 
     copyright = "Copyright (c) 1999-2008 by Digital Mars";
     written = "written by Walter Bright";
-    version = "v2.015";
+    version = "v2.020";
     global.structalign = 8;
 
     memset(&params, 0, sizeof(Param));
@@ -303,6 +305,7 @@ int main(int argc, char *argv[])
     global.params.isWindows = 1;
 #endif
 #if linux
+    VersionCondition::addPredefinedGlobalIdent("Posix");
     VersionCondition::addPredefinedGlobalIdent("linux");
     global.params.isLinux = 1;
 #endif /* linux */
@@ -815,6 +818,7 @@ int main(int argc, char *argv[])
 	     * D source file extension
 	     */
 	    if (stricmp(ext, global.mars_ext) == 0 ||
+		stricmp(ext, global.hdr_ext) == 0 ||
 		stricmp(ext, "dd") == 0 ||
 		stricmp(ext, "htm") == 0 ||
 		stricmp(ext, "html") == 0 ||
