@@ -123,9 +123,10 @@ extern int Tindex;
 struct Type : Object
 {
     TY ty;
-    unsigned char mod;	// modifiers (MODconst, MODinvariant)
+    unsigned char mod;	// modifiers MODxxxx
 	#define MODconst     1	// type is const
 	#define MODinvariant 2	// type is invariant
+	#define MODshared    4	// type is shared
     char *deco;
     Type *cto;		// MODconst ? mutable version of this type : const version
     Type *ito;		// MODinvariant ? mutable version of this type : invariant version
@@ -301,7 +302,7 @@ struct TypeNext : Type
 
 struct TypeBasic : Type
 {
-    char *dstring;
+    const char *dstring;
     unsigned flags;
 
     TypeBasic(TY ty);
@@ -423,7 +424,7 @@ struct TypeAArray : TypeArray
 #endif
 
     // Back end
-    Symbol *aaGetSymbol(char *func, int flags);
+    Symbol *aaGetSymbol(const char *func, int flags);
 
     type *toCtype();
 };
@@ -478,6 +479,7 @@ struct TypeFunction : TypeNext
 			// 2: T t ...) style for variable number of arguments
     bool isnothrow;	// true: nothrow
     bool ispure;	// true: pure
+    bool isref;		// true: returns a reference
     enum LINK linkage;	// calling convention
 
     int inuse;
