@@ -532,7 +532,7 @@ Expression *BinExp::optimize(int result)
 	    integer_t i2 = e2->toInteger();
 	    d_uns64 sz = e1->type->size() * 8;
 	    if (i2 < 0 || i2 > sz)
-	    {   error("shift assign by %jd is outside the range 0..%zu", i2, sz);
+	    {   error("shift assign by %jd is outside the range 0..%"PRIuTSIZE, i2, sz);
 		e2 = new IntegerExp(0);
 	    }
 	}
@@ -627,7 +627,7 @@ Expression *shift_optimize(int result, BinExp *e, Expression *(*shift)(Type *, E
 	integer_t i2 = e->e2->toInteger();
 	target_size_t sz = e->e1->type->size() * 8;
 	if (i2 < 0 || i2 > sz)
-	{   error("shift left by %"PRIdMAX" exceeds %"PRIuTSIZE, i2, sz);
+	{   e->error("shift by %"PRIdMAX" is outside the range 0..%"PRIuTSIZE, i2, sz);
 	    e->e2 = new IntegerExp(0);
 	}
 	if (e->e1->isConst() == 1)
