@@ -211,7 +211,7 @@ Statement *ExpStatement::semantic(Scope *sc)
 	exp->checkSideEffect(0);
 	exp = exp->optimize(0);
 	if (exp->op == TOKdeclaration && !isDeclarationStatement())
-	{	Statement *s = new DeclarationStatement(loc, exp);
+	{   Statement *s = new DeclarationStatement(loc, exp);
 	    return s;
 	}
 	//exp = exp->optimize(isDeclarationStatement() ? WANTvalue : 0);
@@ -304,15 +304,14 @@ Statements *CompileStatement::flatten(Scope *sc)
 }
 
 Statement *CompileStatement::semantic(Scope *sc)
- {
-     //printf("CompileStatement::semantic() %s\n", exp->toChars());
-     /* Shouldn't happen unless errors, as CompileStatement::flatten()
-      * should have replaced it.
-      * Return NULL so no further errors happen.
-      */
-     assert(global.errors);
-     return NULL;
- }
+{
+    //printf("CompileStatement::semantic() %s\n", exp->toChars());
+    Statements *a = flatten(sc);
+    if (!a)
+	return NULL;
+    Statement *s = new CompoundStatement(loc, a);
+    return s->semantic(sc);
+}
 
 
 /******************************** DeclarationStatement ***************************/
@@ -590,7 +589,7 @@ int CompoundStatement::blockExit()
 int CompoundStatement::fallOffEnd()
 {   int falloff = TRUE;
 
-    //printf("CompoundStatement::fallOffEnd()\n");
+    //printf("CompoundStatement::fallOffEnd() %s\n", toChars());
     for (int i = 0; i < statements->dim; i++)
     {	Statement *s = (Statement *)statements->data[i];
 
@@ -933,14 +932,25 @@ int WhileStatement::blockExit()
     int result = BEnone;
     if (condition->canThrow())
 	result |= BEthrow;
-    if (body)
-    {	result |= body->blockExit();
-	if (result & BEbreak)
-	    result |= BEfallthru;
-	result &= ~(BEbreak | BEcontinue);
+    if (condition->isBool(TRUE))
+    {
+	if (body)
+	{   result |= body->blockExit();
+	    if (result & BEbreak)
+		result |= BEfallthru;
+	}
+    }
+    else if (condition->isBool(FALSE))
+    {
+	result |= BEfallthru;
     }
     else
+    {
+	if (body)
+	    result |= body->blockExit();
 	result |= BEfallthru;
+    }
+    result &= ~(BEbreak | BEcontinue);
     return result;
 }
 
@@ -1019,20 +1029,20 @@ int DoStatement::blockExit()
 
     if (body)
     {	result = body->blockExit();
-	if (result & BEbreak)
-	{
-	    if (result == BEbreak)
-		return BEfallthru;
-	    result |= BEfallthru;
-	}
+	if (result == BEbreak)
+	    return BEfallthru;
 	if (result & BEcontinue)
 	    result |= BEfallthru;
-	result &= ~(BEbreak | BEcontinue);
     }
     else
 	result = BEfallthru;
-    if (result & BEfallthru && condition->canThrow())
-	result |= BEthrow;
+    if (result & BEfallthru)
+    {	if (condition->canThrow())
+	    result |= BEthrow;
+	if (!(result & BEbreak) && condition->isBool(TRUE))
+	    result &= ~BEfallthru;
+    }
+    result &= ~(BEbreak | BEcontinue);
     return result;
 }
 
@@ -1434,9 +1444,14 @@ Statement *ForeachStatement::semantic(Scope *sc)
 		    //var->storage_class |= STCfinal;
 		}
 		else
-		{   //if (!(arg->storageClass & STCref))
-			//var->storage_class |= STCfinal;
+		{
 		    value = var;
+		    /* Reference to immutable data should be marked as const
+		     */
+		    if (var->storage_class & STCref && !tn->isMutable())
+		    {
+			var->storage_class |= STCconst;
+		    }
 		}
 #if 1
 		DeclarationExp *de = new DeclarationExp(loc, var);
@@ -2045,14 +2060,31 @@ int IfStatement::blockExit()
     int result = BEnone;
     if (condition->canThrow())
 	result |= BEthrow;
-    if (ifbody)
-	result |= ifbody->blockExit();
+    if (condition->isBool(TRUE))
+    {
+	if (ifbody)
+	    result |= ifbody->blockExit();
+	else
+	    result |= BEfallthru;
+    }
+    else if (condition->isBool(FALSE))
+    {
+	if (elsebody)
+	    result |= elsebody->blockExit();
+	else
+	    result |= BEfallthru;
+    }
     else
-	result |= BEfallthru;
-    if (elsebody)
-	result |= elsebody->blockExit();
-    else
-	result |= BEfallthru;
+    {
+	if (ifbody)
+	    result |= ifbody->blockExit();
+	else
+	    result |= BEfallthru;
+	if (elsebody)
+	    result |= elsebody->blockExit();
+	else
+	    result |= BEfallthru;
+    }
     //printf("IfStatement::blockExit(%p) = x%x\n", this, result);
     return result;
 }
@@ -2074,8 +2106,10 @@ void IfStatement::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 	if (arg->type)
 	    arg->type->toCBuffer(buf, arg->ident, hgs);
 	else
+	{   buf->writestring("auto ");
 	    buf->writestring(arg->ident->toChars());
-	buf->writebyte(';');
+	}
+	buf->writestring(" = ");
     }
     condition->toCBuffer(buf, hgs);
     buf->writebyte(')');
@@ -2147,6 +2181,14 @@ int ConditionalStatement::usesEH()
     return (ifbody && ifbody->usesEH()) || (elsebody && elsebody->usesEH());
 }
 
+int ConditionalStatement::blockExit()
+{
+    int result = ifbody->blockExit();
+    if (elsebody)
+	result |= elsebody->blockExit();
+    return result;
+}
+
 void ConditionalStatement::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 {
     condition->toCBuffer(buf, hgs);
@@ -2621,8 +2663,7 @@ int CaseStatement::usesEH()
 
 int CaseStatement::blockExit()
 {
-    // Assume the worst
-    return BEany;
+    return statement->blockExit();
 }
 
 int CaseStatement::fallOffEnd()
@@ -2688,8 +2729,7 @@ int DefaultStatement::usesEH()
 
 int DefaultStatement::blockExit()
 {
-    // Assume the worst
-    return BEany;
+    return statement->blockExit();
 }
 
 int DefaultStatement::fallOffEnd()
@@ -2907,7 +2947,10 @@ Statement *ReturnStatement::semantic(Scope *sc)
 	{   VarExp *ve = (VarExp *)exp;
 	    VarDeclaration *v = ve->var->isVarDeclaration();
 
-	    if (!v || v->isOut() || v->isRef())
+	    if (((TypeFunction *)fd->type)->isref)
+		// Function returns a reference
+		fd->nrvo_can = 0;
+	    else if (!v || v->isOut() || v->isRef())
 		fd->nrvo_can = 0;
 	    else if (tbret->ty == Tstruct && ((TypeStruct *)tbret)->sym->dtor)
 		// Struct being returned has destructors
@@ -3037,6 +3080,22 @@ Statement *ReturnStatement::semantic(Scope *sc)
 	    exp = new AssignExp(loc, v, exp);
 	    exp = exp->semantic(sc);
 	}
+
+	if (((TypeFunction *)fd->type)->isref)
+	{   // Function returns a reference
+	    if (tbret->isMutable())
+		exp = exp->modifiableLvalue(sc, exp);
+	    else
+		exp = exp->toLvalue(sc, exp);
+
+	    if (exp->op == TOKvar)
+	    {	VarExp *ve = (VarExp *)exp;
+		VarDeclaration *v = ve->var->isVarDeclaration();
+		if (v && !v->isDataseg() && !(v->storage_class & (STCref | STCout)))
+		    error("escaping reference to local variable %s", v->toChars());
+	    }
+	}
+
 	//exp->dump(0);
 	//exp->print();
 	exp->checkEscape();
