@@ -381,8 +381,10 @@ dt_t *ArrayInitializer::toDt()
 	    dt_t * cdt;
 	    cdt = NULL;
 	    if (tb->ty == Tarray)
+	    {
 		dtcontainer(& cdt, type, d);
-	    d = cdt;
+		d = cdt;
+	    }
 #endif
 	    break;
 
@@ -995,7 +997,7 @@ void ClassDeclaration::toDt2(dt_t **pdt, ClassDeclaration *cd)
 	if (dt)
 	{
 	    if (v->offset < offset)
-		error("2duplicate union initialization for %s", v->toChars());
+		error("duplicated union initialization for %s", v->toChars());
 	    else
 	    {
 		if (offset < v->offset)
