@@ -303,14 +303,19 @@ void ClassDeclaration::toObjFile(int multiobj)
     if (!members)
 	return;
 
+    if (multiobj)
+    {	obj_append(this);
+	return;
+    }
+
     if (global.params.symdebug)
 	toDebug();
 
     assert(!scope);	// semantic() should have been run to completion
 
-	scclass = SCglobal;
-	if (inTemplateInstance())
-	  	    scclass = SCcomdat;
+    scclass = SCglobal;
+    if (inTemplateInstance())
+	scclass = SCcomdat;
 
     // Put out the members
     for (i = 0; i < members->dim; i++)
@@ -993,10 +998,14 @@ void InterfaceDeclaration::toObjFile(int multiobj)
 /* ================================================================== */
 
 void StructDeclaration::toObjFile(int multiobj)
-{   unsigned i;
-
+{
     //printf("StructDeclaration::toObjFile('%s')\n", toChars());
 
+    if (multiobj)
+    {	obj_append(this);
+	return;
+    }
+
     // Anonymous structs/unions only exist as part of others,
     // do not output forward referenced structs's
     if (!isAnonymous() && members)
@@ -1033,7 +1042,8 @@ void StructDeclaration::toObjFile(int multiobj)
 	    if (sinit->Sclass == SCcomdat &&
 		sinit->Sdt &&
 		sinit->Sdt->dt == DT_azeros &&
-		sinit->Sdt->DTnext == NULL)
+		sinit->Sdt->DTnext == NULL &&
+		!global.params.multiobj)
 	    {
 		sinit->Sclass = SCglobal;
 		sinit->Sdt->dt = DT_common;
@@ -1047,7 +1057,7 @@ void StructDeclaration::toObjFile(int multiobj)
 	}
 
 	// Put out the members
-	for (i = 0; i < members->dim; i++)
+	for (unsigned i = 0; i < members->dim; i++)
 	{
 	    Dsymbol *member;
 
@@ -1217,9 +1227,9 @@ void TypedefDeclaration::toObjFile(int multiobj)
 	;
     else
     {
-    enum_SC scclass = SCglobal;
+	enum_SC scclass = SCglobal;
    	if (inTemplateInstance())
-   		scclass = SCcomdat;
+	    scclass = SCcomdat;
 
 	// Generate static initializer
 	toInitializer();
@@ -1255,9 +1265,9 @@ void EnumDeclaration::toObjFile(int multiobj)
 	;
     else
     {
-    enum_SC scclass = SCglobal;
-    if (inTemplateInstance())
-    	scclass = SCcomdat;
+	enum_SC scclass = SCglobal;
+	if (inTemplateInstance())
+	    scclass = SCcomdat;
 
 	// Generate static initializer
 	toInitializer();
