@@ -1,5 +1,8 @@
 /* GDC -- D front-end for GCC
    Copyright (C) 2004 David Friedman
+
+   Modified by
+    Iain Buclaw, (C) 2010
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,7 +20,11 @@
 */
 
 #include "symbol.h"
+#if V1
+#include "rmem.h"
+#else
 #include "mem.h"
+#endif
 
 Symbol::Symbol()
 {
@@ -97,3 +104,12 @@ symbol_tree(tree t)
     s->Stree = t;
     return s;
 }
+
+void slist_add(Symbol * /*s*/)
+{
+}
+
+void slist_reset()
+{
+}
+
